@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2000,2008 Oracle.  All rights reserved.
  *
- * $Id: ByteArrayBinding.java,v 1.24.2.1 2007/02/01 14:49:38 cwl Exp $
+ * $Id: ByteArrayBinding.java,v 1.27 2008/05/27 15:30:32 mark Exp $
  */
 
 package com.sleepycat.bind;
@@ -16,7 +16,7 @@ import com.sleepycat.je.DatabaseEntry;
  *
  * @author Mark Hayes
  */
-public class ByteArrayBinding implements EntryBinding {
+public class ByteArrayBinding implements EntryBinding<byte[]> {
 
     /*
      * We can return the same byte[] for 0 length arrays.
@@ -30,7 +30,7 @@ public class ByteArrayBinding implements EntryBinding {
     }
 
     // javadoc is inherited
-    public Object entryToObject(DatabaseEntry entry) {
+    public byte[] entryToObject(DatabaseEntry entry) {
 
 	int len = entry.getSize();
 	if (len == 0) {
@@ -44,9 +44,8 @@ public class ByteArrayBinding implements EntryBinding {
     }
 
     // javadoc is inherited
-    public void objectToEntry(Object object, DatabaseEntry entry) {
+    public void objectToEntry(byte[] object, DatabaseEntry entry) {
 
-        byte[] bytes = (byte[]) object;
-        entry.setData(bytes, 0, bytes.length);
+        entry.setData(object, 0, object.length);
     }
 }
