@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2000,2008 Oracle.  All rights reserved.
  *
- * $Id: EntityBinding.java,v 1.21.2.1 2007/02/01 14:49:38 cwl Exp $
+ * $Id: EntityBinding.java,v 1.25 2008/05/27 15:30:32 mark Exp $
  */
 
 package com.sleepycat.bind;
@@ -13,9 +13,15 @@ import com.sleepycat.je.DatabaseEntry;
 /**
  * A binding between a key-value entry pair and an entity object.
  *
+ * <p><em>WARNING:</em> Binding instances are typically shared by multiple
+ * threads and binding methods are called without any special synchronization.
+ * Therefore, bindings must be thread safe.  In general no shared state should
+ * be used and any caching of computed values must be done with proper
+ * synchronization.</p>
+ *
  * @author Mark Hayes
  */
-public interface EntityBinding {
+public interface EntityBinding<E> {
 
     /**
      * Converts key and data entry buffers into an entity Object.
@@ -26,7 +32,7 @@ public interface EntityBinding {
      *
      * @return the resulting Object.
      */
-    Object entryToObject(DatabaseEntry key, DatabaseEntry data);
+    E entryToObject(DatabaseEntry key, DatabaseEntry data);
 
     /**
      * Extracts the key entry from an entity Object.
@@ -35,7 +41,7 @@ public interface EntityBinding {
      *
      * @param key is the destination entry buffer.
      */
-    void objectToKey(Object object, DatabaseEntry key);
+    void objectToKey(E object, DatabaseEntry key);
 
     /**
      * Extracts the data entry from an entity Object.
@@ -44,5 +50,5 @@ public interface EntityBinding {
      *
      * @param data is the destination entry buffer.
      */
-    void objectToData(Object object, DatabaseEntry data);
+    void objectToData(E object, DatabaseEntry data);
 }
