@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2000,2008 Oracle.  All rights reserved.
  *
- * $Id: EntryBinding.java,v 1.20.2.1 2007/02/01 14:49:38 cwl Exp $
+ * $Id: EntryBinding.java,v 1.24 2008/05/27 15:30:32 mark Exp $
  */
 
 package com.sleepycat.bind;
@@ -13,9 +13,15 @@ import com.sleepycat.je.DatabaseEntry;
 /**
  * A binding between a key or data entry and a key or data object.
  *
+ * <p><em>WARNING:</em> Binding instances are typically shared by multiple
+ * threads and binding methods are called without any special synchronization.
+ * Therefore, bindings must be thread safe.  In general no shared state should
+ * be used and any caching of computed values must be done with proper
+ * synchronization.</p>
+ *
  * @author Mark Hayes
  */
-public interface EntryBinding {
+public interface EntryBinding<E> {
 
     /**
      * Converts a entry buffer into an Object.
@@ -24,7 +30,7 @@ public interface EntryBinding {
      *
      * @return the resulting Object.
      */
-    Object entryToObject(DatabaseEntry entry);
+    E entryToObject(DatabaseEntry entry);
 
     /**
      * Converts an Object into a entry buffer.
@@ -33,5 +39,5 @@ public interface EntryBinding {
      *
      * @param entry is the destination entry buffer.
      */
-    void objectToEntry(Object object, DatabaseEntry entry);
+    void objectToEntry(E object, DatabaseEntry entry);
 }
