@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2000,2008 Oracle.  All rights reserved.
  *
- * $Id: BaseIterator.java,v 1.3.2.1 2007/02/01 14:49:39 cwl Exp $
+ * $Id: BaseIterator.java,v 1.6 2008/05/27 15:30:34 mark Exp $
  */
 
 package com.sleepycat.collections;
@@ -13,20 +13,23 @@ import java.util.ListIterator;
 /**
  * Common interface for BlockIterator and StoredIterator.
  */
-interface BaseIterator extends ListIterator {
+interface BaseIterator<E> extends ListIterator<E> {
 
     /**
+     * @hidden
      * Duplicate a cursor.  Called by StoredCollections.iterator.
      */
-    ListIterator dup();
+    ListIterator<E> dup();
 
     /**
+     * @hidden
      * Returns whether the given data is the current iterator data.  Called by
      * StoredMapEntry.setValue.
      */
     boolean isCurrentData(Object currentData);
 
     /**
+     * @hidden
      * Initializes a list iterator at the given index.  Called by
      * StoredList.iterator(int).
      */
