@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2000,2008 Oracle.  All rights reserved.
  *
- * $Id: StoredList.java,v 1.47.2.1 2007/02/01 14:49:40 cwl Exp $
+ * $Id: StoredList.java,v 1.53 2008/05/27 15:30:34 mark Exp $
  */
 
 package com.sleepycat.collections;
@@ -23,6 +23,9 @@ import com.sleepycat.je.OperationStatus;
 import com.sleepycat.util.keyrange.KeyRangeException;
 
 /**
+ * <!-- begin JE only -->
+ * @hidden
+ * <!-- end JE only -->
  * A List view of a {@link Database}.
  *
  * <p>For all stored lists the keys of the underlying Database
@@ -40,10 +43,9 @@ import com.sleepycat.util.keyrange.KeyRangeException;
  * <ul>
  * <li>{@link #append(Object)}</li>
  * </ul>
- *
  * @author Mark Hayes
  */
-public class StoredList extends StoredCollection implements List {
+public class StoredList<E> extends StoredCollection<E> implements List<E> {
 
     private static final EntryBinding DEFAULT_KEY_BINDING =
         new IndexKeyBinding(1);
@@ -68,7 +70,8 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public StoredList(Database database, EntryBinding valueBinding,
+    public StoredList(Database database,
+                      EntryBinding<E> valueBinding,
                       boolean writeAllowed) {
 
         super(new DataView(database, DEFAULT_KEY_BINDING, valueBinding, null,
@@ -92,7 +95,8 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public StoredList(Database database, EntityBinding valueEntityBinding,
+    public StoredList(Database database,
+                      EntityBinding<E> valueEntityBinding,
                       boolean writeAllowed) {
 
         super(new DataView(database, DEFAULT_KEY_BINDING, null,
@@ -117,7 +121,8 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public StoredList(Database database, EntryBinding valueBinding,
+    public StoredList(Database database,
+                      EntryBinding<E> valueBinding,
                       PrimaryKeyAssigner keyAssigner) {
 
         super(new DataView(database, DEFAULT_KEY_BINDING, valueBinding,
@@ -142,7 +147,8 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public StoredList(Database database, EntityBinding valueEntityBinding,
+    public StoredList(Database database,
+                      EntityBinding<E> valueEntityBinding,
                       PrimaryKeyAssigner keyAssigner) {
 
         super(new DataView(database, DEFAULT_KEY_BINDING, null,
@@ -168,7 +174,7 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public void add(int index, Object value) {
+    public void add(int index, E value) {
 
         checkIterAddAllowed();
         DataCursor cursor = null;
@@ -176,7 +182,7 @@ public class StoredList extends StoredCollection implements List {
         try {
             cursor = new DataCursor(view, true);
             OperationStatus status =
-                cursor.getSearchKey(new Long(index), null, false);
+                cursor.getSearchKey(Long.valueOf(index), null, false);
             if (status == OperationStatus.SUCCESS) {
                 cursor.putBefore(value);
                 closeCursor(cursor);
@@ -204,7 +210,7 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public boolean add(Object value) {
+    public boolean add(E value) {
 
         checkIterAddAllowed();
         boolean doAutoCommit = beginAutoCommit();
@@ -237,7 +243,7 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public int append(Object value) {
+    public int append(E value) {
 
         boolean doAutoCommit = beginAutoCommit();
         try {
@@ -275,11 +281,11 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public boolean addAll(int index, Collection coll) {
+    public boolean addAll(int index, Collection<? extends E> coll) {
 
         checkIterAddAllowed();
         DataCursor cursor = null;
-	Iterator i = null;
+	Iterator<? extends E> i = null;
         boolean doAutoCommit = beginAutoCommit();
         try {
             i = storedOrExternalIterator(coll);
@@ -288,7 +294,7 @@ public class StoredList extends StoredCollection implements List {
             }
             cursor = new DataCursor(view, true);
             OperationStatus status =
-                cursor.getSearchKey(new Long(index), null, false);
+                cursor.getSearchKey(Long.valueOf(index), null, false);
             if (status == OperationStatus.SUCCESS) {
                 while (i.hasNext()) {
                     cursor.putBefore(i.next());
@@ -330,9 +336,9 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public Object get(int index) {
+    public E get(int index) {
 
-        return super.get(new Long(index));
+        return (E) getValue(Long.valueOf(index));
     }
 
     /**
@@ -398,7 +404,7 @@ public class StoredList extends StoredCollection implements List {
      *
      * @see #isWriteAllowed
      */
-    public ListIterator listIterator() {
+    public ListIterator<E> listIterator() {
 
         return blockIterator();
     }
@@ -419,7 +425,7 @@ public class StoredList extends StoredCollection implements List {
      *
      * @see #isWriteAllowed
      */
-    public ListIterator listIterator(int index) {
+    public ListIterator<E> listIterator(int index) {
 
         BlockIterator i = blockIterator();
         if (i.moveToIndex(index)) {
@@ -440,12 +446,12 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public Object remove(int index) {
+    public E remove(int index) {
 
         try {
             Object[] oldVal = new Object[1];
-            removeKey(new Long(index), oldVal);
-            return oldVal[0];
+            removeKey(Long.valueOf(index), oldVal);
+            return (E) oldVal[0];
         } catch (IllegalArgumentException e) {
             throw new IndexOutOfBoundsException(e.getMessage());
         }
@@ -482,10 +488,10 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public Object set(int index, Object value) {
+    public E set(int index, E value) {
 
         try {
-            return put(new Long(index), value);
+            return (E) putKeyValue(Long.valueOf(index), value);
         } catch (IllegalArgumentException e) {
             throw new IndexOutOfBoundsException(e.getMessage());
         }
@@ -500,7 +506,7 @@ public class StoredList extends StoredCollection implements List {
      * @throws RuntimeExceptionWrapper if a {@link DatabaseException} is
      * thrown.
      */
-    public List subList(int fromIndex, int toIndex) {
+    public List<E> subList(int fromIndex, int toIndex) {
 
         if (fromIndex < 0 || fromIndex > toIndex) {
             throw new IndexOutOfBoundsException(String.valueOf(fromIndex));
@@ -508,8 +514,8 @@ public class StoredList extends StoredCollection implements List {
         try {
             int newBaseIndex = baseIndex + fromIndex;
             return new StoredList(
-                view.subView(new Long(fromIndex), true,
-                             new Long(toIndex), false,
+                view.subView(Long.valueOf(fromIndex), true,
+                             Long.valueOf(toIndex), false,
                              new IndexKeyBinding(newBaseIndex)),
                 newBaseIndex);
         } catch (KeyRangeException e) {
@@ -580,12 +586,12 @@ public class StoredList extends StoredCollection implements List {
 	return super.hashCode();
     }
 
-    Object makeIteratorData(BaseIterator iterator,
-                            DatabaseEntry keyEntry,
-                            DatabaseEntry priKeyEntry,
-                            DatabaseEntry valueEntry) {
+    E makeIteratorData(BaseIterator iterator,
+                       DatabaseEntry keyEntry,
+                       DatabaseEntry priKeyEntry,
+                       DatabaseEntry valueEntry) {
 
-        return view.makeValue(priKeyEntry, valueEntry);
+        return (E) view.makeValue(priKeyEntry, valueEntry);
     }
 
     boolean hasValues() {
@@ -602,12 +608,12 @@ public class StoredList extends StoredCollection implements List {
             this.baseIndex = baseIndex;
         }
 
-        public Object entryToObject(DatabaseEntry data) {
+        public Long entryToObject(DatabaseEntry data) {
 
-            return new Long(entryToRecordNumber(data) - baseIndex);
+            return Long.valueOf(entryToRecordNumber(data) - baseIndex);
         }
 
-        public void objectToEntry(Object object, DatabaseEntry data) {
+        public void objectToEntry(Long object, DatabaseEntry data) {
 
             recordNumberToEntry(((Number) object).intValue() + baseIndex,
                                 data);
