@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: CheckpointConfig.java,v 1.19 2008/05/29 03:38:23 linda Exp $
+ * $Id: CheckpointConfig.java,v 1.20.2.2 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -121,4 +121,18 @@ public class CheckpointConfig  {
     public boolean getMinimizeRecoveryTime() {
         return minimizeRecoveryTime;
     }
+
+    /**
+     * Returns the values for each configuration attribute.
+     *
+     * @return the values for each configuration attribute.
+     */
+    @Override
+    public String toString() {
+        return "minutes=" + minutes +
+            "\nkBytes=" + kbytes +
+            "\nforce=" + force +
+            "\nminimizeRecoveryTime=" + minimizeRecoveryTime +
+            "\n";
+    }
 }
