@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: Cursor.java,v 1.215 2008/05/29 03:17:28 linda Exp $
+ * $Id: Cursor.java,v 1.216.2.5 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -186,8 +186,8 @@ public class Cursor {
 
         assert locker != null;
 
-        DatabaseImpl dbImpl = dbHandle.getDatabaseImpl();
-        cursorImpl = new CursorImpl(dbImpl, locker, retainNonTxnLocks);
+        DatabaseImpl databaseImpl = dbHandle.getDatabaseImpl();
+        cursorImpl = new CursorImpl(databaseImpl, locker, retainNonTxnLocks);
 
         /* Perform eviction for user cursors. */
         cursorImpl.setAllowEviction(true);
@@ -200,14 +200,14 @@ public class Cursor {
             cursorImpl.getLocker().isSerializableIsolation();
 
         updateOperationsProhibited =
-            (dbImpl.isTransactional() && !locker.isTransactional()) ||
+            (databaseImpl.isTransactional() && !locker.isTransactional()) ||
             !dbHandle.isWritable();
 
-        this.dbImpl = dbImpl;
+        this.dbImpl = databaseImpl;
         this.dbHandle = dbHandle;
         dbHandle.addCursor(this);
         this.config = cursorConfig;
-        this.logger = dbImpl.getDbEnvironment().getLogger();
+        this.logger = databaseImpl.getDbEnvironment().getLogger();
     }
 
     /**
@@ -427,7 +427,7 @@ public class Cursor {
      * <p>If the key already appears in the database and duplicates are
      * supported, the new data value is inserted at the correct sorted
      * location.  If the key already appears in the database and duplicates are
-     * not supported, the existing key/data pair will be replaced.</p>
+     * not supported, the data associated with the key will be replaced.</p>
      *
      * @param key the key {@link com.sleepycat.je.DatabaseEntry
      * DatabaseEntry} operated on.
@@ -637,7 +637,7 @@ public class Cursor {
     }
 
     /**
-     * Moves the cursor to the first key/data pair of the database, and return
+     * Moves the cursor to the first key/data pair of the database, and returns
      * that pair.  If the first key has duplicate values, the first data item
      * in the set of duplicates is returned.
      *
@@ -681,7 +681,7 @@ public class Cursor {
     }
 
     /**
-     * Moves the cursor to the last key/data pair of the database, and return
+     * Moves the cursor to the last key/data pair of the database, and returns
      * that pair.  If the last key has duplicate values, the last data item in
      * the set of duplicates is returned.
      *
@@ -725,9 +725,7 @@ public class Cursor {
     }
 
     /**
-     * Moves the cursor to the next key/data pair and return that pair.  If the
-     * matching key has duplicate values, the first data item in the set of
-     * duplicates is returned.
+     * Moves the cursor to the next key/data pair and returns that pair.
      *
      * <p>If the cursor is not yet initialized, move the cursor to the first
      * key/data pair of the database, and return that pair.  Otherwise, the
@@ -781,7 +779,7 @@ public class Cursor {
     /**
      * If the next key/data pair of the database is a duplicate data record for
      * the current key/data pair, moves the cursor to the next key/data pair of
-     * the database and return that pair.
+     * the database and returns that pair.
      *
      * <p>If this method fails for any reason, the position of the cursor will
      * be unchanged.</p>
@@ -823,9 +821,9 @@ public class Cursor {
     }
 
     /**
-     * Moves the cursor to the next non-duplicate key/data pair and return that
-     * pair.  If the matching key has duplicate values, the first data item in
-     * the set of duplicates is returned.
+     * Moves the cursor to the next non-duplicate key/data pair and returns
+     * that pair.  If the matching key has duplicate values, the first data
+     * item in the set of duplicates is returned.
      *
      * <p>If the cursor is not yet initialized, move the cursor to the first
      * key/data pair of the database, and return that pair.  Otherwise, the
@@ -876,9 +874,7 @@ public class Cursor {
     }
 
     /**
-     * Moves the cursor to the previous key/data pair and return that pair. If
-     * the matching key has duplicate values, the last data item in the set of
-     * duplicates is returned.
+     * Moves the cursor to the previous key/data pair and returns that pair.
      *
      * <p>If the cursor is not yet initialized, move the cursor to the last
      * key/data pair of the database, and return that pair.  Otherwise, the
@@ -932,7 +928,7 @@ public class Cursor {
     /**
      * If the previous key/data pair of the database is a duplicate data record
      * for the current key/data pair, moves the cursor to the previous key/data
-     * pair of the database and return that pair.
+     * pair of the database and returns that pair.
      *
      * <p>If this method fails for any reason, the position of the cursor will
      * be unchanged.</p>
@@ -974,7 +970,7 @@ public class Cursor {
     }
 
     /**
-     * Moves the cursor to the previous non-duplicate key/data pair and return
+     * Moves the cursor to the previous non-duplicate key/data pair and returns
      * that pair.  If the matching key has duplicate values, the last data item
      * in the set of duplicates is returned.
      *
@@ -1027,7 +1023,7 @@ public class Cursor {
     }
 
     /**
-     * Moves the cursor to the given key of the database, and return the datum
+     * Moves the cursor to the given key of the database, and returns the datum
      * associated with the given key.  If the matching key has duplicate
      * values, the first data item in the set of duplicates is returned.
      *
@@ -1072,10 +1068,10 @@ public class Cursor {
     }
 
     /**
-     * Moves the cursor to the closest matching key of the database, and return
-     * the data item associated with the matching key.  If the matching key has
-     * duplicate values, the first data item in the set of duplicates is
-     * returned.
+     * Moves the cursor to the closest matching key of the database, and
+     * returns the data item associated with the matching key.  If the matching
+     * key has duplicate values, the first data item in the set of duplicates
+     * is returned.
      *
      * <p>The returned key/data pair is for the smallest key greater than or
      * equal to the specified key (as determined by the key comparison
@@ -1698,7 +1694,6 @@ public class Cursor {
             SearchMode.SET_RANGE : SearchMode.BOTH_RANGE;
 
         KeyChangeStatus result = null;
-        boolean noNextKeyFound;
 
         CursorImpl dup =
             beginRead(false /* searchAndPosition will add cursor */);
@@ -1714,9 +1709,6 @@ public class Cursor {
                 (dup, key, data, searchLockType, advanceLockType, searchMode,
                  true /*advanceAfterRangeSearch*/);
 
-            /* The keyChange value is independent of the status value. */
-            noNextKeyFound = !result.keyChange;
-
             /* If the key changed, then we do not have an exact match. */
             if (result.keyChange && result.status == OperationStatus.SUCCESS) {
                 result.status = OperationStatus.NOTFOUND;
@@ -1726,8 +1718,11 @@ public class Cursor {
                          result.status == OperationStatus.SUCCESS);
         }
 
-        /* Lock the EOF node if no more records, whether or not more dups. */
-        if (noNextKeyFound) {
+        /*
+         * Lock the EOF node if there was no exact match and we did not
+         * range-lock the next record.
+         */
+        if (result.status != OperationStatus.SUCCESS && !result.keyChange) {
             cursorImpl.lockEofNode(LockType.RANGE_READ);
         }
 
@@ -1928,7 +1923,7 @@ public class Cursor {
 
                             /*
                              * If we did not match the key (exactly) for
-                             * BOTH_RANGE, and advanceAfterSearchRangeBoth is
+                             * BOTH_RANGE, and advanceAfterRangeSearch is
                              * false, then return NOTFOUND.
                              */
                              status = OperationStatus.NOTFOUND;
