@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: CursorConfig.java,v 1.24 2008/04/07 22:28:29 mark Exp $
+ * $Id: CursorConfig.java,v 1.25.2.4 2010/03/22 21:53:33 mark Exp $
  */
 
 package com.sleepycat.je;
@@ -115,11 +115,14 @@ public class CursorConfig implements Cloneable {
     }
 
     /**
-     * Configures read operations performed by the cursor to return modified
-     * but not yet committed data.
+     * Configures read operations performed by the cursor to obey read
+     * committed isolation. Read committed isolation provides for cursor
+     * stability but not repeatable reads. Data items which have been
+     * previously read by this transaction may be deleted or modified by other
+     * transactions before the cursor is closed or the transaction completes.
      *
      * @param readCommitted If true, configure read operations performed by
-     * the cursor to return modified but not yet committed data.
+     * the cursor to obey read commited isolation.
      *
      * @see LockMode#READ_COMMITTED
      */
@@ -129,10 +132,10 @@ public class CursorConfig implements Cloneable {
 
     /**
      * Returns true if read operations performed by the cursor are configured
-     * to return modified but not yet committed data.
+     * to obey read committed isolation.
      *
      * @return true if read operations performed by the cursor are configured
-     * to return modified but not yet committed data.
+     * to obey read committed isolation.w
      *
      * @see LockMode#READ_COMMITTED
      */
@@ -151,4 +154,16 @@ public class CursorConfig implements Cloneable {
             return null;
         }
     }
+
+    /**
+     * Returns the values for each configuration attribute.
+     *
+     * @return the values for each configuration attribute.
+     */
+    @Override
+    public String toString() {
+        return "readUncommitted=" + readUncommitted +
+            "\nreadCommitted=" + readCommitted +
+            "\n";
+    }
 }
