@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: Database.java,v 1.241 2008/05/30 14:04:15 mark Exp $
+ * $Id: Database.java,v 1.242.2.3 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -61,6 +61,7 @@ public class Database {
             this.stateName = stateName;
         }
 
+        @Override
         public String toString() {
             return "DbState." + stateName;
         }
@@ -844,9 +845,10 @@ public class Database {
      * Stores the key/data pair into the database.
      *
      * <p>If the key already appears in the database and duplicates are not
-     * configured, the existing key/data pair will be replaced.  If the key
-     * already appears in the database and sorted duplicates are configured,
-     * the new data value is inserted at the correct sorted location.</p>
+     * configured, the data associated with the key will be replaced.  If the
+     * key already appears in the database and sorted duplicates are
+     * configured, the new data value is inserted at the correct sorted
+     * location.</p>
      *
      * @param txn For a transactional database, an explicit transaction may be
      * specified, or null may be specified to use auto-commit.  For a
