@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: DatabaseEntry.java,v 1.47 2008/01/24 14:59:27 linda Exp $
+ * $Id: DatabaseEntry.java,v 1.48.2.3 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -104,7 +104,7 @@ public class DatabaseEntry {
 
     /* FindBugs - ignore not "final" since a user can set this. */
     /** @hidden
-     * The maximum number of bytes to show when toString() is called. 
+     * The maximum number of bytes to show when toString() is called.
      */
     public static int MAX_DUMP_BYTES = 100;
 
@@ -115,12 +115,16 @@ public class DatabaseEntry {
      * defaults to 100.  MAX_DUMP_BYTES may be changed by an application if it
      * wishes to cause more bytes to be formatted.
      */
+    @Override
     public String toString() {
 	StringBuffer sb = new StringBuffer("<DatabaseEntry");
-	sb.append(" dlen=\"").append(dlen);
-	sb.append("\" doff=\"").append(doff);
-	sb.append("\" doff=\"").append(doff);
-	sb.append("\" offset=\"").append(offset);
+        if (partial) {
+            sb.append(" partial=\"true");
+            sb.append("\" doff=\"").append(doff);
+            sb.append("\" dlen=\"").append(dlen);
+            sb.append("\"");
+        }
+	sb.append(" offset=\"").append(offset);
 	sb.append("\" size=\"").append(size);
 	sb.append("\" data=\"").append(dumpData());
 	if ((size - 1) > MAX_DUMP_BYTES) {
@@ -426,6 +430,7 @@ public class DatabaseEntry {
      * equal both entries must have the same partial properties: partial,
      * partialOffset and partialLength.
      */
+    @Override
     public boolean equals(Object o) {
         if (!(o instanceof DatabaseEntry)) {
             return false;
@@ -458,6 +463,7 @@ public class DatabaseEntry {
     /**
      * Returns a hash code based on the data value.
      */
+    @Override
     public int hashCode() {
         int hash = 0;
         if (data != null) {
