@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: DbInternal.java,v 1.56 2008/03/18 15:53:04 mark Exp $
+ * $Id: DbInternal.java,v 1.57.2.2 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -16,6 +16,7 @@ import com.sleepycat.je.dbi.DatabaseImpl;
 import com.sleepycat.je.dbi.EnvironmentImpl;
 import com.sleepycat.je.dbi.GetMode;
 import com.sleepycat.je.txn.Locker;
+import com.sleepycat.je.txn.Txn;
 
 /**
  * @hidden
@@ -311,4 +312,8 @@ public class DbInternal {
 
         return new Environment(envHome, config, replicationIntended);
     }
+
+    public static Txn getTxn(Transaction transaction) {
+        return transaction.getTxn();
+    }
 }
