@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: Environment.java,v 1.217 2008/05/30 19:07:40 mark Exp $
+ * $Id: Environment.java,v 1.217.2.4 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -139,9 +139,13 @@ public class Environment {
     * @throws IllegalArgumentException if an invalid parameter was specified.
     *
     * @throws DatabaseException if a failure occurs.
+    *
+    * @throws EnvironmentLockedException when an environment cannot be opened
+    * for write access because another process has the same environment open
+    * for write access.
      */
     public Environment(File envHome, EnvironmentConfig configuration)
-        throws DatabaseException {
+        throws DatabaseException, EnvironmentLockedException {
 
         this(envHome, configuration, true /*openIfNeeded*/,
              false /*replicationIntended*/);
@@ -1551,6 +1555,8 @@ public class Environment {
     }
 
     /**
+     * For internal use only.
+     * @hidden
      * Scans raw log entries in the JE log between two given points, passing
      * all records for a given set of databases to the scanRecord method of the
      * given LogScanner object.
