@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: EnvironmentConfig.java,v 1.50 2008/06/02 16:31:43 linda Exp $
+ * $Id: EnvironmentConfig.java,v 1.52.2.4 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -710,6 +710,8 @@ public class EnvironmentConfig extends EnvironmentMutableConfig {
      * <td>false</td>
      * </tr>
      * </table></p>
+     * @deprecated NIO is no longer used by JE and this parameter has no
+     * effect.
      */
     public static final String LOG_USE_NIO = "je.log.useNIO";
 
@@ -726,6 +728,8 @@ public class EnvironmentConfig extends EnvironmentMutableConfig {
      * <td>false</td>
      * </tr>
      * </table></p>
+     * @deprecated NIO is no longer used by JE and this parameter has no
+     * effect.
      */
     public static final String LOG_DIRECT_NIO = "je.log.directNIO";
 
@@ -745,6 +749,8 @@ public class EnvironmentConfig extends EnvironmentMutableConfig {
      * <td>67108864 (64M)</td>
      * </tr>
      * </table></p>
+     * @deprecated NIO is no longer used by JE and this parameter has no
+     * effect.
      */
     public static final String LOG_CHUNKED_NIO = "je.log.chunkedNIO";
 
@@ -907,8 +913,8 @@ public class EnvironmentConfig extends EnvironmentMutableConfig {
         "je.compressor.lockTimeout";
 
     /**
-     * If true, when the compressor encounters an empty tree, the root node of
-     * the tree is deleted.
+     * If true, when the compressor encounters an empty database, the root node
+     * of the Btree is deleted.
      *
      * <p><table border"1">
      * <tr><td>Name</td><td>Type</td><td>Mutable</td><td>Default</td></tr>
@@ -919,6 +925,10 @@ public class EnvironmentConfig extends EnvironmentMutableConfig {
      * <td>false</td>
      * </tr>
      * </table></p>
+     *
+     * @deprecated as of 3.3.87.  Compression of the root node no longer has
+     * any benefit and this feature has been removed.  This parameter has no
+     * effect.
      */
     public static final String COMPRESSOR_PURGE_ROOT =
         "je.compressor.purgeRoot";
@@ -1762,7 +1772,7 @@ public class EnvironmentConfig extends EnvironmentMutableConfig {
      * @param timeout The lock timeout, in microseconds. A value of 0 turns off
      * lock timeouts.
      *
-     * @throws IllegalArgumentException If the value of timeout is negative 
+     * @throws IllegalArgumentException If the value of timeout is negative
      * @see Transaction#setLockTimeout
      */
     public void setLockTimeout(long timeout)
@@ -1901,7 +1911,7 @@ public class EnvironmentConfig extends EnvironmentMutableConfig {
      * @param timeout The transaction timeout, in microseconds. A value of 0
      * turns off transaction timeouts.
      *
-     * @throws IllegalArgumentException If the value of timeout is negative 
+     * @throws IllegalArgumentException If the value of timeout is negative
      *
      * @see Transaction#setTxnTimeout
      */
@@ -2026,6 +2036,7 @@ public class EnvironmentConfig extends EnvironmentMutableConfig {
     }
 
     /* Documentation inherited from EnvironmentMutableConfig.setConfigParam. */
+    @Override
     public void setConfigParam(String paramName,
 			       String value)
         throws IllegalArgumentException {
@@ -2082,6 +2093,7 @@ public class EnvironmentConfig extends EnvironmentMutableConfig {
     /**
      * Display configuration values.
      */
+    @Override
     public String toString() {
         return ("allowCreate=" + allowCreate + "\n" + super.toString());
     }
