@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: EnvironmentMutableConfig.java,v 1.41 2008/05/19 17:52:16 linda Exp $
+ * $Id: EnvironmentMutableConfig.java,v 1.44.2.3 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -295,9 +295,6 @@ public class EnvironmentMutableConfig implements Cloneable {
      * calling setCacheSize() with a non-zero value overrides the percentage
      * based calculation and sets the cache size explicitly.</p>
      *
-     * <p>Note that the cache does not include transient objects created by the
-     * JE library, such as cursors, locks and transactions.</p>
-     *
      * <p>Note that the log buffer cache may be cleared if the cache size is
      * changed after the environment has been opened.</p>
      *
@@ -497,6 +494,7 @@ public class EnvironmentMutableConfig implements Cloneable {
      * Overrides Object.clone() to clone all properties, used by this class and
      * EnvironmentConfig.
      */
+    @Override
     protected Object clone()
         throws CloneNotSupportedException {
 
@@ -534,8 +532,11 @@ public class EnvironmentMutableConfig implements Cloneable {
 
     /**
      * Copies all mutable props to the given config object.
+     * Unchecked suppress here because Properties don't play well with 
+     * generics in Java 1.5 
      */
-    void copyMutablePropsTo(EnvironmentMutableConfig toConfig) {
+    @SuppressWarnings("unchecked") 
+	void copyMutablePropsTo(EnvironmentMutableConfig toConfig) {
 
         Properties toProps = toConfig.props;
         Enumeration propNames = props.propertyNames();
@@ -560,9 +561,12 @@ public class EnvironmentMutableConfig implements Cloneable {
         cacheSize = envImpl.getMemoryBudget().getMaxMemory();
     }
 
-    /**
-     * Removes all immutable props.
-     */
+   /**
+    * Removes all immutable props.
+    * Unchecked suppress here because Properties don't play well with 
+    * generics in Java 1.5
+    */ 
+    @SuppressWarnings("unchecked")
     private void clearImmutableProps() {
         Enumeration propNames = props.propertyNames();
         while (propNames.hasMoreElements()) {
@@ -604,10 +608,11 @@ public class EnvironmentMutableConfig implements Cloneable {
     /**
      * Display configuration values.
      */
+    @Override
     public String toString() {
         return ("cacheSize=" + cacheSize + "\n" +
                 "txnNoSync=" + txnNoSync + "\n" +
                 "txnWriteNoSync=" + txnWriteNoSync + "\n" +
-                props.toString());
+                props.toString() + "\n");
     }
 }
