@@ -1,14 +1,16 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: LogScanConfig.java,v 1.6 2008/05/13 17:41:43 cwl Exp $
+ * $Id: LogScanConfig.java,v 1.7.2.3 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
 
 /**
+ * For internal use only.
+ * @hidden
  * Specify the attributes of a log scan.
  */
 public class LogScanConfig {
@@ -39,4 +41,15 @@ public class LogScanConfig {
     public boolean getForwards() {
         return forwards;
     }
+
+    /**
+     * Returns the values for each configuration attribute.
+     *
+     * @return the values for each configuration attribute.
+     */
+    @Override
+    public String toString() {
+        return "forwards=" + forwards +
+            "\n";
+    }
 }
