@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: PreloadConfig.java,v 1.8 2008/01/24 14:59:27 linda Exp $
+ * $Id: PreloadConfig.java,v 1.9.2.2 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -104,4 +104,17 @@ public class PreloadConfig implements Cloneable {
             return null;
         }
     }
+
+    /**
+     * Returns the values for each configuration attribute.
+     *
+     * @return the values for each configuration attribute.
+     */
+    @Override
+    public String toString() {
+        return "maxBytes=" + maxBytes +
+            "\nmaxMillisecs=" + maxMillisecs +
+            "\nloadLNs=" + loadLNs +
+            "\n";
+    }
 }
