@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: SecondaryConfig.java,v 1.23 2008/02/08 03:22:38 mark Exp $
+ * $Id: SecondaryConfig.java,v 1.25.2.2 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -374,6 +374,7 @@ public class SecondaryConfig extends DatabaseConfig {
     /*
      * For JCA Database handle caching.
      */
+    @Override
     void validate(DatabaseConfig configArg)
 	throws DatabaseException {
 
@@ -464,4 +465,22 @@ public class SecondaryConfig extends DatabaseConfig {
 
 	return ret.toString();
     }
+
+    /**
+     * Returns the values for each configuration attribute.
+     *
+     * @return the values for each configuration attribute.
+     */
+    @Override
+    public String toString() {
+        return "keyCreator=" + keyCreator +
+            "\nmultiKeyCreator=" + multiKeyCreator +
+            "\nallowPopulate=" + allowPopulate +
+            "\nforeignKeyDatabase=" + foreignKeyDatabase +
+            "\nforeignKeyDeleteAction=" + foreignKeyDeleteAction +
+            "\nforeignKeyNullifier=" + foreignKeyNullifier +
+            "\nforeignMultiKeyNullifier=" + foreignMultiKeyNullifier +
+            "\nimmutableSecondaryKey=" + immutableSecondaryKey +
+            "\n";
+    }
 }
