@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: StatsConfig.java,v 1.18 2008/01/24 14:59:27 linda Exp $
+ * $Id: StatsConfig.java,v 1.19.2.2 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -113,4 +113,18 @@ public class StatsConfig {
     public int getShowProgressInterval() {
         return showProgressInterval;
     }
+
+    /**
+     * Returns the values for each configuration attribute.
+     *
+     * @return the values for each configuration attribute.
+     */
+    @Override
+    public String toString() {
+        return "fast=" + fast +
+            "\nclear=" + clear +
+            "\nshowProgressStream=" + showProgressStream +
+            "\nshowProgressInterval=" + showProgressInterval +
+            "\n";
+    }
 }
