@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: TransactionConfig.java,v 1.23 2008/05/28 14:46:42 linda Exp $
+ * $Id: TransactionConfig.java,v 1.24.2.2 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -33,8 +33,6 @@ public class TransactionConfig implements Cloneable {
     private boolean readCommitted = false;
     private boolean serializableIsolation = false;
 
-
-
     /* Convenience constants for local (non-replicated) use. */
 
     /**
@@ -454,4 +452,23 @@ public class TransactionConfig implements Cloneable {
              " supported");
         }
     }
+
+    /**
+     * Returns the values for each configuration attribute.
+     *
+     * @return the values for each configuration attribute.
+     */
+    @Override
+    public String toString() {
+        return "sync=" + sync +
+            "\nnoSync=" + noSync +
+            "\nwriteNoSync=" + writeNoSync +
+            "\ndurability=" + durability +
+            "\nconsistencyPolicy=" + consistencyPolicy +
+            "\nnoWait=" + noWait +
+            "\nreadUncommitted=" + readUncommitted +
+            "\nreadCommitted=" + readCommitted +
+            "\nSerializableIsolation=" + serializableIsolation +
+            "\n";
+    }
 }
