@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: Cleaner.java,v 1.213 2008/05/15 01:52:40 linda Exp $
+ * $Id: Cleaner.java,v 1.213.2.5 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je.cleaner;
@@ -22,6 +22,7 @@ import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.EnvironmentMutableConfig;
 import com.sleepycat.je.EnvironmentStats;
 import com.sleepycat.je.ExceptionListener;
+import com.sleepycat.je.RunRecoveryException;
 import com.sleepycat.je.StatsConfig;
 import com.sleepycat.je.cleaner.FileSelector.CheckpointStartCleanerState;
 import com.sleepycat.je.config.EnvironmentParams;
@@ -100,7 +101,6 @@ public class Cleaner implements DaemonRunner, EnvConfigObserver {
      * This could produce errors in counting, but avoids contention around stat
      * updates.
      */
-    int  nBacklogFiles = 0;
     long nCleanerRuns = 0;
     long nCleanerDeletions = 0;
     long nINsObsolete = 0;
@@ -141,11 +141,10 @@ public class Cleaner implements DaemonRunner, EnvConfigObserver {
     boolean lazyMigration;
 
     /**
-     * All files that are to-be-cleaning or being-cleaned.  Used to perform
-     * proactive migration.  Is read-only after assignment, so no
-     * synchronization is needed.
+     * All files that are to-be-cleaned.  Used to perform proactive migration.
+     * Is read-only after assignment, so no synchronization is needed.
      */
-    private Set<Long> mustBeCleanedFiles = Collections.emptySet();
+    private Set<Long> toBeCleanedFiles = Collections.emptySet();
 
     /**
      * All files that are below the minUtilization threshold.  Used to perform
@@ -291,6 +290,11 @@ public class Cleaner implements DaemonRunner, EnvConfigObserver {
         return profile;
     }
 
+    /** For unit testing. */
+    FileSelector getFileSelector() {
+        return fileSelector;
+    }
+
     public boolean getFetchObsoleteSize() {
         return fetchObsoleteSize;
     }
@@ -393,7 +397,7 @@ public class Cleaner implements DaemonRunner, EnvConfigObserver {
     public void loadStats(StatsConfig config, EnvironmentStats stat)
         throws DatabaseException {
 
-        stat.setCleanerBacklog(nBacklogFiles);
+        stat.setCleanerBacklog(getBacklog());
         stat.setNCleanerRuns(nCleanerRuns);
         stat.setNCleanerDeletions(nCleanerDeletions);
         stat.setNINsObsolete(nINsObsolete);
@@ -442,13 +446,6 @@ public class Cleaner implements DaemonRunner, EnvConfigObserver {
         }
     }
 
-    /**
-     * For unit testing.
-     */
-    void injectFileForCleaning(Long fileNum) {
-        fileSelector.putBackFileForCleaning(fileNum);
-    }
-
     /**
      * Deletes all files that are safe-to-delete, if there are no read/only
      * processes and concurrent backups.
@@ -594,13 +591,17 @@ public class Cleaner implements DaemonRunner, EnvConfigObserver {
         }
     }
 
-    private void traceFileNotDeleted(Exception e, long fileNum) {
-        Tracer.trace
-            (env, "Cleaner", "deleteSafeToDeleteFiles",
+    private void traceFileNotDeleted(Throwable e, long fileNum)
+        throws DatabaseException {
+
+        String msg = "Cleaner deleteSafeToDeleteFiles" +
              "Log file 0x" + Long.toHexString(fileNum) + " could not be " +
              (expunge ? "deleted" : "renamed") +
-             ".  This operation will be retried at the next checkpoint.",
-             e);
+             ".  This operation will be retried at the next checkpoint." +
+             e.toString() +
+             " FileSelector: " + fileSelector;
+        Tracer.trace
+            (env, "Cleaner", "deleteSafeToDeleteFiles", msg, e);
     }
 
     /**
@@ -632,13 +633,16 @@ public class Cleaner implements DaemonRunner, EnvConfigObserver {
     }
 
     /**
-     * Update the lowUtilizationFiles and mustBeCleanedFiles fields with new
-     * read-only collections, and update the backlog file count.
+     * Update the lowUtilizationFiles and toBeCleanedFiles fields with new
+     * read-only collections.
      */
     public void updateReadOnlyFileCollections() {
-        mustBeCleanedFiles = fileSelector.getMustBeCleanedFiles();
+        toBeCleanedFiles = fileSelector.getToBeCleanedFiles();
         lowUtilizationFiles = fileSelector.getLowUtilizationFiles();
-        nBacklogFiles = fileSelector.getBacklog();
+    }
+
+    final int getBacklog() {
+        return toBeCleanedFiles.size();
     }
 
     /**
@@ -861,7 +865,7 @@ public class Cleaner implements DaemonRunner, EnvConfigObserver {
             Long fileNum = Long.valueOf(DbLsn.getFileNumber(lsn));
 
             if ((PROACTIVE_MIGRATION || isResident) &&
-                mustBeCleanedFiles.contains(fileNum)) {
+                toBeCleanedFiles.contains(fileNum)) {
                 return false;
             }
 
@@ -1065,7 +1069,7 @@ public class Cleaner implements DaemonRunner, EnvConfigObserver {
             Long fileNum = Long.valueOf(DbLsn.getFileNumber(childLsn));
 
             if ((PROACTIVE_MIGRATION || isResident) &&
-                mustBeCleanedFiles.contains(fileNum)) {
+                toBeCleanedFiles.contains(fileNum)) {
 
                 /* Migrate because it will be cleaned soon. */
                 doMigration = true;
@@ -1533,5 +1537,6 @@ public class Cleaner implements DaemonRunner, EnvConfigObserver {
     public void close() {
         profile.close();
         tracker.close();
+        fileSelector.close(env.getMemoryBudget());
     }
 }
