@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: FileProcessor.java,v 1.42 2008/05/15 01:52:40 linda Exp $
+ * $Id: FileProcessor.java,v 1.44.2.5 2010/01/30 01:10:55 mark Exp $
  */
 
 package com.sleepycat.je.cleaner;
@@ -121,6 +121,7 @@ class FileProcessor extends DaemonThread {
     /**
      * Return the number of retries when a deadlock exception occurs.
      */
+    @Override
     protected long nDeadlockRetries()
         throws DatabaseException {
 
@@ -221,7 +222,7 @@ class FileProcessor extends DaemonThread {
                 String traceMsg =
                     "CleanerRun " + runId +
                     " on file 0x" + Long.toHexString(fileNumValue) +
-                    " begins backlog=" + cleaner.nBacklogFiles;
+                    " begins backlog=" + cleaner.getBacklog();
                 Tracer.trace(Level.INFO, env, traceMsg);
                 if (DEBUG_TRACING) {
                     System.out.println("\n" + traceMsg);
@@ -241,14 +242,14 @@ class FileProcessor extends DaemonThread {
                 /*
                  * File was deleted.  Although it is possible that the file was
                  * deleted externally it is much more likely that the file was
-                 * deleted normally after being cleaned earlier (this was
-                 * observed prior to JE 3.2.29), and that we are mistakedly
-                 * processing the file repeatedly.  Since the file does not
-                 * exist, ignore the error so that the cleaner will continue.
-                 * The tracing below will indicate that the file was deleted.
-                 * Remove the file completely from the FileSelector and
-                 * UtilizationProfile so that we don't repeatedly attempt to
-                 * process it. [#15528]
+                 * deleted normally after being cleaned earlier.  This can
+                 * occur when tracked obsolete information is collected and
+                 * processed after the file has been cleaned and deleted.
+                 * Since the file does not exist, ignore the error so that the
+                 * cleaner will continue.  The tracing below will indicate that
+                 * the file was deleted.  Remove the file completely from the
+                 * FileSelector and UtilizationProfile so that we don't
+                 * repeatedly attempt to process it. [#15528]
                  */
                 fileDeleted = true;
                 profile.removeFile(fileNum, null /*databases*/);
@@ -368,7 +369,8 @@ class FileProcessor extends DaemonThread {
          * releaseDb after getDb with the dbCache, since the entire dbCache
          * will be released at the end of thie method.
          */
-        Map<DatabaseId, DatabaseImpl> dbCache = new HashMap<DatabaseId, DatabaseImpl>();
+        Map<DatabaseId, DatabaseImpl> dbCache =
+            new HashMap<DatabaseId, DatabaseImpl>();
         DbTree dbMapTree = env.getDbTree();
 
         try {
@@ -555,6 +557,27 @@ class FileProcessor extends DaemonThread {
         return true;
     }
 
+    /**
+     * Unit testing.  Simulates processing of a single LN.
+     */
+    void testProcessLN(LN targetLN,
+                       long logLsn,
+                       byte[] key,
+                       byte[] dupKey,
+                       DatabaseId dbId,
+                       Map<DatabaseId, DatabaseImpl> dbCache)
+        throws DatabaseException {
+
+        LookAheadCache lookAheadCache = new LookAheadCache(1);
+
+        lookAheadCache.add
+            (Long.valueOf(DbLsn.getFileOffset(logLsn)),
+             new LNInfo(targetLN, dbId, key, dupKey));
+
+        processLN(DbLsn.getFileNumber(logLsn), new TreeLocation(),
+                  lookAheadCache, dbCache);
+    }
+
     /**
      * Processes the first LN in the look ahead cache and removes it from the
      * cache.  While the BIN is latched, look through the BIN for other LNs in
@@ -894,6 +917,11 @@ class FileProcessor extends DaemonThread {
                  * the tree and log LSNs are equal.  We will always restore the
                  * LN to the BIN slot here, and always log the dirty LN when
                  * logging the BIN.
+                 *
+                 * Also for temporary databases, make both the target LN and
+                 * the BIN or IN parent dirty. Otherwise, when the BIN or IN is
+                 * evicted in the future, it will be written to disk without
+                 * flushing its dirty, migrated LNs.  [#18227]
                  */
                 if (isDupCountLN) {
                     ChildReference dclRef = parentDIN.getDupCountLNRef();
@@ -905,6 +933,7 @@ class FileProcessor extends DaemonThread {
                     if (isTemporary) {
                         ((LN) dclRef.getTarget()).setDirty();
                         dclRef.setLsn(DbLsn.NULL_LSN);
+                        parentDIN.setDirty(true);
                     } else if (cleaner.lazyMigration) {
                         dclRef.setMigrate(true);
                         parentDIN.setDirty(true);
@@ -934,6 +963,7 @@ class FileProcessor extends DaemonThread {
                     if (isTemporary) {
                         ((LN) bin.getTarget(index)).setDirty();
                         bin.clearLsn(index);
+                        bin.setDirty(true);
                     } else if (cleaner.lazyMigration) {
                         bin.setMigrate(index, true);
                         bin.setDirty(true);
