@@ -1,13 +1,14 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: FileSelector.java,v 1.24 2008/05/15 01:52:40 linda Exp $
+ * $Id: FileSelector.java,v 1.24.2.10 2010/01/04 15:30:27 cwl Exp $
  */
 
 package com.sleepycat.je.cleaner;
 
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -20,6 +21,8 @@ import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.dbi.DatabaseId;
 import com.sleepycat.je.dbi.MemoryBudget;
 import com.sleepycat.je.tree.LN;
+import com.sleepycat.je.utilint.TestHook;
+import com.sleepycat.je.utilint.TestHookExecute;
 
 /**
  * Keeps track of the status of files for which cleaning is in progres.
@@ -131,6 +134,9 @@ public class FileSelector {
      */
     private Set<Long> lowUtilizationFiles;
 
+    /* For unit tests */
+    private TestHook fileChosenHook;
+
     FileSelector() {
         toBeCleanedFiles = new TreeSet<Long>();
         cleanedFiles = new HashSet<Long>();
@@ -144,10 +150,14 @@ public class FileSelector {
         beingCleanedFiles = new HashSet<Long>();
     }
 
+    /* For unit testing only. */
+    public void setFileChosenHook(TestHook hook) {
+        fileChosenHook = hook;
+    }
+
     /**
      * Returns the best file that qualifies for cleaning, or null if no file
-     * qualifies.  This method is not thread safe and should only be called
-     * from the cleaner thread.
+     * qualifies.
      *
      * @param forceCleaning is true to always select a file, even if its
      * utilization is above the minimum utilization threshold.
@@ -166,6 +176,15 @@ public class FileSelector {
                                int maxBatchFiles)
         throws DatabaseException {
 
+        /*
+         * Note that because it calls UtilizationProfile methods, this method
+         * itself cannot be synchronized.  Synchronization is done elsewhere in
+         * the order [UtilizationProfile, FileSelector], so this method can't
+         * synchronize first on FileSelector and then call a UtilizationProfile
+         * method without causing a deadlock.  However, it must synchronize
+         * while accessing private fields.  Retries are performed when
+         * necessary to work around the lack of synchronization.  See below.
+         */
         Set<Long> newLowUtilizationFiles = calcLowUtilizationFiles ?
             (new HashSet<Long>()) : null;
 
@@ -192,8 +211,23 @@ public class FileSelector {
                 break;
             }
 
+            assert TestHookExecute.doHookIfSet(fileChosenHook);
+
+            /*
+             * Because we don't synchronize on the FileSelector while calling
+             * UtilizationProfile.getBestFileForCleaning, another thread may
+             * have selected this file.  Add the file only if not already being
+             * cleaned.  [#17079]
+             *
+             * There is also a possibility that another thread will have
+             * already cleaned and deleted the file.  In that case, we will
+             * return it and attempt to clean it again, but
+             * FileProcessor.doClean will detect this situation and recover.
+             */
             synchronized (this) {
-                toBeCleanedFiles.add(fileNum);
+                if (!isFileCleaningInProgress(fileNum)) {
+                    toBeCleanedFiles.add(fileNum);
+                }
             }
         }
 
@@ -204,21 +238,32 @@ public class FileSelector {
 
         /*
          * Select the cheapest file to clean from a copy of the to-be-cleaned
-         * set.  Then move the file from the to-be-cleaned set to the
+         * set.  Move the selected file from the to-be-cleaned set to the
          * being-cleaned set.
+         *
+         * Because we don't synchronize on the FileSelector while calling
+         * UtilizationProfile.getCheapestFileToClean, another thread may have
+         * selected this file.  Select the file only if it is waiting to be
+         * cleaned.  [#17079]
          */
         SortedSet<Long> availableFiles;
         synchronized (this) {
             availableFiles = new TreeSet<Long>(toBeCleanedFiles);
         }
-        Long file = profile.getCheapestFileToClean(availableFiles);
-        if (file != null) {
+        while (availableFiles.size() > 0) {
+            Long fileNum = profile.getCheapestFileToClean(availableFiles);
+            if (fileNum == null) {
+                return null;
+            }
             synchronized (this) {
-                toBeCleanedFiles.remove(file);
-                beingCleanedFiles.add(file);
+                if (toBeCleanedFiles.remove(fileNum)) {
+                    beingCleanedFiles.add(fileNum);
+                    return fileNum;
+                }
+                availableFiles.remove(fileNum);
             }
         }
-        return file;
+        return null;
     }
 
     /**
@@ -233,6 +278,15 @@ public class FileSelector {
             safeToDeleteFiles.contains(file);
     }
 
+    private boolean isFileCleaningInProgress(Collection<Long> files) {
+        for (Long file : files) {
+            if (isFileCleaningInProgress(file)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     /**
      * Removes all references to a file.
      */
@@ -252,8 +306,18 @@ public class FileSelector {
      * set to the to-be-cleaned set.
      */
     synchronized void putBackFileForCleaning(Long fileNum) {
-        toBeCleanedFiles.add(fileNum);
         beingCleanedFiles.remove(fileNum);
+        assert !isFileCleaningInProgress(fileNum);
+        toBeCleanedFiles.add(fileNum);
+    }
+
+    /**
+     * For unit testing.
+     */
+    synchronized void injectFileForCleaning(Long fileNum) {
+        if (!isFileCleaningInProgress(fileNum)) {
+            toBeCleanedFiles.add(fileNum);
+        }
     }
 
     /**
@@ -263,11 +327,12 @@ public class FileSelector {
     synchronized void addCleanedFile(Long fileNum,
                                      Set<DatabaseId> databases,
                                      MemoryBudget budget) {
+        beingCleanedFiles.remove(fileNum);
+        assert !isFileCleaningInProgress(fileNum);
         cleanedFiles.add(fileNum);
         Set<DatabaseId> oldDatabases =
             cleanedFilesDatabases.put(fileNum, databases);
         adjustMemoryBudget(budget, oldDatabases, databases);
-        beingCleanedFiles.remove(fileNum);
     }
 
     /**
@@ -280,20 +345,11 @@ public class FileSelector {
     }
 
     /**
-     * Returns a read-only copy of to-be-cleaned and being-cleaned files that
-     * can be accessed without synchronization.
+     * Returns a read-only copy of to-be-cleaned files that can be accessed
+     * without synchronization.
      */
-    synchronized Set<Long> getMustBeCleanedFiles() {
-        Set<Long> set = new HashSet<Long>(toBeCleanedFiles);
-        set.addAll(beingCleanedFiles);
-        return set;
-    }
-
-    /**
-     * Returns the number of files waiting to-be-cleaned.
-     */
-    synchronized int getBacklog() {
-        return toBeCleanedFiles.size();
+    synchronized Set<Long> getToBeCleanedFiles() {
+        return new HashSet<Long>(toBeCleanedFiles);
     }
 
     /**
@@ -314,26 +370,30 @@ public class FileSelector {
      * When a checkpoint is complete, move the previously cleaned and
      * fully-processed files to the checkpointed and safe-to-delete sets.
      */
-    synchronized void updateFilesAtCheckpointEnd(
-                     CheckpointStartCleanerState info) {
+    synchronized void
+        updateFilesAtCheckpointEnd(CheckpointStartCleanerState info) {
 
         if (!info.isEmpty()) {
 
             Set<Long> previouslyCleanedFiles = info.getCleanedFiles();
             if (previouslyCleanedFiles != null) {
+                cleanedFiles.removeAll(previouslyCleanedFiles);
+                assert !isFileCleaningInProgress(previouslyCleanedFiles) :
+                    toString();
                 if (anyPendingDuringCheckpoint) {
                     checkpointedFiles.addAll(previouslyCleanedFiles);
                 } else {
                     safeToDeleteFiles.addAll(previouslyCleanedFiles);
                 }
-                cleanedFiles.removeAll(previouslyCleanedFiles);
             }
 
             Set<Long> previouslyProcessedFiles =
                 info.getFullyProcessedFiles();
             if (previouslyProcessedFiles != null) {
-                safeToDeleteFiles.addAll(previouslyProcessedFiles);
                 fullyProcessedFiles.removeAll(previouslyProcessedFiles);
+                assert !isFileCleaningInProgress(previouslyProcessedFiles) :
+                    toString();
+                safeToDeleteFiles.addAll(previouslyProcessedFiles);
             }
 
             updateProcessedFiles();
@@ -343,8 +403,10 @@ public class FileSelector {
     /**
      * Adds the given LN info to the pending LN set.
      */
-    synchronized boolean addPendingLN(LN ln, DatabaseId dbId,
-                                      byte[] key, byte[] dupKey) {
+    synchronized boolean addPendingLN(LN ln,
+                                      DatabaseId dbId,
+                                      byte[] key,
+                                      byte[] dupKey) {
         assert ln != null;
 
         boolean added = pendingLNs.put
@@ -437,9 +499,18 @@ public class FileSelector {
      * finally been deleted.
      */
     synchronized void removeDeletedFile(Long fileNum, MemoryBudget budget) {
-        safeToDeleteFiles.remove(fileNum);
-        Set<DatabaseId> oldDatabases = cleanedFilesDatabases.remove(fileNum);
-        adjustMemoryBudget(budget, oldDatabases, null /*newDatabases*/);
+        /* Ensure that no remnants of a deleted file remain. [#17752] */
+        removeAllFileReferences(fileNum, budget);
+    }
+
+    /**
+     * Update memory budgets when the environment is closed and will never be
+     * accessed again.
+     */
+    synchronized void close(MemoryBudget budget) {
+        for (Set<DatabaseId> oldDatabases : cleanedFilesDatabases.values()) {
+            adjustMemoryBudget(budget, oldDatabases, null /*newDatabases*/);
+        }
     }
 
     /**
@@ -516,4 +587,19 @@ public class FileSelector {
             return fullyProcessedFiles;
         }
     }
+
+    @Override
+    public String toString() {
+        return "toBeCleanedFiles = " + toBeCleanedFiles +
+               " beingCleanedFiles = " + beingCleanedFiles +
+               " cleanedFiles = " + cleanedFiles +
+               " cleanedFilesDatabases = " + cleanedFilesDatabases +
+               " checkpointedFiles = " + checkpointedFiles +
+               " fullyProcessedFiles = " + fullyProcessedFiles +
+               " safeToDeleteFiles = " + safeToDeleteFiles +
+               " pendingLNs = " + pendingLNs +
+               " pendingDBs = " + pendingDBs +
+               " anyPendingDuringCheckpoint = " + anyPendingDuringCheckpoint +
+               " lowUtilizationFiles = " + lowUtilizationFiles;
+    }
 }
