@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: UtilizationTracker.java,v 1.26 2008/05/13 01:44:49 cwl Exp $
+ * $Id: UtilizationTracker.java,v 1.27.2.3 2010/01/04 15:30:28 cwl Exp $
  */
 
 package com.sleepycat.je.cleaner;
@@ -50,6 +50,7 @@ public class UtilizationTracker extends BaseUtilizationTracker {
         super(env, cleaner);
     }
 
+    @Override
     public EnvironmentImpl getEnvironment() {
         return env;
     }
@@ -185,6 +186,9 @@ public class UtilizationTracker extends BaseUtilizationTracker {
      * database key.
      *
      * <p>Must be called under the log write latch.</p>
+     *
+     * @return the summary, or null if the DB should not be tracked because
+     * the file has been deleted, or null if the databaseKey param is null.
      */
     DbFileSummary getDbFileSummary(Object databaseKey, long fileNum) {
         DatabaseImpl db = (DatabaseImpl) databaseKey;
