@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: DatabaseImpl.java,v 1.204 2008/05/19 17:52:17 linda Exp $
+ * $Id: DatabaseImpl.java,v 1.205.2.7 2010/03/23 15:02:07 cwl Exp $
  */
 
 package com.sleepycat.je.dbi;
@@ -111,6 +111,8 @@ public class DatabaseImpl implements Loggable, Cloneable {
     private static final byte IS_REPLICATED_BIT = 0x4; // isReplicated()
     private static final byte NOT_REPLICATED_BIT = 0x8;// notReplicated()
     private static final byte PREFIXING_ENABLED = 0x10;// getKeyPrefixing()
+    /* getUtilizationRepairDone() */
+    private static final byte UTILIZATION_REPAIR_DONE = 0x20;
 
     private DatabaseId id;             // unique id
     private Tree tree;
@@ -168,7 +170,7 @@ public class DatabaseImpl implements Loggable, Cloneable {
     /* For unit tests */
     private TestHook pendingDeletedHook;
 
-    /* 
+    /*
      * For debugging -- this gives the ability to force all non-internal
      * databases to use key prefixing.
      *
@@ -230,6 +232,12 @@ public class DatabaseImpl implements Loggable, Cloneable {
 
         createdAtLogVersion = LogEntryType.LOG_VERSION;
 
+        /*
+         * New DB records do not need utilization repair.  Set this before
+         * calling initWithEnvironment to avoid repair overhead.
+         */
+        setUtilizationRepairDone();
+
         commonInit();
 
         initWithEnvironment();
@@ -332,6 +340,19 @@ public class DatabaseImpl implements Loggable, Cloneable {
 
         /* Budgets memory for the utilization info. */
         dbFileSummaries.init(envImpl);
+
+        /*
+         * Repair utilization info if necessary.  The repair flag will not be
+         * set for MapLNs written by JE 3.3.74 and earlier, and will be set for
+         * all MapLNs written thereafter.  Make the utilization dirty to force
+         * the MapLN to be flushed.  Even if no repair is performed, we want to
+         * write the updated flag.  [#16610]
+         */
+        if (!getUtilizationRepairDone()) {
+            dbFileSummaries.repair(envImpl);
+            setDirtyUtilization();
+            setUtilizationRepairDone();
+        }
     }
 
     /**
@@ -508,6 +529,24 @@ public class DatabaseImpl implements Loggable, Cloneable {
         flags |= NOT_REPLICATED_BIT;
     }
 
+    /**
+     * Is public for unit testing.
+     */
+    public boolean getUtilizationRepairDone() {
+        return (flags & UTILIZATION_REPAIR_DONE) != 0;
+    }
+
+    private void setUtilizationRepairDone() {
+        flags |= UTILIZATION_REPAIR_DONE;
+    }
+
+    /**
+     * Is public for unit testing.
+     */
+    public void clearUtilizationRepairDone() {
+        flags &= ~UTILIZATION_REPAIR_DONE;
+    }
+
     public int getNodeMaxEntries() {
         return maxMainTreeEntriesPerNode;
     }
@@ -543,7 +582,7 @@ public class DatabaseImpl implements Loggable, Cloneable {
             val += 2 * MemoryBudget.byteArraySize
                 (duplicateComparatorBytes.length);
         }
-        
+
         return val;
     }
 
@@ -729,9 +768,9 @@ public class DatabaseImpl implements Loggable, Cloneable {
      * garbage collected, due to eviction or recovery.
      */
     public void releaseTreeAdminMemory() {
-        /* 
+        /*
          * There's no need to account for INs which belong to this database,
-         * because those are closed by the EnvironmentImpl when clearing 
+         * because those are closed by the EnvironmentImpl when clearing
          * the INList.  Do adjust memory budget for utilization info.
          */
         dbFileSummaries.subtractFromMemoryBudget();
@@ -860,7 +899,9 @@ public class DatabaseImpl implements Loggable, Cloneable {
             dirtyUtilization = true;
         }
         assert dbFileSummaries != null;
-        return dbFileSummaries.get(fileNum, true /* adjustMemBudget */);
+        return dbFileSummaries.get(fileNum, true /*adjustMemBudget*/,
+                                   true /*checkResurrected*/,
+                                   envImpl.getFileManager());
     }
 
     /**
@@ -1249,14 +1290,15 @@ public class DatabaseImpl implements Loggable, Cloneable {
             new StatsAccumulator(out,
                                  config.getShowProgressInterval(),
                                  emptyStats) {
-                    void verifyNode(Node node) {
+                @Override
+                void verifyNode(Node node) {
 
-                        try {
-                            node.verify(null);
-                        } catch (DatabaseException INE) {
-                            progressStream.println(INE);
-                        }
+                    try {
+                        node.verify(null);
+                    } catch (DatabaseException INE) {
+                        progressStream.println(INE);
                     }
+                }
                 };
         boolean ok = walkDatabaseTree(statsAcc, out, config.getPrintInfo());
         statsAcc.copyToStats(emptyStats);
@@ -1304,6 +1346,11 @@ public class DatabaseImpl implements Loggable, Cloneable {
                 }
                 boolean done = false;
                 while (!done) {
+
+                    /* Perform eviction before each cursor operation. */
+                    envImpl.getEvictor().doCriticalEviction
+                        (false /*backgroundIO*/);
+
                     try {
                         status = cursor.getNext
                             (key, foundData, LockType.NONE, true /*forward*/,
@@ -1405,7 +1452,7 @@ public class DatabaseImpl implements Loggable, Cloneable {
             binsSeenByLevel = new long[MAX_LEVELS];
             dinsSeenByLevel = new long[MAX_LEVELS];
             dbinsSeenByLevel = new long[MAX_LEVELS];
-        
+
             this.useStats = useStats;
         }
 
@@ -1613,8 +1660,6 @@ public class DatabaseImpl implements Loggable, Cloneable {
                                byte[] ignore2)
             throws DatabaseException {
 
-            assert childLsn != DbLsn.NULL_LSN;
-
             /*
              * Check if we've exceeded either the max time or max bytes
              * allowed for this preload() call.
@@ -1733,7 +1778,7 @@ public class DatabaseImpl implements Loggable, Cloneable {
 	protected void releaseRootIN(IN root)
 	    throws DatabaseException {
 
-	    root.releaseLatch();
+            root.releaseLatch();
 	}
 
 	/*
@@ -1811,20 +1856,20 @@ public class DatabaseImpl implements Loggable, Cloneable {
                      cacheBudget + " bytes.");
             }
 
-            PreloadStats stats = new PreloadStats();
+            PreloadStats pstats = new PreloadStats();
             PreloadProcessor callback =
-                new PreloadProcessor(envImpl, maxBytes, targetTime, stats);
+                new PreloadProcessor(envImpl, maxBytes, targetTime, pstats);
             SortedLSNTreeWalker walker =
                 new PreloadLSNTreeWalker(this, callback, config);
             walker.setPassNullLSNNodes(true);
             try {
                 walker.walk();
             } catch (HaltPreloadException HPE) {
-                stats.setStatus(HPE.getStatus());
+                pstats.setStatus(HPE.getStatus());
             }
 
             assert LatchSupport.countLatchesHeld() == 0;
-            return stats;
+            return pstats;
         } catch (Error E) {
             envImpl.invalidate(E);
             throw E;
@@ -1902,9 +1947,9 @@ public class DatabaseImpl implements Loggable, Cloneable {
         throws DatabaseException {
 
         try {
-            PreloadStats stats = new PreloadStats();
+            PreloadStats pstats = new PreloadStats();
 
-            CountProcessor callback = new CountProcessor(envImpl, stats);
+            CountProcessor callback = new CountProcessor(envImpl, pstats);
             ExceptionPredicate excPredicate = new CountExceptionPredicate();
             SortedLSNTreeWalker walker =
                 new SortedLSNTreeWalker(this, false /* setDbState */,
@@ -1912,13 +1957,11 @@ public class DatabaseImpl implements Loggable, Cloneable {
                                         excPredicate);
             /* Don't descend down into the dup tree. Use the DupCountLN. */
             walker.setProcessDupTree(false);
-            if (isDurableDeferredWrite()) {
-                walker.setPassNullLSNNodes(true);
-            }
+            walker.setPassNullLSNNodes(true);
             walker.walk();
 
             assert LatchSupport.countLatchesHeld() == 0;
-            return stats.getNLNsLoaded();
+            return pstats.getNLNsLoaded();
         } catch (Error E) {
             envImpl.invalidate(E);
             throw E;
@@ -1958,7 +2001,7 @@ public class DatabaseImpl implements Loggable, Cloneable {
         }
         sb.append(">");
         if (dbFileSummaries != null) {
-            Iterator<Map.Entry<Long,DbFileSummary>> entries = 
+            Iterator<Map.Entry<Long,DbFileSummary>> entries =
                 (Iterator<Map.Entry<Long,DbFileSummary>>)
                 dbFileSummaries.entrySet().iterator();
             while (entries.hasNext()) {
@@ -2000,7 +2043,8 @@ public class DatabaseImpl implements Loggable, Cloneable {
             1;  // createdAtLogVersion
 
         size += LogUtils.getPackedIntLogSize(dbFileSummaries.size());
-        Iterator<Map.Entry<Long,DbFileSummary>> i = 
+
+        Iterator<Map.Entry<Long,DbFileSummary>> i =
             (Iterator<Map.Entry<Long,DbFileSummary>>)
             dbFileSummaries.entrySet().iterator();
         while (i.hasNext()) {
@@ -2027,7 +2071,7 @@ public class DatabaseImpl implements Loggable, Cloneable {
         LogUtils.writePackedInt(logBuffer, maxDupTreeEntriesPerNode);
         logBuffer.put(createdAtLogVersion);
         LogUtils.writePackedInt(logBuffer, dbFileSummaries.size());
-        Iterator<Map.Entry<Long,DbFileSummary>> i = 
+        Iterator<Map.Entry<Long,DbFileSummary>> i =
             (Iterator<Map.Entry<Long,DbFileSummary>>)
             dbFileSummaries.entrySet().iterator();
 
@@ -2095,8 +2139,8 @@ public class DatabaseImpl implements Loggable, Cloneable {
                     if (obj instanceof String) {
                     	String className = (String)obj;
 
-                        Class<? extends Comparator<byte[]>> cls = 
-                       (Class<? extends Comparator<byte[]>>) 
+                        Class<? extends Comparator<byte[]>> cls =
+                       (Class<? extends Comparator<byte[]>>)
                             Class.forName(className);
 
                         btreeComparator = instantiateComparator(cls, "Btree");
@@ -2116,8 +2160,8 @@ public class DatabaseImpl implements Loggable, Cloneable {
                         (duplicateComparatorBytes, "Duplicate");
                     if (obj instanceof String) {
 
-                        Class<? extends Comparator<byte[]>> cls = 
-                             (Class<? extends Comparator<byte[]>>) 
+                        Class<? extends Comparator<byte[]>> cls =
+                             (Class<? extends Comparator<byte[]>>)
                             Class.forName((String) obj);
 
                         duplicateComparator =
@@ -2152,7 +2196,8 @@ public class DatabaseImpl implements Loggable, Cloneable {
             for (int i = 0; i < nFiles; i += 1) {
                 long fileNum = LogUtils.readPackedLong(itemBuffer);
                 DbFileSummary summary = dbFileSummaries.get
-                    (Long.valueOf(fileNum), false /* adjustMemBudget */);
+                    (Long.valueOf(fileNum), false /*adjustMemBudget*/,
+                     false /*checkResurrected*/, null /*fileManager*/);
                 summary.readFromLog(itemBuffer, entryVersion);
             }
         }
@@ -2173,8 +2218,8 @@ public class DatabaseImpl implements Loggable, Cloneable {
         id.dumpLog(sb, verbose);
         tree.dumpLog(sb, verbose);
         if (dbFileSummaries != null) {
-            Iterator<Map.Entry<Long,DbFileSummary>> entries = 
-                (Iterator<Map.Entry<Long,DbFileSummary>>) 
+            Iterator<Map.Entry<Long,DbFileSummary>> entries =
+                (Iterator<Map.Entry<Long,DbFileSummary>>)
                 dbFileSummaries.entrySet().iterator();
 
             while (entries.hasNext()) {
@@ -2193,7 +2238,7 @@ public class DatabaseImpl implements Loggable, Cloneable {
     static void dumpFlags(StringBuffer sb, boolean verbose, byte flags) {
         sb.append(" dupsort=\"").append((flags & DUPS_ALLOWED_BIT) != 0);
         sb.append("\" replicated=\"").append((flags & IS_REPLICATED_BIT) != 0);
-        sb.append("\" temp=\"").append((flags & TEMPORARY_BIT) 
+        sb.append("\" temp=\"").append((flags & TEMPORARY_BIT)
                                        != 0).append("\" ");
     }
 
@@ -2215,7 +2260,7 @@ public class DatabaseImpl implements Loggable, Cloneable {
     /**
      * Used for log dumping.
      */
-    private static String 
+    private static String
         getComparatorClassName(Comparator<byte[]> comparator) {
 
         if (comparator != null) {
@@ -2229,8 +2274,8 @@ public class DatabaseImpl implements Loggable, Cloneable {
      * Used both to read from the log and to validate a comparator when set in
      * DatabaseConfig.
      */
-    public static Comparator<byte[]> 
-        instantiateComparator(Class<? extends Comparator<byte[]>> 
+    public static Comparator<byte[]>
+        instantiateComparator(Class<? extends Comparator<byte[]>>
                               comparatorClass,
                               String comparatorType)
         throws LogException {
@@ -2255,7 +2300,7 @@ public class DatabaseImpl implements Loggable, Cloneable {
     /**
      * Used to validate a comparator when set in DatabaseConfig.
      */
-    public static Comparator<byte[]> 
+    public static Comparator<byte[]>
         instantiateComparator(Comparator<byte[]> comparator,
                               String comparatorType)
         throws DatabaseException {
@@ -2348,7 +2393,7 @@ public class DatabaseImpl implements Loggable, Cloneable {
             Object obj = bytesToObject(comparatorBytes, comparatorType);
             if (obj instanceof String) {
                 String className = (String)obj;
-                Class<? extends Comparator<byte[]>> cls = 
+                Class<? extends Comparator<byte[]>> cls =
                     (Class<? extends Comparator<byte[]>>)
                     Class.forName(className);
                 comparator = instantiateComparator(cls, comparatorType);
