@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: EnvironmentImpl.java,v 1.301 2008/05/30 19:07:41 mark Exp $
+ * $Id: EnvironmentImpl.java,v 1.301.2.8 2010/01/04 15:30:28 cwl Exp $
  */
 
 package com.sleepycat.je.dbi;
@@ -222,6 +222,13 @@ public class EnvironmentImpl implements EnvConfigObserver {
         IS_DALVIK = "Dalvik".equals(System.getProperty("java.vm.name"));
     }
 
+    public static final boolean IS_WINDOWS_7;
+
+    static {
+        String osName = System.getProperty("os.name");
+        IS_WINDOWS_7 = (osName != null && osName.startsWith("Windows 7"));
+    }
+
     /*
      * Timeout for waiting for API lockout to finish.
      */
@@ -300,8 +307,7 @@ public class EnvironmentImpl implements EnvConfigObserver {
                 }
             }
 
-            directNIO =
-                configManager.getBoolean(EnvironmentParams.LOG_DIRECT_NIO);
+            directNIO = false;
             fairLatches =
                 configManager.getBoolean(EnvironmentParams.ENV_FAIR_LATCHES);
             isReadOnly =
@@ -351,7 +357,7 @@ public class EnvironmentImpl implements EnvConfigObserver {
             /*
              * The node sequence transient ids, but not the node ids, must be
              * created before the DbTree is created because the transient node
-             * id sequence is used during the creation of the dbtree. 
+             * id sequence is used during the creation of the dbtree.
              */
             nodeSequence = new NodeSequence();
             nodeSequence.initTransientNodeId();
@@ -359,7 +365,7 @@ public class EnvironmentImpl implements EnvConfigObserver {
             /*
              * Instantiate a new, blank dbtree. If the environment already
              * exists, recovery will recreate the dbMapTree from the log and
-             * overwrite this instance. 
+             * overwrite this instance.
              */
             dbMapTree = new DbTree(this, replicationIntended);
 
@@ -501,9 +507,10 @@ public class EnvironmentImpl implements EnvConfigObserver {
              * exists, recovery will recreate the dbMapTree from the log and
              * overwrite this instance.
              */
-        if (dbMapTree != null) {
-            dbMapTree.close();
-        }
+            if (dbMapTree != null) {
+                dbMapTree.close();
+            }
+
             dbMapTree = new DbTree(this, replicationIntended);
             mapTreeRootLsn = DbLsn.NULL_LSN;
             referenceCount = 0;
@@ -835,7 +842,7 @@ public class EnvironmentImpl implements EnvConfigObserver {
                                                 false,
                                                 null);
             LockGrantType grant = lr.getLockGrant();
-                
+
             if (grant == LockGrantType.DENIED) {
                 throw new APILockedException("API Lock timeout");
             }
@@ -1446,7 +1453,7 @@ public class EnvironmentImpl implements EnvConfigObserver {
                 errors.println();
             }
 
-            /* 
+            /*
              * Close the memory budgets on these components before the
              * INList is forcibly released and the treeAdmin budget is
              * cleared.
@@ -1537,11 +1544,11 @@ public class EnvironmentImpl implements EnvConfigObserver {
         return threadLocalReferenceCount;
     }
 
-    public static synchronized void incThreadLocalReferenceCount() {
+    static synchronized void incThreadLocalReferenceCount() {
         threadLocalReferenceCount++;
     }
 
-    public static synchronized void decThreadLocalReferenceCount() {
+    static synchronized void decThreadLocalReferenceCount() {
         threadLocalReferenceCount--;
     }
 
@@ -1594,9 +1601,17 @@ public class EnvironmentImpl implements EnvConfigObserver {
         }
 
         long memoryUsage = memoryBudget.getVariableCacheUsage();
-        if (memoryUsage!= 0) {
+        if (memoryUsage != 0) {
             clean = false;
             System.err.println("Local Cache Usage = " +  memoryUsage);
+            System.err.println("Tree Memory Usage = " +
+                               memoryBudget.getTreeMemoryUsage());
+            System.err.println("Admin Memory Usage = " +
+                               memoryBudget.getAdminMemoryUsage());
+            System.err.println("Tree Admin Memory Usage = " +
+                               memoryBudget.getTreeAdminMemoryUsage());
+            System.err.println("Lock Memory Usage = " +
+                               memoryBudget.getLockMemoryUsage());
             EnvironmentStats memoryStats = new EnvironmentStats();
             memoryBudget.loadStats(new StatsConfig(),
                                    memoryStats);
@@ -1666,6 +1681,11 @@ public class EnvironmentImpl implements EnvConfigObserver {
     public int invokeCleaner()
         throws DatabaseException {
 
+        if (isReadOnly || isMemOnly) {
+            throw new IllegalStateException
+                ("Log cleaning not allowed in a read-only or memory-only " +
+                 "environment");
+        }
         if (cleaner != null) {
             return cleaner.doClean(true,   // cleanMultipleFiles
                                    false); // forceCleaning
@@ -1840,7 +1860,7 @@ public class EnvironmentImpl implements EnvConfigObserver {
         return isMemOnly;
     }
 
-    /* 
+    /*
      * FUTURE: change this to be non-static. It's static now just to avoid
      * passing down parameters in various places.
      */
@@ -1859,10 +1879,6 @@ public class EnvironmentImpl implements EnvConfigObserver {
         return dbEviction;
     }
 
-    public boolean useDirectNIO() {
-        return directNIO;
-    }
-
     public static int getAdler32ChunkSize() {
         return adler32ChunkSize;
     }
@@ -1916,7 +1932,7 @@ public class EnvironmentImpl implements EnvConfigObserver {
     public NodeSequence getNodeSequence() {
         return nodeSequence;
     }
-    
+
     /**
      * Clones the current configuration.
      */
