@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: MemoryBudget.java,v 1.85 2008/05/29 13:50:07 cwl Exp $
+ * $Id: MemoryBudget.java,v 1.86.2.4 2010/01/04 15:30:28 cwl Exp $
  */
 
 package com.sleepycat.je.dbi;
@@ -54,6 +54,7 @@ public class MemoryBudget implements EnvConfigObserver {
      *
      * _32 values are the same on Windows and Solaris.
      * _64 values are from 1.5.0_05 on Solaris.
+     * _Dalvik values are from running the Sizeof test on Android.
      *
      * Specifically:
      *
@@ -69,71 +70,90 @@ public class MemoryBudget implements EnvConfigObserver {
     // 7
     private final static int LONG_OVERHEAD_32 = 16;
     private final static int LONG_OVERHEAD_64 = 24;
+    private final static int LONG_OVERHEAD_DALVIK = 24;
 
     // 8
     private final static int ARRAY_OVERHEAD_32 = 16;
     private final static int ARRAY_OVERHEAD_64 = 24;
+    private final static int ARRAY_OVERHEAD_DALVIK = 24;
 
     private final static int ARRAY_SIZE_INCLUDED_32 = 4;
     private final static int ARRAY_SIZE_INCLUDED_64 = 0;
+    private final static int ARRAY_SIZE_INCLUDED_DALVIK = 4;
 
     // 2
     private final static int OBJECT_OVERHEAD_32 = 8;
     private final static int OBJECT_OVERHEAD_64 = 16;
+    private final static int OBJECT_OVERHEAD_DALVIK = 16;
 
     // (4 - ARRAY_OVERHEAD) / 256
     // 64b: 4 is 2072
+    // Dalvik; 4 is 1048
     private final static int OBJECT_ARRAY_ITEM_OVERHEAD_32 = 4;
     private final static int OBJECT_ARRAY_ITEM_OVERHEAD_64 = 8;
+    private final static int OBJECT_ARRAY_ITEM_OVERHEAD_DALVIK = 4;
 
     // 20
     private final static int HASHMAP_OVERHEAD_32 = 120;
     private final static int HASHMAP_OVERHEAD_64_15 = 216;
     private final static int HASHMAP_OVERHEAD_64_16 = 218;
+    private final static int HASHMAP_OVERHEAD_DALVIK = 128;
 
     // 21 - OBJECT_OVERHEAD - HASHMAP_OVERHEAD
     // 64b: 21 is max(280,...,287) on Linux/Solaris 1.5/1.6
+    // Dalvik: 21 is 176
     private final static int HASHMAP_ENTRY_OVERHEAD_32 = 24;
     private final static int HASHMAP_ENTRY_OVERHEAD_64 = 55;
+    private final static int HASHMAP_ENTRY_OVERHEAD_DALVIK = 32;
 
     // 22
     private final static int HASHSET_OVERHEAD_32 = 136;
     private final static int HASHSET_OVERHEAD_64 = 240;
+    private final static int HASHSET_OVERHEAD_DALVIK = 144;
 
     // 23 - OBJECT_OVERHEAD - HASHSET_OVERHEAD
     // 64b: 23 is max(304,...,311) on Linux/Solaris
+    // Dalvik: 23 is 192
     private final static int HASHSET_ENTRY_OVERHEAD_32 = 24;
     private final static int HASHSET_ENTRY_OVERHEAD_64 = 55;
+    private final static int HASHSET_ENTRY_OVERHEAD_DALVIK = 32;
 
     // HASHMAP_OVERHEAD * 2
     private final static int TWOHASHMAPS_OVERHEAD_32 = 240;
     private final static int TWOHASHMAPS_OVERHEAD_64_15 = 432;
     private final static int TWOHASHMAPS_OVERHEAD_64_16 = 436;
+    private final static int TWOHASHMAPS_OVERHEAD_DALVIK = 256;
 
     // 34
     private final static int TREEMAP_OVERHEAD_32_15 = 40;
     private final static int TREEMAP_OVERHEAD_32_16 = 48;
     private final static int TREEMAP_OVERHEAD_64_15 = 64;
     private final static int TREEMAP_OVERHEAD_64_16 = 80;
+    private final static int TREEMAP_OVERHEAD_DALVIK = 40;
 
     // 35 - OBJECT_OVERHEAD - TREEMAP_OVERHEAD
     // 64b: 35 is 144 on 1.5 and 160 on 1.6, result is 64 for both
+    // Dalvik: 35 is 96
     private final static int TREEMAP_ENTRY_OVERHEAD_32 = 32;
     private final static int TREEMAP_ENTRY_OVERHEAD_64 = 64;
+    private final static int TREEMAP_ENTRY_OVERHEAD_DALVIK = 40;
 
     // 36
     private final static int MAPLN_OVERHEAD_32_15 = 640;
     private final static int MAPLN_OVERHEAD_32_16 = 664;
     private final static int MAPLN_OVERHEAD_64_15 = 1096;
     private final static int MAPLN_OVERHEAD_64_16 = 1136;
+    private final static int MAPLN_OVERHEAD_DALVIK = 744;
 
     // 9
     private final static int LN_OVERHEAD_32 = 24;
     private final static int LN_OVERHEAD_64 = 40;
+    private final static int LN_OVERHEAD_DALVIK = 32;
 
     // 19
     private final static int DUPCOUNTLN_OVERHEAD_32 = 24;
     private final static int DUPCOUNTLN_OVERHEAD_64 = 48;
+    private final static int DUPCOUNTLN_OVERHEAD_DALVIK = 32;
 
     // 12
     // 64b: 12 is max(536, 539) on Linux/Solaris on 1.5
@@ -141,43 +161,52 @@ public class MemoryBudget implements EnvConfigObserver {
     private final static int BIN_FIXED_OVERHEAD_32 = 370; // 344 in 1.5
     private final static int BIN_FIXED_OVERHEAD_64_15 = 544;
     private final static int BIN_FIXED_OVERHEAD_64_16 = 584;
+    private final static int BIN_FIXED_OVERHEAD_DALVIK = 458;
 
     // 18
     private final static int DIN_FIXED_OVERHEAD_32 = 377; // 352 in 1.5
     private final static int DIN_FIXED_OVERHEAD_64_15 = 552;
     private final static int DIN_FIXED_OVERHEAD_64_16 = 596;
+    private final static int DIN_FIXED_OVERHEAD_DALVIK = 451;
 
     // 17
     // 64b: 17 is max(592,593) on Linux/Solaris on 1.6
     private final static int DBIN_FIXED_OVERHEAD_32 = 377; // 352 in 1.5
     private final static int DBIN_FIXED_OVERHEAD_64_15 = 560;
     private final static int DBIN_FIXED_OVERHEAD_64_16 = 600;
+    private final static int DBIN_FIXED_OVERHEAD_DALVIK = 458;
 
     // 13
     // 339 is max(312,339) on Solaris 1.5 vs 1.6
     private final static int IN_FIXED_OVERHEAD_32 = 339; // 312 in 1.5
     private final static int IN_FIXED_OVERHEAD_64_15 = 488;
     private final static int IN_FIXED_OVERHEAD_64_16 = 528;
+    private final static int IN_FIXED_OVERHEAD_DALVIK = 402;
 
     // 6
     private final static int KEY_OVERHEAD_32 = 16;
     private final static int KEY_OVERHEAD_64 = 24;
+    private final static int KEY_OVERHEAD_DALVIK = 16;
 
     // 24
     private final static int LOCKIMPL_OVERHEAD_32 = 24;
     private final static int LOCKIMPL_OVERHEAD_64 = 48;
+    private final static int LOCKIMPL_OVERHEAD_DALVIK = 32;
 
     // 42
     private final static int THINLOCKIMPL_OVERHEAD_32 = 16;
     private final static int THINLOCKIMPL_OVERHEAD_64 = 32;
+    private final static int THINLOCKIMPL_OVERHEAD_DALVIK = 24;
 
     // 25
     private final static int LOCKINFO_OVERHEAD_32 = 16;
     private final static int LOCKINFO_OVERHEAD_64 = 32;
+    private final static int LOCKINFO_OVERHEAD_DALVIK = 24;
 
     // 37
     private final static int WRITE_LOCKINFO_OVERHEAD_32 = 32;
     private final static int WRITE_LOCKINFO_OVERHEAD_64 = 40;
+    private final static int WRITE_LOCKINFO_OVERHEAD_DALVIK = 40;
 
     /*
      * Txn memory is the size for the Txn + a hashmap entry
@@ -186,58 +215,56 @@ public class MemoryBudget implements EnvConfigObserver {
     // 15
     private final static int TXN_OVERHEAD_32 = 186;
     private final static int TXN_OVERHEAD_64 = 281;
+    private final static int TXN_OVERHEAD_DALVIK = 218;
 
     // 26
     private final static int CHECKPOINT_REFERENCE_SIZE_32 = 40 +
         HASHSET_ENTRY_OVERHEAD_32;
     private final static int CHECKPOINT_REFERENCE_SIZE_64 = 56 +
         HASHSET_ENTRY_OVERHEAD_64;
+    private final static int CHECKPOINT_REFERENCE_SIZE_DALVIK = 40;
 
     /* The per-log-file bytes used in UtilizationProfile. */
     // 29 / 10
     private final static int UTILIZATION_PROFILE_ENTRY_32 = 101;
     private final static int UTILIZATION_PROFILE_ENTRY_64 = 153;
+    private final static int UTILIZATION_PROFILE_ENTRY_DALVIK = 124;
 
     //  38
     private final static int DBFILESUMMARY_OVERHEAD_32 = 40;
     private final static int DBFILESUMMARY_OVERHEAD_64 = 48;
+    private final static int DBFILESUMMARY_OVERHEAD_DALVIK = 48;
 
     /* Tracked File Summary overheads. */
     // 31
     private final static int TFS_LIST_INITIAL_OVERHEAD_32 = 464;
     private final static int TFS_LIST_INITIAL_OVERHEAD_64 = 504;
+    private final static int TFS_LIST_INITIAL_OVERHEAD_DALVIK = 472;
 
     // 30
     // 64b: 30 is max(464,464,464,465) on Linux/Solaris on 1.5/1.6
     private final static int TFS_LIST_SEGMENT_OVERHEAD_32 = 440;
     private final static int TFS_LIST_SEGMENT_OVERHEAD_64 = 465;
+    private final static int TFS_LIST_SEGMENT_OVERHEAD_DALVIK = 448;
 
     // 33
     private final static int LN_INFO_OVERHEAD_32 = 24;
     private final static int LN_INFO_OVERHEAD_64 = 48;
+    private final static int LN_INFO_OVERHEAD_DALVIK = 32;
 
     // 43
     private final static int FILESUMMARYLN_OVERHEAD_32 = 112;
     private final static int FILESUMMARYLN_OVERHEAD_64 = 168;
+    private final static int FILESUMMARYLN_OVERHEAD_DALVIK = 136;
 
     /* Approximate element size in an ArrayList of Long. */
     // (28 - 27) / 10
     // 32b: 28 and 27 are 240 and 40, resp.
     // 64b: 28 and 27 are 384 and 64, resp.
+    // Dalvik: 28 and 27 are 336 and 32, resp.
     private final static int LONG_LIST_PER_ITEM_OVERHEAD_32 = 20;
     private final static int LONG_LIST_PER_ITEM_OVERHEAD_64 = 32;
-
-    // 43
-    private final static int INCOMING_CONNECTION_HANDLER_OVERHEAD_32 = 24;
-    private final static int INCOMING_CONNECTION_HANDLER_OVERHEAD_64 = 48;
-
-    // 44
-    private final static int RECEIVE_MESSAGE_HANDLER_OVERHEAD_32 = 40;
-    private final static int RECEIVE_MESSAGE_HANDLER_OVERHEAD_64 = 80;
-
-    // 45
-    private final static int SEND_MESSAGE_HANDLER_OVERHEAD_32 = 56;
-    private final static int SEND_MESSAGE_HANDLER_OVERHEAD_64 = 96;
+    private final static int LONG_LIST_PER_ITEM_OVERHEAD_DALVIK = 30;
 
     public final static int LONG_OVERHEAD;
     public final static int ARRAY_OVERHEAD;
@@ -280,109 +307,150 @@ public class MemoryBudget implements EnvConfigObserver {
     private final static String FORCE_JVM_ARCH = "je.forceJVMArch";
 
     static {
-        String javaVersion = System.getProperty("java.version");
-        boolean isJVM15 = javaVersion != null &&
-                          javaVersion.startsWith("1.5.");
-
-        boolean is64 = false;
-        String overrideArch = System.getProperty(FORCE_JVM_ARCH);
-        try {
-            if (overrideArch == null) {
-                String arch = System.getProperty(JVM_ARCH_PROPERTY);
-                if (arch != null) {
-                    is64 = Integer.parseInt(arch) == 64;
-                }
-            } else {
-                is64 = Integer.parseInt(overrideArch) == 64;
-            }
-        } catch (NumberFormatException NFE) {
-            NFE.printStackTrace(System.err);
-        }
-
-        if (is64) {
-            LONG_OVERHEAD = LONG_OVERHEAD_64;
-            ARRAY_OVERHEAD = ARRAY_OVERHEAD_64;
-            ARRAY_SIZE_INCLUDED = ARRAY_SIZE_INCLUDED_64;
-            OBJECT_OVERHEAD = OBJECT_OVERHEAD_64;
-            OBJECT_ARRAY_ITEM_OVERHEAD = OBJECT_ARRAY_ITEM_OVERHEAD_64;
-            HASHMAP_ENTRY_OVERHEAD = HASHMAP_ENTRY_OVERHEAD_64;
-            HASHSET_OVERHEAD = HASHSET_OVERHEAD_64;
-            HASHSET_ENTRY_OVERHEAD = HASHSET_ENTRY_OVERHEAD_64;
-            if (isJVM15) {
-                TREEMAP_OVERHEAD = TREEMAP_OVERHEAD_64_15;
-                MAPLN_OVERHEAD = MAPLN_OVERHEAD_64_15;
-                BIN_FIXED_OVERHEAD = BIN_FIXED_OVERHEAD_64_15;
-                DIN_FIXED_OVERHEAD = DIN_FIXED_OVERHEAD_64_15;
-                DBIN_FIXED_OVERHEAD = DBIN_FIXED_OVERHEAD_64_15;
-                IN_FIXED_OVERHEAD = IN_FIXED_OVERHEAD_64_15;
-                HASHMAP_OVERHEAD = HASHMAP_OVERHEAD_64_15;
-                TWOHASHMAPS_OVERHEAD = TWOHASHMAPS_OVERHEAD_64_15;
-            } else {
-                TREEMAP_OVERHEAD = TREEMAP_OVERHEAD_64_16;
-                MAPLN_OVERHEAD = MAPLN_OVERHEAD_64_16;
-                BIN_FIXED_OVERHEAD = BIN_FIXED_OVERHEAD_64_16;
-                DIN_FIXED_OVERHEAD = DIN_FIXED_OVERHEAD_64_16;
-                DBIN_FIXED_OVERHEAD = DBIN_FIXED_OVERHEAD_64_16;
-                IN_FIXED_OVERHEAD = IN_FIXED_OVERHEAD_64_16;
-                HASHMAP_OVERHEAD = HASHMAP_OVERHEAD_64_16;
-                TWOHASHMAPS_OVERHEAD = TWOHASHMAPS_OVERHEAD_64_16;
-            }
-            TREEMAP_ENTRY_OVERHEAD = TREEMAP_ENTRY_OVERHEAD_64;
-            LN_OVERHEAD = LN_OVERHEAD_64;
-            DUPCOUNTLN_OVERHEAD = DUPCOUNTLN_OVERHEAD_64;
-            TXN_OVERHEAD = TXN_OVERHEAD_64;
-            CHECKPOINT_REFERENCE_SIZE = CHECKPOINT_REFERENCE_SIZE_64;
-            KEY_OVERHEAD = KEY_OVERHEAD_64;
-            LOCKIMPL_OVERHEAD = LOCKIMPL_OVERHEAD_64;
-            THINLOCKIMPL_OVERHEAD = THINLOCKIMPL_OVERHEAD_64;
-            LOCKINFO_OVERHEAD = LOCKINFO_OVERHEAD_64;
-            WRITE_LOCKINFO_OVERHEAD = WRITE_LOCKINFO_OVERHEAD_64;
-            UTILIZATION_PROFILE_ENTRY = UTILIZATION_PROFILE_ENTRY_64;
-            DBFILESUMMARY_OVERHEAD = DBFILESUMMARY_OVERHEAD_64;
-            TFS_LIST_INITIAL_OVERHEAD = TFS_LIST_INITIAL_OVERHEAD_64;
-            TFS_LIST_SEGMENT_OVERHEAD = TFS_LIST_SEGMENT_OVERHEAD_64;
-            LN_INFO_OVERHEAD = LN_INFO_OVERHEAD_64;
-            FILESUMMARYLN_OVERHEAD = FILESUMMARYLN_OVERHEAD_64;
-            LONG_LIST_PER_ITEM_OVERHEAD = LONG_LIST_PER_ITEM_OVERHEAD_64;
+    
+        /* 
+         * On the Dalvik VM (Android), there is no difference in sizes between
+         * any platforms.
+         */
+        if (EnvironmentImpl.IS_DALVIK) {
+            LONG_OVERHEAD = LONG_OVERHEAD_DALVIK;
+            ARRAY_OVERHEAD = ARRAY_OVERHEAD_DALVIK;
+            ARRAY_SIZE_INCLUDED = ARRAY_SIZE_INCLUDED_DALVIK;
+            OBJECT_OVERHEAD = OBJECT_OVERHEAD_DALVIK;
+            OBJECT_ARRAY_ITEM_OVERHEAD = OBJECT_ARRAY_ITEM_OVERHEAD_DALVIK;
+            HASHMAP_OVERHEAD = HASHMAP_OVERHEAD_DALVIK;
+            HASHMAP_ENTRY_OVERHEAD = HASHMAP_ENTRY_OVERHEAD_DALVIK;
+            HASHSET_OVERHEAD = HASHSET_OVERHEAD_DALVIK;
+            HASHSET_ENTRY_OVERHEAD = HASHSET_ENTRY_OVERHEAD_DALVIK;
+            TWOHASHMAPS_OVERHEAD = TWOHASHMAPS_OVERHEAD_DALVIK;
+            TREEMAP_OVERHEAD = TREEMAP_OVERHEAD_DALVIK;
+            TREEMAP_ENTRY_OVERHEAD = TREEMAP_ENTRY_OVERHEAD_DALVIK;
+            MAPLN_OVERHEAD = MAPLN_OVERHEAD_DALVIK;
+            LN_OVERHEAD = LN_OVERHEAD_DALVIK;
+            DUPCOUNTLN_OVERHEAD = DUPCOUNTLN_OVERHEAD_DALVIK;
+            BIN_FIXED_OVERHEAD = BIN_FIXED_OVERHEAD_DALVIK;
+            DIN_FIXED_OVERHEAD = DIN_FIXED_OVERHEAD_DALVIK;
+            DBIN_FIXED_OVERHEAD = DBIN_FIXED_OVERHEAD_DALVIK;
+            IN_FIXED_OVERHEAD = IN_FIXED_OVERHEAD_DALVIK;
+            KEY_OVERHEAD = KEY_OVERHEAD_DALVIK;
+            LOCKIMPL_OVERHEAD = LOCKIMPL_OVERHEAD_DALVIK;
+            THINLOCKIMPL_OVERHEAD = THINLOCKIMPL_OVERHEAD_DALVIK;
+            LOCKINFO_OVERHEAD = LOCKINFO_OVERHEAD_DALVIK;
+            WRITE_LOCKINFO_OVERHEAD = WRITE_LOCKINFO_OVERHEAD_DALVIK;
+            TXN_OVERHEAD = TXN_OVERHEAD_DALVIK;
+            CHECKPOINT_REFERENCE_SIZE = CHECKPOINT_REFERENCE_SIZE_DALVIK;
+            UTILIZATION_PROFILE_ENTRY = UTILIZATION_PROFILE_ENTRY_DALVIK;
+            DBFILESUMMARY_OVERHEAD = DBFILESUMMARY_OVERHEAD_DALVIK;
+            TFS_LIST_INITIAL_OVERHEAD = TFS_LIST_INITIAL_OVERHEAD_DALVIK;
+            TFS_LIST_SEGMENT_OVERHEAD = TFS_LIST_SEGMENT_OVERHEAD_DALVIK;
+            LN_INFO_OVERHEAD = LN_INFO_OVERHEAD_DALVIK;
+            FILESUMMARYLN_OVERHEAD = FILESUMMARYLN_OVERHEAD_DALVIK;
+            LONG_LIST_PER_ITEM_OVERHEAD = LONG_LIST_PER_ITEM_OVERHEAD_DALVIK;
         } else {
-            LONG_OVERHEAD = LONG_OVERHEAD_32;
-            ARRAY_OVERHEAD = ARRAY_OVERHEAD_32;
-            ARRAY_SIZE_INCLUDED = ARRAY_SIZE_INCLUDED_32;
-            OBJECT_OVERHEAD = OBJECT_OVERHEAD_32;
-            OBJECT_ARRAY_ITEM_OVERHEAD = OBJECT_ARRAY_ITEM_OVERHEAD_32;
-            HASHMAP_OVERHEAD = HASHMAP_OVERHEAD_32;
-            HASHMAP_ENTRY_OVERHEAD = HASHMAP_ENTRY_OVERHEAD_32;
-            HASHSET_OVERHEAD = HASHSET_OVERHEAD_32;
-            HASHSET_ENTRY_OVERHEAD = HASHSET_ENTRY_OVERHEAD_32;
-            TWOHASHMAPS_OVERHEAD = TWOHASHMAPS_OVERHEAD_32;
-            if (isJVM15) {
-                TREEMAP_OVERHEAD = TREEMAP_OVERHEAD_32_15;
-                MAPLN_OVERHEAD = MAPLN_OVERHEAD_32_15;
+            String javaVersion = System.getProperty("java.version");
+            boolean isJVM15 = javaVersion != null &&
+                javaVersion.startsWith("1.5.");
+
+            boolean is64 = false;
+            String overrideArch = System.getProperty(FORCE_JVM_ARCH);
+            try {
+                if (overrideArch == null) {
+                    String arch = System.getProperty(JVM_ARCH_PROPERTY);
+                    if (arch != null) {
+                        is64 = Integer.parseInt(arch) == 64;
+                    }
+                } else {
+                    is64 = Integer.parseInt(overrideArch) == 64;
+                }
+            } catch (NumberFormatException NFE) {
+                NFE.printStackTrace(System.err);
+            }
+
+            if (is64) {
+                LONG_OVERHEAD = LONG_OVERHEAD_64;
+                ARRAY_OVERHEAD = ARRAY_OVERHEAD_64;
+                ARRAY_SIZE_INCLUDED = ARRAY_SIZE_INCLUDED_64;
+                OBJECT_OVERHEAD = OBJECT_OVERHEAD_64;
+                OBJECT_ARRAY_ITEM_OVERHEAD = OBJECT_ARRAY_ITEM_OVERHEAD_64;
+                HASHMAP_ENTRY_OVERHEAD = HASHMAP_ENTRY_OVERHEAD_64;
+                HASHSET_OVERHEAD = HASHSET_OVERHEAD_64;
+                HASHSET_ENTRY_OVERHEAD = HASHSET_ENTRY_OVERHEAD_64;
+                if (isJVM15) {
+                    TREEMAP_OVERHEAD = TREEMAP_OVERHEAD_64_15;
+                    MAPLN_OVERHEAD = MAPLN_OVERHEAD_64_15;
+                    BIN_FIXED_OVERHEAD = BIN_FIXED_OVERHEAD_64_15;
+                    DIN_FIXED_OVERHEAD = DIN_FIXED_OVERHEAD_64_15;
+                    DBIN_FIXED_OVERHEAD = DBIN_FIXED_OVERHEAD_64_15;
+                    IN_FIXED_OVERHEAD = IN_FIXED_OVERHEAD_64_15;
+                    HASHMAP_OVERHEAD = HASHMAP_OVERHEAD_64_15;
+                    TWOHASHMAPS_OVERHEAD = TWOHASHMAPS_OVERHEAD_64_15;
+                } else {
+                    TREEMAP_OVERHEAD = TREEMAP_OVERHEAD_64_16;
+                    MAPLN_OVERHEAD = MAPLN_OVERHEAD_64_16;
+                    BIN_FIXED_OVERHEAD = BIN_FIXED_OVERHEAD_64_16;
+                    DIN_FIXED_OVERHEAD = DIN_FIXED_OVERHEAD_64_16;
+                    DBIN_FIXED_OVERHEAD = DBIN_FIXED_OVERHEAD_64_16;
+                    IN_FIXED_OVERHEAD = IN_FIXED_OVERHEAD_64_16;
+                    HASHMAP_OVERHEAD = HASHMAP_OVERHEAD_64_16;
+                    TWOHASHMAPS_OVERHEAD = TWOHASHMAPS_OVERHEAD_64_16;
+                }
+                TREEMAP_ENTRY_OVERHEAD = TREEMAP_ENTRY_OVERHEAD_64;
+                LN_OVERHEAD = LN_OVERHEAD_64;
+                DUPCOUNTLN_OVERHEAD = DUPCOUNTLN_OVERHEAD_64;
+                TXN_OVERHEAD = TXN_OVERHEAD_64;
+                CHECKPOINT_REFERENCE_SIZE = CHECKPOINT_REFERENCE_SIZE_64;
+                KEY_OVERHEAD = KEY_OVERHEAD_64;
+                LOCKIMPL_OVERHEAD = LOCKIMPL_OVERHEAD_64;
+                THINLOCKIMPL_OVERHEAD = THINLOCKIMPL_OVERHEAD_64;
+                LOCKINFO_OVERHEAD = LOCKINFO_OVERHEAD_64;
+                WRITE_LOCKINFO_OVERHEAD = WRITE_LOCKINFO_OVERHEAD_64;
+                UTILIZATION_PROFILE_ENTRY = UTILIZATION_PROFILE_ENTRY_64;
+                DBFILESUMMARY_OVERHEAD = DBFILESUMMARY_OVERHEAD_64;
+                TFS_LIST_INITIAL_OVERHEAD = TFS_LIST_INITIAL_OVERHEAD_64;
+                TFS_LIST_SEGMENT_OVERHEAD = TFS_LIST_SEGMENT_OVERHEAD_64;
+                LN_INFO_OVERHEAD = LN_INFO_OVERHEAD_64;
+                FILESUMMARYLN_OVERHEAD = FILESUMMARYLN_OVERHEAD_64;
+                LONG_LIST_PER_ITEM_OVERHEAD = LONG_LIST_PER_ITEM_OVERHEAD_64;
             } else {
-                TREEMAP_OVERHEAD = TREEMAP_OVERHEAD_32_16;
-                MAPLN_OVERHEAD = MAPLN_OVERHEAD_32_16;
+                LONG_OVERHEAD = LONG_OVERHEAD_32;
+                ARRAY_OVERHEAD = ARRAY_OVERHEAD_32;
+                ARRAY_SIZE_INCLUDED = ARRAY_SIZE_INCLUDED_32;
+                OBJECT_OVERHEAD = OBJECT_OVERHEAD_32;
+                OBJECT_ARRAY_ITEM_OVERHEAD = OBJECT_ARRAY_ITEM_OVERHEAD_32;
+                HASHMAP_OVERHEAD = HASHMAP_OVERHEAD_32;
+                HASHMAP_ENTRY_OVERHEAD = HASHMAP_ENTRY_OVERHEAD_32;
+                HASHSET_OVERHEAD = HASHSET_OVERHEAD_32;
+                HASHSET_ENTRY_OVERHEAD = HASHSET_ENTRY_OVERHEAD_32;
+                TWOHASHMAPS_OVERHEAD = TWOHASHMAPS_OVERHEAD_32;
+                if (isJVM15) {
+                    TREEMAP_OVERHEAD = TREEMAP_OVERHEAD_32_15;
+                    MAPLN_OVERHEAD = MAPLN_OVERHEAD_32_15;
+                } else {
+                    TREEMAP_OVERHEAD = TREEMAP_OVERHEAD_32_16;
+                    MAPLN_OVERHEAD = MAPLN_OVERHEAD_32_16;
+                }
+                TREEMAP_ENTRY_OVERHEAD = TREEMAP_ENTRY_OVERHEAD_32;
+                LN_OVERHEAD = LN_OVERHEAD_32;
+                DUPCOUNTLN_OVERHEAD = DUPCOUNTLN_OVERHEAD_32;
+                BIN_FIXED_OVERHEAD = BIN_FIXED_OVERHEAD_32;
+                DIN_FIXED_OVERHEAD = DIN_FIXED_OVERHEAD_32;
+                DBIN_FIXED_OVERHEAD = DBIN_FIXED_OVERHEAD_32;
+                IN_FIXED_OVERHEAD = IN_FIXED_OVERHEAD_32;
+                TXN_OVERHEAD = TXN_OVERHEAD_32;
+                CHECKPOINT_REFERENCE_SIZE = CHECKPOINT_REFERENCE_SIZE_32;
+                KEY_OVERHEAD = KEY_OVERHEAD_32;
+                LOCKIMPL_OVERHEAD = LOCKIMPL_OVERHEAD_32;
+                THINLOCKIMPL_OVERHEAD = THINLOCKIMPL_OVERHEAD_32;
+                LOCKINFO_OVERHEAD = LOCKINFO_OVERHEAD_32;
+                WRITE_LOCKINFO_OVERHEAD = WRITE_LOCKINFO_OVERHEAD_32;
+                UTILIZATION_PROFILE_ENTRY = UTILIZATION_PROFILE_ENTRY_32;
+                DBFILESUMMARY_OVERHEAD = DBFILESUMMARY_OVERHEAD_32;
+                TFS_LIST_INITIAL_OVERHEAD = TFS_LIST_INITIAL_OVERHEAD_32;
+                TFS_LIST_SEGMENT_OVERHEAD = TFS_LIST_SEGMENT_OVERHEAD_32;
+                LN_INFO_OVERHEAD = LN_INFO_OVERHEAD_32;
+                FILESUMMARYLN_OVERHEAD = FILESUMMARYLN_OVERHEAD_32;
+                LONG_LIST_PER_ITEM_OVERHEAD = LONG_LIST_PER_ITEM_OVERHEAD_32;
             }
-            TREEMAP_ENTRY_OVERHEAD = TREEMAP_ENTRY_OVERHEAD_32;
-            LN_OVERHEAD = LN_OVERHEAD_32;
-            DUPCOUNTLN_OVERHEAD = DUPCOUNTLN_OVERHEAD_32;
-            BIN_FIXED_OVERHEAD = BIN_FIXED_OVERHEAD_32;
-            DIN_FIXED_OVERHEAD = DIN_FIXED_OVERHEAD_32;
-            DBIN_FIXED_OVERHEAD = DBIN_FIXED_OVERHEAD_32;
-            IN_FIXED_OVERHEAD = IN_FIXED_OVERHEAD_32;
-            TXN_OVERHEAD = TXN_OVERHEAD_32;
-            CHECKPOINT_REFERENCE_SIZE = CHECKPOINT_REFERENCE_SIZE_32;
-            KEY_OVERHEAD = KEY_OVERHEAD_32;
-            LOCKIMPL_OVERHEAD = LOCKIMPL_OVERHEAD_32;
-            THINLOCKIMPL_OVERHEAD = THINLOCKIMPL_OVERHEAD_32;
-            LOCKINFO_OVERHEAD = LOCKINFO_OVERHEAD_32;
-            WRITE_LOCKINFO_OVERHEAD = WRITE_LOCKINFO_OVERHEAD_32;
-            UTILIZATION_PROFILE_ENTRY = UTILIZATION_PROFILE_ENTRY_32;
-            DBFILESUMMARY_OVERHEAD = DBFILESUMMARY_OVERHEAD_32;
-            TFS_LIST_INITIAL_OVERHEAD = TFS_LIST_INITIAL_OVERHEAD_32;
-            TFS_LIST_SEGMENT_OVERHEAD = TFS_LIST_SEGMENT_OVERHEAD_32;
-            LN_INFO_OVERHEAD = LN_INFO_OVERHEAD_32;
-            FILESUMMARYLN_OVERHEAD = FILESUMMARYLN_OVERHEAD_32;
-            LONG_LIST_PER_ITEM_OVERHEAD = LONG_LIST_PER_ITEM_OVERHEAD_32;
         }
     }
 
@@ -390,7 +458,7 @@ public class MemoryBudget implements EnvConfigObserver {
     public final static long MIN_MAX_MEMORY_SIZE = 96 * 1024;
     public final static String MIN_MAX_MEMORY_SIZE_STRING =
         Long.toString(MIN_MAX_MEMORY_SIZE);
- 
+
     /* This value prevents cache churn for apps with a high write rate. */
     @SuppressWarnings("unused")
     private final static int DEFAULT_MIN_BTREE_CACHE_SIZE = 500 * 1024;
@@ -440,7 +508,7 @@ public class MemoryBudget implements EnvConfigObserver {
      * Amount of memory cached for locks. Protected by the
      * LockManager.lockTableLatches[lockTableIndex].
      */
-    private long[] lockMemoryUsage;
+    private AtomicLong[] lockMemoryUsage;
 
     /*
      * Memory available to JE, based on je.maxMemory and the memory available
@@ -453,11 +521,11 @@ public class MemoryBudget implements EnvConfigObserver {
 
     /* Maximum allowed use of the admin budget by the UtilizationTracker. */
     private long trackerBudget;
-    
+
     /* Mininum to prevent cache churn. */
     private long minTreeMemoryUsage;
-    
-    /* 
+
+    /*
      * Overheads that are a function of node capacity.
      */
     private long inOverhead;
@@ -631,7 +699,7 @@ public class MemoryBudget implements EnvConfigObserver {
          * reduce the log buffer budget again.
          */
         long newLogBufferBudget =
-            configManager.getLong(EnvironmentParams.LOG_MEM_SIZE);      
+            configManager.getLong(EnvironmentParams.LOG_MEM_SIZE);
         if (newLogBufferBudget == 0) {
             if (EnvironmentImpl.IS_DALVIK) {
                 /* If Dalvik JVM, use 1/128th instead of 1/16th of cache. */
@@ -674,9 +742,9 @@ public class MemoryBudget implements EnvConfigObserver {
 
         long newMinTreeMemoryUsage = Math.min
             (configManager.getLong(EnvironmentParams.MIN_TREE_MEMORY),
-             myCachePortion - newLogBufferBudget); 
+             myCachePortion - newLogBufferBudget);
 
-        /* 
+        /*
          * If all has gone well, update the budget fields.  Once the log buffer
          * budget is determined, the remainder of the memory is left for tree
          * nodes.
@@ -687,7 +755,10 @@ public class MemoryBudget implements EnvConfigObserver {
         if (lockMemoryUsage == null) {
             nLockTables =
                 configManager.getInt(EnvironmentParams.N_LOCK_TABLES);
-            lockMemoryUsage = new long[nLockTables];
+            lockMemoryUsage = new AtomicLong[nLockTables];
+            for (int i = 0; i < nLockTables; i++) {
+                lockMemoryUsage[i] = new AtomicLong(0);
+            }
         }
         minTreeMemoryUsage = newMinTreeMemoryUsage;
 
@@ -728,7 +799,7 @@ public class MemoryBudget implements EnvConfigObserver {
      * and txns yet, and the items in the admin category are cleaner items and
      * aren't affected by the recovery splicing process.
      */
-    void initCacheMemoryUsage(long dbTreeAdminMemory) 
+    void initCacheMemoryUsage(long dbTreeAdminMemory)
         throws DatabaseException {
 
         long totalTree = 0;
@@ -816,17 +887,17 @@ public class MemoryBudget implements EnvConfigObserver {
      * @param increment note that increment may be negative.
      */
     public void updateTreeAdminMemoryUsage(long increment) {
-        updateCounter(increment, treeAdminMemoryUsage, "treeAdmin", 
-                      DEBUG_TREEADMIN); 
+        updateCounter(increment, treeAdminMemoryUsage, "treeAdmin",
+                      DEBUG_TREEADMIN);
     }
 
-    private void updateCounter(long increment, 
+    private void updateCounter(long increment,
                                AtomicLong counter,
                                String debugName,
                                boolean debug) {
         if (increment != 0) {
             long newSize = counter.addAndGet(increment);
-            
+
             assert (sizeNotNegative(newSize)) :
                    makeErrorMessage(debugName, newSize, increment);
 
@@ -857,22 +928,22 @@ public class MemoryBudget implements EnvConfigObserver {
 
     public void updateLockMemoryUsage(long increment, int lockTableIndex) {
         if (increment != 0) {
-            lockMemoryUsage[lockTableIndex] += increment;
+            lockMemoryUsage[lockTableIndex].addAndGet(increment);
 
-            assert lockMemoryUsage[lockTableIndex] >= 0:
+            assert lockMemoryUsage[lockTableIndex].get() >= 0:
                    makeErrorMessage("lockMem",
-                                     lockMemoryUsage[lockTableIndex],
-                                     increment);
+                                    lockMemoryUsage[lockTableIndex].get(),
+                                    increment);
             if (DEBUG_LOCK) {
                 if (increment > 0) {
                     System.err.println("INC-------- =" + increment +
-                            " lock[" +
-                                      lockTableIndex + "] " +
-                                      lockMemoryUsage[lockTableIndex]);
+                                       " lock[" +
+                                       lockTableIndex + "] " +
+                                       lockMemoryUsage[lockTableIndex].get());
                 } else {
                     System.err.println("-------DEC=" + increment +
-                            " lock[" + lockTableIndex + "] " +
-                                           lockMemoryUsage[lockTableIndex]);
+                                       " lock[" + lockTableIndex + "] " +
+                                       lockMemoryUsage[lockTableIndex].get());
                 }
             }
 
@@ -915,10 +986,10 @@ public class MemoryBudget implements EnvConfigObserver {
     public long getLockMemoryUsage() {
         long accLockMemoryUsage = txnMemoryUsage.get();
         if (nLockTables == 1) {
-            accLockMemoryUsage += lockMemoryUsage[0];
+            accLockMemoryUsage += lockMemoryUsage[0].get();
         } else {
             for (int i = 0; i < nLockTables; i++) {
-                accLockMemoryUsage += lockMemoryUsage[i];
+                accLockMemoryUsage += lockMemoryUsage[i].get();
             }
         }
 
@@ -950,7 +1021,7 @@ public class MemoryBudget implements EnvConfigObserver {
     public long getAdminMemoryUsage() {
         return adminMemoryUsage.get();
     }
-    
+
     /*
      * For unit testing
      */
@@ -1027,7 +1098,7 @@ public class MemoryBudget implements EnvConfigObserver {
         stats.setLockBytes(getLockMemoryUsage());
     }
 
-    @Override 
+    @Override
     public String toString() {
         StringBuilder sb = new StringBuilder();
         sb.append("treeUsage = ").append(treeMemoryUsage.get());
