@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: SortedLSNTreeWalker.java,v 1.37 2008/05/13 17:09:00 mark Exp $
+ * $Id: SortedLSNTreeWalker.java,v 1.37.2.4 2010/03/23 15:02:07 cwl Exp $
  */
 
 package com.sleepycat.je.dbi;
@@ -219,8 +219,11 @@ public class SortedLSNTreeWalker {
 	    root = getRootIN(rootLsn);
 	}
 	if (root != null) {
-	    accumulateLSNs(root);
-	    releaseRootIN(root);
+            try {
+                accumulateLSNs(root);
+            } finally {
+                releaseRootIN(root);
+            }
 	}
 
         if (setDbState) {
@@ -231,7 +234,6 @@ public class SortedLSNTreeWalker {
 	    maybeGetMoreINs();
 	    if (currentLSNs != null &&
 		currentLSNIdx < currentLSNs.length) {
-		didSomeMore = true;
                 fetchAndProcessLSN(currentLSNs[currentLSNIdx++]);
 	    } else {
 		break;
@@ -239,8 +241,6 @@ public class SortedLSNTreeWalker {
 	}
     }
 
-    public boolean didSomeMore = false;
-
     private void maybeGetMoreINs() {
 
 	if ((currentLSNs != null &&
