@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: Evictor.java,v 1.114 2008/05/20 03:27:35 linda Exp $
+ * $Id: Evictor.java,v 1.115.2.6 2010/01/30 01:10:55 mark Exp $
  */
 
 package com.sleepycat.je.evictor;
@@ -41,6 +41,7 @@ import com.sleepycat.je.utilint.DaemonThread;
 import com.sleepycat.je.utilint.DbLsn;
 import com.sleepycat.je.utilint.TestHook;
 import com.sleepycat.je.utilint.Tracer;
+import com.sleepycat.je.utilint.TestHookExecute;
 
 /**
  * The Evictor looks through the INList for IN's and BIN's that are worthy of
@@ -117,6 +118,7 @@ public abstract class Evictor extends DaemonThread {
     /* Debugging and unit test support. */
     EvictProfile evictProfile;
     private TestHook runnableHook;
+    private TestHook preEvictINHook;
 
     Evictor(EnvironmentImpl envImpl, String name)
         throws DatabaseException {
@@ -172,6 +174,7 @@ public abstract class Evictor extends DaemonThread {
     /**
      * Return the number of retries when a deadlock exception occurs.
      */
+    @Override
     protected long nDeadlockRetries()
         throws DatabaseException {
 
@@ -514,13 +517,10 @@ public abstract class Evictor extends DaemonThread {
             }
 
             /*
-             * If this is a read only database and we have at least one
-             * target, skip any dirty INs (recovery dirties INs even in a
-             * read-only environment). We take at least one target so we
-             * don't loop endlessly if everything is dirty.
+             * If this is a read-only environment, skip any dirty INs (recovery
+             * dirties INs even in a read-only environment).
              */
             if (db.getDbEnvironment().isReadOnly() &&
-                target != null &&
                 in.getDirty()) {
                 continue;
             }
@@ -648,12 +648,14 @@ public abstract class Evictor extends DaemonThread {
                 rootIN.latch(CacheMode.UNCHANGED);
                 try {
                     /* Re-check that all conditions still hold. */
+                    boolean isDirty = rootIN.getDirty();
                     if (rootIN == target &&
                         rootIN.isDbRoot() &&
-                        rootIN.isEvictable()) {
+                        rootIN.isEvictable() &&
+                        !(envImpl.isReadOnly() && isDirty)) {
 
                         /* Flush if dirty. */
-                        if (!envImpl.isReadOnly() && rootIN.getDirty()) {
+                        if (isDirty) {
                             long newLsn = rootIN.log
                                 (envImpl.getLogManager(),
                                  false, // allowDeltas
@@ -774,6 +776,14 @@ public abstract class Evictor extends DaemonThread {
                     /* Regular eviction. */
                     Tree tree = db.getTree();
 
+                    /*
+                     * Unit testing.  The target is latched and we are about to
+                     * release that latch and search for the parent.  Make sure
+                     * that other operations, such as dirtying an LN in the
+                     * target BIN, can occur safely in this window.  [#18227]
+                     */
+                    assert TestHookExecute.doHookIfSet(preEvictINHook);
+
                     /* getParentINForChildIN unlatches target. */
 		    targetIsLatched = false;
                     SearchResult result =
@@ -806,8 +816,7 @@ public abstract class Evictor extends DaemonThread {
     }
 
     /**
-     * Evict an IN. Dirty nodes are logged before they're evicted. inlist is
-     * latched with the major latch by the caller.
+     * Evict an IN. Dirty nodes are logged before they're evicted.
      */
     private long evictIN(IN child, IN parent, int index, boolean backgroundIO)
         throws DatabaseException {
@@ -827,7 +836,7 @@ public abstract class Evictor extends DaemonThread {
 
             /*
              * See the evictIN() method in this class for an explanation for
-             * calling latchNoWait(false).
+             * calling latchNoWait().
              */
             if ((renewedChild != null) &&
                 (renewedChild.getGeneration() <= oldGenerationCount) &&
@@ -946,6 +955,11 @@ public abstract class Evictor extends DaemonThread {
         runnableHook = hook;
     }
 
+    /* For unit testing only. */
+    public void setPreEvictINHook(TestHook hook) {
+        preEvictINHook = hook;
+    }
+
     /**
      * Standard daemon method to set envImpl to null.
      */
@@ -969,7 +983,7 @@ public abstract class Evictor extends DaemonThread {
     /**
      * Only supported by SharedEvictor.
      */
-    public abstract  boolean checkEnvs(Set<EnvironmentImpl> envs);
+    public abstract boolean checkEnvs(Set<EnvironmentImpl> envs);
 
     abstract Logger getLogger();
 
