@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: FileManager.java,v 1.193 2008/05/22 20:21:04 linda Exp $
+ * $Id: FileManager.java,v 1.194.2.9 2010/01/04 15:30:29 cwl Exp $
  */
 
 package com.sleepycat.je.log;
@@ -28,6 +28,7 @@ import java.util.Set;
 
 import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.EnvironmentStats;
+import com.sleepycat.je.EnvironmentLockedException;
 import com.sleepycat.je.RunRecoveryException;
 import com.sleepycat.je.StatsConfig;
 import com.sleepycat.je.config.EnvironmentParams;
@@ -163,14 +164,6 @@ public class FileManager {
      */
     private Map<Long, Long> perFileLastUsedLsn;
 
-    /* Whether to use NIO for file I/O. */
-    private boolean useNIO;
-
-    /*
-     * If non-0, do NIO in chunks of this size.
-     */
-    private long chunkedNIOSize = 0;
-
     /*
      * Use O_DSYNC to open JE log files.
      */
@@ -230,27 +223,11 @@ public class FileManager {
         DbConfigManager configManager = envImpl.getConfigManager();
         maxFileSize = configManager.getLong(EnvironmentParams.LOG_FILE_MAX);
 
-        useNIO =
-            configManager.getBoolean(EnvironmentParams.LOG_USE_NIO);
-        chunkedNIOSize =
-            configManager.getLong(EnvironmentParams.LOG_CHUNKED_NIO);
         useODSYNC =
             configManager.getBoolean(EnvironmentParams.LOG_USE_ODSYNC);
-        boolean directNIO =
-            configManager.getBoolean(EnvironmentParams.LOG_DIRECT_NIO);
         VERIFY_CHECKSUMS =
             configManager.getBoolean(EnvironmentParams.LOG_VERIFY_CHECKSUMS);
 
-        if (!useNIO && (chunkedNIOSize > 0 || directNIO)) {
-            throw new IllegalArgumentException
-                (EnvironmentParams.LOG_USE_NIO.getName() +
-                 " is false and therefore " +
-                 EnvironmentParams.LOG_DIRECT_NIO.getName() +
-                 " or " +
-                 EnvironmentParams.LOG_CHUNKED_NIO.getName() +
-                 " may not be used.");
-        }
-
         if (!envImpl.isMemOnly()) {
             if (!dbEnvHome.exists()) {
                 throw new LogException("Environment home " + dbEnvHome +
@@ -372,7 +349,9 @@ public class FileManager {
     }
 
     /**
-     * For unit tests.
+     * Returns the highest (current) file number.  Because a long value cannot
+     * be read atomically without synchronization, this method should be called
+     * while holding the log write latch.
      */
     public long getCurrentFileNum() {
         return currentFileNum;
@@ -862,7 +841,13 @@ public class FileManager {
                 fileName = fileNames[i];
                 try {
                     newFile =
-                        new RandomAccessFile(fileName, mode.getModeValue());
+                        new RandomAccessFile(fileName, mode.getModeValue()) {
+                            public synchronized long length()
+                                throws IOException {
+
+                                return super.length();
+                            }
+                        };
                     break;
                 } catch (FileNotFoundException e) {
                     /* Save the first exception thrown. */
@@ -1160,6 +1145,11 @@ public class FileManager {
                 endOfLog.getWritableFile(DbLsn.getFileNumber(firstLsn));
             ByteBuffer data = fullBuffer.getDataBuffer();
 
+            if (VERIFY_CHECKSUMS) {
+                verifyChecksums(data, DbLsn.getFileOffset(firstLsn),
+                                "pre-write");
+            }
+
             try {
 
                 /*
@@ -1215,7 +1205,7 @@ public class FileManager {
                      * it to disk on the write, then rewriting should allow
                      * them to be rewritten.  See [11271].
                      *
-                     * Rewriting committed transactions in replication is 
+                     * Rewriting committed transactions in replication is
                      * highly problematic, and can lead to divergence between
                      * the replica and master. If this path is re-enabled, we
                      * must assess its impact in replication.
@@ -1240,7 +1230,7 @@ public class FileManager {
     }
 
     /**
-     * Write a buffer to a file at a given offset, using NIO if so configured.
+     * Write a buffer to a file at a given offset.
      */
     private int writeToFile(RandomAccessFile file,
                             ByteBuffer data,
@@ -1249,95 +1239,46 @@ public class FileManager {
         throws IOException, DatabaseException {
 
         int totalBytesWritten = 0;
-        if (useNIO) {
-            FileChannel channel = file.getChannel();
 
-            if (chunkedNIOSize > 0) {
+        bumpWriteCount("write");
 
-                /*
-                 * We can't change the limit without impacting readers that
-                 * might find this buffer in the buffer pool.  Duplicate the
-                 * buffer so we can set the limit independently.
-                 */
-                ByteBuffer useData = data.duplicate();
+        /*
+         * Perform a RandomAccessFile write and update the buffer position.
+         * ByteBuffer.array() is safe to use since all non-direct ByteBuffers
+         * have a backing array.  Synchronization on the file object is needed
+         * because two threads may call seek() on the same file object.
+         */
+        synchronized (file) {
+            assert data.hasArray();
 
-                /*
-                 * Write small chunks of data by manipulating the position and
-                 * limit properties of the buffer, and submitting it for
-                 * writing repeatedly.
-                 *
-                 * For each chunk, the limit is set to the position +
-                 * chunkedNIOSize, capped by the original limit of the buffer.
-                 *
-                 * Preconditions: data to be written is betweek data.position()
-                 * and data.limit()
+            int pos = data.position();
+            int size = data.limit() - pos;
 
-                 * Postconditions: data.limit() has not changed,
-                 * data.position() == data.limit(), offset of the channel has
-                 * not been modified.
-                 */
-                int originalLimit = useData.limit();
-                useData.limit(useData.position());
-                while (useData.limit() < originalLimit) {
-                    bumpWriteCount("nio write");
-
-                    useData.limit((int)
-                                  (Math.min(useData.limit() + chunkedNIOSize,
-                                            originalLimit)));
-                    int bytesWritten = channel.write(useData, destOffset);
-                    destOffset += bytesWritten;
-                    totalBytesWritten += bytesWritten;
-                }
+            if (lastFileNumberTouched == fileNum &&
+                (Math.abs(destOffset - lastFileTouchedOffset) <
+                 ADJACENT_TRACK_SEEK_DELTA)) {
+                nSequentialWrites++;
+                nSequentialWriteBytes += size;
             } else {
-
-                /*
-                 * Perform a single write using NIO.
-                 */
-                totalBytesWritten = channel.write(data, destOffset);
+                nRandomWrites++;
+                nRandomWriteBytes += size;
             }
-        } else {
 
-            bumpWriteCount("write");
-
-            /*
-             * Perform a RandomAccessFile write and update the buffer position.
-             * ByteBuffer.array() is safe to use since all non-direct
-             * ByteBuffers have a backing array.  Synchronization on the file
-             * object is needed because two threads may call seek() on the same
-             * file object.
-             */
-            synchronized (file) {
-                assert data.hasArray();
-
-                int pos = data.position();
-                int size = data.limit() - pos;
-
-                if (lastFileNumberTouched == fileNum &&
-                    (Math.abs(destOffset - lastFileTouchedOffset) <
-                    ADJACENT_TRACK_SEEK_DELTA)) {
-                    nSequentialWrites++;
-                    nSequentialWriteBytes += size;
-                } else {
-                    nRandomWrites++;
-                    nRandomWriteBytes += size;
-                }
-
-                if (VERIFY_CHECKSUMS) {
-                    verifyChecksums(data, destOffset, "pre-write");
-                }
+            if (VERIFY_CHECKSUMS) {
+                verifyChecksums(data, destOffset, "pre-write");
+            }
+            file.seek(destOffset);
+            file.write(data.array(), pos + data.arrayOffset(), size);
+            if (VERIFY_CHECKSUMS) {
                 file.seek(destOffset);
-                file.write(data.array(), pos + data.arrayOffset(), size);
-                if (VERIFY_CHECKSUMS) {
-                    file.seek(destOffset);
-                    file.read(data.array(), pos + data.arrayOffset(), size);
-                    verifyChecksums(data, destOffset, "post-write");
-                }
-                data.position(pos + size);
-                totalBytesWritten = size;
-
-                lastFileNumberTouched = fileNum;
-                lastFileTouchedOffset = destOffset + size;
+                file.read(data.array(), pos + data.arrayOffset(), size);
+                verifyChecksums(data, destOffset, "post-write");
             }
+            data.position(pos + size);
+            totalBytesWritten = size;
+
+            lastFileNumberTouched = fileNum;
+            lastFileTouchedOffset = destOffset + size;
         }
         return totalBytesWritten;
     }
@@ -1362,7 +1303,7 @@ public class FileManager {
     }
 
     /**
-     * Read a buffer from a file at a given offset, using NIO if so configured.
+     * Read a buffer from a file at a given offset.
      */
     void readFromFile(RandomAccessFile file,
                       ByteBuffer readBuffer,
@@ -1387,82 +1328,41 @@ public class FileManager {
                                       long fileNum)
         throws DatabaseException, IOException {
 
-        if (useNIO) {
-            FileChannel channel = file.getChannel();
+        /*
+         * Perform a RandomAccessFile read and update the buffer position.
+         * ByteBuffer.array() is safe to use since all non-direct ByteBuffers
+         * have a backing array.  Synchronization on the file object is needed
+         * because two threads may call seek() on the same file object.
+         */
+        synchronized (file) {
+            assert readBuffer.hasArray();
 
-            if (chunkedNIOSize > 0) {
+            int pos = readBuffer.position();
+            int size = readBuffer.limit() - pos;
 
-                /*
-                 * Read a chunk at a time to prevent large direct memory
-                 * allocations by NIO.
-                 */
-                int readLength = readBuffer.limit();
-                long currentPosition = offset;
-                while (readBuffer.position() < readLength) {
-                    readBuffer.limit((int)
-                                     (Math.min(readBuffer.limit() +
-                                               chunkedNIOSize,
-                                               readLength)));
-                    if (IO_EXCEPTION_TESTING_ON_READ) {
-                        throw new IOException("generated for testing (read)");
-                    }
-                    int bytesRead = channel.read(readBuffer, currentPosition);
-
-                    if (bytesRead < 1)
-                        break;
-
-                    currentPosition += bytesRead;
-                }
+            if (lastFileNumberTouched == fileNum &&
+                (Math.abs(offset - lastFileTouchedOffset) <
+                 ADJACENT_TRACK_SEEK_DELTA)) {
+                nSequentialReads++;
+                nSequentialReadBytes += size;
             } else {
-
-                if (IO_EXCEPTION_TESTING_ON_READ) {
-                    throw new IOException("generated for testing (read)");
-                }
-
-                /*
-                 * Perform a single read using NIO.
-                 */
-                channel.read(readBuffer, offset);
+                nRandomReads++;
+                nRandomReadBytes += size;
             }
-        } else {
 
-            /*
-             * Perform a RandomAccessFile read and update the buffer position.
-             * ByteBuffer.array() is safe to use since all non-direct
-             * ByteBuffers have a backing array.  Synchronization on the file
-             * object is needed because two threads may call seek() on the same
-             * file object.
-             */
-            synchronized (file) {
-                assert readBuffer.hasArray();
-
-                int pos = readBuffer.position();
-                int size = readBuffer.limit() - pos;
-
-                if (lastFileNumberTouched == fileNum &&
-                    (Math.abs(offset - lastFileTouchedOffset) <
-                    ADJACENT_TRACK_SEEK_DELTA)) {
-                    nSequentialReads++;
-                    nSequentialReadBytes += size;
-                } else {
-                    nRandomReads++;
-                    nRandomReadBytes += size;
-                }
-
-                file.seek(offset);
-                if (IO_EXCEPTION_TESTING_ON_READ) {
-                    throw new IOException("generated for testing (read)");
-                }
-                int bytesRead = file.read(readBuffer.array(),
-                                          pos + readBuffer.arrayOffset(),
-                                          size);
-                if (bytesRead > 0) {
-                    readBuffer.position(pos + bytesRead);
-                }
-
-                lastFileNumberTouched = fileNum;
-                lastFileTouchedOffset = offset + bytesRead;
+            file.seek(offset);
+            if (IO_EXCEPTION_TESTING_ON_READ) {
+                throw new IOException("generated for testing (read)");
             }
+            int bytesRead = file.read(readBuffer.array(),
+                                      pos + readBuffer.arrayOffset(),
+                                      size);
+            if (bytesRead > 0) {
+                readBuffer.position(pos + bytesRead);
+            }
+
+            lastFileNumberTouched = fileNum;
+            lastFileTouchedOffset = offset + bytesRead;
         }
     }
 
@@ -1711,7 +1611,8 @@ public class FileManager {
                 throwIt = true;
             }
             if (throwIt) {
-                throw new LogException
+                close();
+                throw new EnvironmentLockedException
                     ("A " + LOCK_FILE + " file exists in " +
                      dbEnvHome.getAbsolutePath() +
                      " The environment can not be locked for " +
@@ -2129,7 +2030,24 @@ public class FileManager {
              * file has flipped to a new file since the fsync was issued.
              */
             synchronized (fsyncFileSynchronizer) {
-                RandomAccessFile file = endOfLogSyncFile;
+
+                /*
+                 * In [#17865] a user reported file corruption when using
+                 * Windows 7.  This SR includes a test program (JETester) which
+                 * readily demonstrates the problem.  We are reasonably sure
+                 * that the problem is caused by concurrent write() and fsync()
+                 * calls on the same file, and in particular, if two different
+                 * file descriptors are used concurrently.  The code below is a
+                 * workaround for this problem since it does not happen when we
+                 * use the same RAF for the write() and fsync().  In fact, the
+                 * test program has slightly better performance on the write
+                 * phase with the change below.  This problem is not present
+                 * in 4.0 because of the write queue code in that release.
+                 */
+                boolean isWin7 = EnvironmentImpl.IS_WINDOWS_7;
+                RandomAccessFile file =
+                    (isWin7 ? endOfLogRWFile : endOfLogSyncFile);
+
                 if (file != null) {
                     bumpWriteCount("fsync");
                     FileChannel channel = file.getChannel();
