@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: LogManager.java,v 1.201 2008/05/15 01:52:41 linda Exp $
+ * $Id: LogManager.java,v 1.203.2.4 2010/02/08 17:19:42 cwl Exp $
  */
 
 package com.sleepycat.je.log;
@@ -362,8 +362,8 @@ public abstract class LogManager {
              * of writing this mapping to disk.
              */
             if (item.repContext.inReplicationStream()) {
-                assert (item.header.getVLSN() != null) : 
-                    "Unexpected null vlsn: " + item.header + " " +
+                assert (item.header.getVLSN() != null) :
+                    "Unexpected null VLSN: " + item.header + " " +
                     item.repContext;
                 envImpl.getReplicator().registerVLSN(item.newLsn, item.header);
             }
@@ -504,7 +504,7 @@ public abstract class LogManager {
                 LogBuffer useLogBuffer =
                     logBufferPool.getWriteBuffer(entrySize, flippedFile);
 
-                /* Add checksum, prev offset, vlsn to entry. */
+                /* Add checksum, prev offset, and VLSN to the entry. */
                 item.buffer = item.header.addPostMarshallingInfo
                     (envImpl, item.buffer, fileManager.getPrevEntryOffset(),
                      item.repContext);
@@ -595,7 +595,7 @@ public abstract class LogManager {
      * Serialize a loggable object into this buffer. (public for
      * unit tests.
      */
-    public ByteBuffer marshallIntoBuffer(LogEntryHeader header, LogEntry entry)
+    private ByteBuffer marshallIntoBuffer(LogEntryHeader header, LogEntry entry)
         throws DatabaseException {
 
         int entrySize = header.getSize() + header.getItemSize();
@@ -623,6 +623,9 @@ public abstract class LogManager {
         LogEntryHeader header = new LogEntryHeader
             (entry, Provisional.NO, ReplicationContext.NO_REPLICATE);
 
+        /* Assert that we're not in a situation demanding Txn mutex. */
+        assert entry.getLogType() != LogEntryType.LOG_LN_TRANSACTIONAL;
+
         ByteBuffer destBuffer = marshallIntoBuffer(header, entry);
 
         return header.addPostMarshallingInfo(envImpl,
@@ -1028,7 +1031,7 @@ public abstract class LogManager {
     /**
      * Counts the given obsolete IN LSNs under the log write latch.
      */
-    public abstract void countObsoleteINs(List<Long> lsnList, 
+    public abstract void countObsoleteINs(List<Long> lsnList,
                                           DatabaseImpl nodeDb)
         throws DatabaseException;
 
@@ -1067,7 +1070,7 @@ public abstract class LogManager {
 
     void loadEndOfLogStatInternal(EnvironmentStats stats) {
 	stats.setEndOfLog(fileManager.getLastUsedLsn());
-    }	
+    }
 
     /* For unit testing only. */
     public void setReadHook(TestHook hook) {
