@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: Checkpointer.java,v 1.169 2008/05/23 18:50:49 mark Exp $
+ * $Id: Checkpointer.java,v 1.170.2.5 2010/03/26 01:52:07 mark Exp $
  */
 
 package com.sleepycat.je.recovery;
@@ -74,6 +74,8 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
      */
     public static TestHook maxFlushLevelHook = null;
 
+    public static TestHook beforeFlushHook = null;
+
     private EnvironmentImpl envImpl;
 
     /* Checkpoint sequence, initialized at recovery. */
@@ -234,6 +236,7 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
     /**
      * Return the number of retries when a deadlock exception occurs.
      */
+    @Override
     protected long nDeadlockRetries()
         throws DatabaseException {
 
@@ -468,6 +471,9 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
              */
             dirtyMap.addCostToMemoryBudget();
 
+            /* Call hook after dirty map creation and before flushing. */
+            TestHookExecute.doHookIfSet(beforeFlushHook);
+
             /* Flush IN nodes. */
             boolean allowDeltas = !config.getMinimizeRecoveryTime();
             flushDirtyNodes(envImpl, dirtyMap, highestFlushLevels, allowDeltas,
@@ -616,6 +622,11 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
         maxFlushLevelHook = hook;
     }
 
+    /* For unit testing only. */
+    public static void setBeforeFlushHook(TestHook hook) {
+        beforeFlushHook = hook;
+    }
+
     /**
      * Flush the nodes in order, from the lowest level to highest level.  As a
      * flush dirties its parent, add it to the dirty map, thereby cascading the
@@ -759,22 +770,22 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
      *
      * If this is a high priority checkpoint, we'll hold the INb latch across
      * the time it takes to flush all three children.  In flushIN(BINd), we
-     * walk through INa, create a local map of all the siblings that can be
+     * walk through INb, create a local map of all the siblings that can be
      * found in the dirty map, and then call logSiblings with that local map.
-     * Then we'll write out INa.
+     * Then we'll write out INb.
      *
      * If high priority is false, we will not hold the INb latch across
      * multiple IOs. Instead, we 
      *  - write BINd out, using logSiblings
-     *  - while still holding the INa latch, we create a local map of dirty
+     *  - while still holding the INb latch, we create a local map of dirty
      *    siblings
-     *  - release the INa latch
+     *  - release the INb latch
      *  - call flushIN() recursively on each entry in the local sibling map, 
      *    which will result in a search and write of each sibling.  These
      *    recursive calls to flushIN are called with the allowLogSubtree
      *    parameter of false to halt the recursion and prevent a repeat of the
      *    sibling examination.
-     *  - write INa
+     *  - write INb
      */
     private static void flushIN(EnvironmentImpl envImpl,
                                 DatabaseImpl db,
@@ -862,6 +873,7 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
              */
             if (result.parent != null) {
                 IN parent = result.parent;
+                int parentLevel = parent.getLevel();
                 boolean mustLogParent = false;
 
                 /*
@@ -874,17 +886,25 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
                  * tree DB.
                  */
                 boolean bottomLevelTarget = db.getSortedDuplicates() ?
-                    (parent.getLevel() == 2) :
-                    ((parent.getLevel() & IN.LEVEL_MASK) == 2);
+                    (parentLevel == 2) :
+                    ((parentLevel & IN.LEVEL_MASK) == 2);
 
                 /*
                  * INs at the max flush level are always non-provisional and
                  * INs at the bottom level (when this is not also the max flush
                  * level) are always provisional.  In between INs are
                  * provisional BEFORE_CKPT_END (see Provisional).
+                 *
+                 * Note that to determine whether this IN is at the
+                 * maxFlushLevel, we check (parentLevel > maxFlushLevel)
+                 * instead of (currentLevel >= maxFlushLevel).  This handles
+                 * the case where this IN is a DIN root, and the parent is a
+                 * BIN that will not be flushed because the maxFlushLevel is
+                 * less than IN.MAIN_LEVEL (0x10000).  For example, this IN is
+                 * a DIN root at level 2 and the maxFlushLevel is 3.  [#16712]
                  */
                 Provisional provisional;
-                if (currentLevel >= maxFlushLevel) {
+                if (parentLevel > maxFlushLevel) {
                     provisional = Provisional.NO;
                 } else if (bottomLevelTarget) {
                     provisional = Provisional.YES;
@@ -969,7 +989,7 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
                              * the non-exact search does not find a sibling
                              * rather than a parent. [#11555]
                              */
-                            if (parent.getLevel() > currentLevel) {
+                            if (parentLevel > currentLevel) {
                                 mustLogParent |= true;
                             }
                             /* nAlreadyEvictedThisRun++; -- for future. */
@@ -984,6 +1004,11 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
                          * does not contain targetRef (the sibling we're
                          * processing) because it was removed before calling
                          * this method, but it is added to the map below.
+                         *
+                         * A TreeMap (sorted map) is used so that siblings are
+                         * latched in node ID order.  A deterministic order is
+                         * needed to avoid deadlocks, if siblings are latched
+                         * in multiple threads in the future.
                          */
                         siblingsToLog = new TreeMap<Long,Integer>();
                         for (int index = 0;
@@ -1037,8 +1062,13 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
                                dumpParentChildInfo(result, parent,
                                                    targetRef.nodeId,
                                                    currentLevel, tree);
-                        dirtyMap.addDirtyIN
-                            (parent, true /* updateMemoryBudget */);
+                        /*
+                         * Add the parent IN to the dirty map unconditionally,
+                         * even if not dirty, to cause changes to propogate
+                         * upward even when a node has been evicted and
+                         * refetched and is no longer dirty. [#16523]
+                         */
+                        dirtyMap.addIN(parent, true /*updateMemoryBudget*/);
                     }
                 } finally {
                     parent.releaseLatch();
@@ -1075,13 +1105,13 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
                  * latch above, since we must search and acquire the
                  * grandparent latch.
                  */
-                if (logSubtree && parent.getLevel() <= maxFlushLevel) {
+                if (logSubtree && parentLevel <= maxFlushLevel) {
                     CheckpointReference parentRef = dirtyMap.removeNode
-                        (parent.getLevel(), parent.getNodeId());
+                        (parentLevel, parent.getNodeId());
                     if (parentRef != null) {
                         flushIN
                             (envImpl, db, logManager, parentRef, dirtyMap,
-                             parent.getLevel(), maxFlushLevel, allowDeltas,
+                             parentLevel, maxFlushLevel, allowDeltas,
                              checkpointStart, highPriority, fstats,
                              localTracker, false /*allowLogSubtree*/);
                     }
@@ -1185,9 +1215,13 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
                 /* Remove it from dirty map if it is present. */
                 dirtyMap.removeNode(child.getLevel(), child.getNodeId());
 
-                /* Latch and add item so we will release the latch below. */
+                /*
+                 * Latch and add item with valid parentIndex, so we will
+                 * release the latch in the finally statement.
+                 */
                 child.latch(CacheMode.UNCHANGED);
                 INLogItem item = new INLogItem();
+                item.parentIndex = index;
                 itemList.add(item);
 
                 /*
@@ -1222,7 +1256,6 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
                     item.provisional = provisional;
                     item.repContext = ReplicationContext.NO_REPLICATE;
                     item.parent = parent;
-                    item.parentIndex = index;
 
                     /*
                      * Allow child to perform "before log" processing.  Note
@@ -1408,6 +1441,7 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
             this.dupTreeKey = dupTreeKey;
         }
 
+        @Override
         public boolean equals(Object o) {
             if (!(o instanceof CheckpointReference)) {
                 return false;
@@ -1417,6 +1451,7 @@ public class Checkpointer extends DaemonThread implements EnvConfigObserver {
             return nodeId == other.nodeId;
         }
 
+        @Override
         public int hashCode() {
             return (int) nodeId;
         }
