@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: DirtyINMap.java,v 1.17 2008/05/15 01:52:42 linda Exp $
+ * $Id: DirtyINMap.java,v 1.17.2.3 2010/01/04 15:30:32 cwl Exp $
  */
 
 package com.sleepycat.je.recovery;
@@ -86,8 +86,8 @@ class DirtyINMap {
                         continue;
                     }
 
-                    Integer level = addDirtyIN
-                        (in, false /* updateMemoryBudget */);
+                    Integer level =
+                        addDirtyIN(in, false /*updateMemoryBudget*/);
                     if (level != null) {
 
                         /*
@@ -172,7 +172,7 @@ class DirtyINMap {
             if (in.getDatabaseId().equals(dbId)) {
                 in.latch(CacheMode.UNCHANGED);
                 try {
-                    addDirtyIN(in, false /* updateMemoryBudget */);
+                    addDirtyIN(in, false /*updateMemoryBudget*/);
                 } finally {
                     in.releaseLatch();
                 }
@@ -208,49 +208,57 @@ class DirtyINMap {
     }
 
     /**
-     * Add a node to the dirty map. The dirty map is keyed by level (Integers)
-     * and holds sets of IN references.
+     * Adds the IN if dirty, otherwise returns null.  See addIN.
+     */
+    private Integer addDirtyIN(IN in, boolean updateMemoryBudget) {
+
+        if (in.getDirty()) {
+            return addIN(in, updateMemoryBudget);
+        } else {
+            return null;
+        }
+    }
+
+
+    /**
+     * Add a node unconditionally to the dirty map. The dirty map is keyed by
+     * level (Integers) and holds sets of IN references.
      *
      * @param updateMemoryBudget if true then update the memory budget as the
      * map is changed; if false then addCostToMemoryBudget must be called
      * later.
      *
-     * @return level of IN added to the dirty map, or null if the IN was not
-     * added to the dirty map.  The level is returned rather than a boolean
-     * simply to avoid allocating another Integer in the caller.
+     * @return level of IN added to the dirty map.  The level is returned
+     * rather than a boolean simply to avoid allocating another Integer in the
+     * caller.
      */
-    Integer addDirtyIN(IN in, boolean updateMemoryBudget) {
+    Integer addIN(IN in, boolean updateMemoryBudget) {
 
-        if (in.getDirty()) {
-
-            Integer level = Integer.valueOf(in.getLevel());
-            Map<Long,CheckpointReference> nodeMap;
-            if (levelMap.containsKey(level)) {
-                nodeMap = levelMap.get(level);
-            } else {
-                nodeMap = new HashMap<Long,CheckpointReference>();
-                levelMap.put(level, nodeMap);
-            }
-
-            nodeMap.put(in.getNodeId(),
-                        new CheckpointReference(in.getDatabase().getId(),
-                                                in.getNodeId(),
-                                                in.containsDuplicates(),
-                                                in.isDbRoot(),
-                                                in.getMainTreeKey(),
-                                                in.getDupTreeKey()));
-            numEntries++;
-
-            if (updateMemoryBudget) {
-                MemoryBudget mb = envImpl.getMemoryBudget();
-                mb.updateAdminMemoryUsage
-                    (MemoryBudget.CHECKPOINT_REFERENCE_SIZE);
-            }
-
-            return level;
+        Integer level = Integer.valueOf(in.getLevel());
+        Map<Long,CheckpointReference> nodeMap;
+        if (levelMap.containsKey(level)) {
+            nodeMap = levelMap.get(level);
         } else {
-            return null;
+            nodeMap = new HashMap<Long,CheckpointReference>();
+            levelMap.put(level, nodeMap);
         }
+
+        nodeMap.put(in.getNodeId(),
+                    new CheckpointReference(in.getDatabase().getId(),
+                                            in.getNodeId(),
+                                            in.containsDuplicates(),
+                                            in.isDbRoot(),
+                                            in.getMainTreeKey(),
+                                            in.getDupTreeKey()));
+        numEntries++;
+
+        if (updateMemoryBudget) {
+            MemoryBudget mb = envImpl.getMemoryBudget();
+            mb.updateAdminMemoryUsage
+                (MemoryBudget.CHECKPOINT_REFERENCE_SIZE);
+        }
+
+        return level;
     }
 
     /**
