@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: FileSummaryLN.java,v 1.33 2008/01/07 14:28:56 cwl Exp $
+ * $Id: FileSummaryLN.java,v 1.34.2.3 2010/01/04 15:30:36 cwl Exp $
  */
 
 package com.sleepycat.je.tree;
@@ -76,7 +76,7 @@ public final class FileSummaryLN extends LN {
     private static final String BEGIN_TAG = "<fileSummaryLN>";
     private static final String END_TAG = "</fileSummaryLN>";
 
-    private MemoryBudget memBudget;
+    private int extraMarshaledMemorySize;
     private FileSummary baseSummary;
     private TrackedFileSummary trackedSummary;
     private PackedOffsets obsoleteOffsets;
@@ -91,7 +91,6 @@ public final class FileSummaryLN extends LN {
         super(new byte[0],
               envImpl,   // envImpl
               false); // replicate
-        memBudget = envImpl.getMemoryBudget();
         assert baseSummary != null;
         this.baseSummary = baseSummary;
         obsoleteOffsets = new PackedOffsets();
@@ -238,13 +237,12 @@ public final class FileSummaryLN extends LN {
      * contains version 1 offsets that can be incorrect when RMW was used, and
      * if je.cleaner.rmwFix is enabled, discard the offsets.  [#13158]
      */
+    @Override
     public void postFetchInit(DatabaseImpl db, long sourceLsn)
         throws DatabaseException {
 
         super.postFetchInit(db, sourceLsn);
 
-        memBudget = db.getDbEnvironment().getMemoryBudget();
-
         if (entryVersion == 1 &&
             db.getDbEnvironment().getUtilizationProfile().isRMWFixEnabled()) {
             obsoleteOffsets = new PackedOffsets();
@@ -255,18 +253,22 @@ public final class FileSummaryLN extends LN {
      * Dumping
      */
 
+    @Override
     public String toString() {
         return dumpString(0, true);
     }
 
+    @Override
     public String beginTag() {
         return BEGIN_TAG;
     }
 
+    @Override
     public String endTag() {
         return END_TAG;
     }
 
+    @Override
     public String dumpString(int nSpaces, boolean dumpTags) {
         StringBuffer sb = new StringBuffer();
         sb.append(super.dumpString(nSpaces, dumpTags));
@@ -282,6 +284,7 @@ public final class FileSummaryLN extends LN {
      * Dump additional fields. Done this way so the additional info can
      * be within the XML tags defining the dumped log entry.
      */
+    @Override
     protected void dumpLogAdditional(StringBuffer sb, boolean verbose) {
         if (!isDeleted()) {
             baseSummary.dumpLog(sb, true);
@@ -298,6 +301,7 @@ public final class FileSummaryLN extends LN {
     /**
      * Log type for transactional entries.
      */
+    @Override
     protected LogEntryType getTransactionalLogType() {
         assert false : "Txnl access to UP db not allowed";
         return LogEntryType.LOG_FILESUMMARYLN;
@@ -306,6 +310,7 @@ public final class FileSummaryLN extends LN {
     /**
      * @see Node#getLogType
      */
+    @Override
     public LogEntryType getLogType() {
         return LogEntryType.LOG_FILESUMMARYLN;
     }
@@ -318,6 +323,7 @@ public final class FileSummaryLN extends LN {
      *
      * @see LN#getLogSize
      */
+    @Override
     public int getLogSize() {
         int size = super.getLogSize();
         if (!isDeleted()) {
@@ -331,6 +337,7 @@ public final class FileSummaryLN extends LN {
     /**
      * @see LN#writeToLog
      */
+    @Override
     public void writeToLog(ByteBuffer logBuffer) {
 
         /*
@@ -361,6 +368,7 @@ public final class FileSummaryLN extends LN {
     /**
      * @see LN#readFromLog
      */
+    @Override
     public void readFromLog(ByteBuffer itemBuffer, byte entryVersion)
         throws LogException {
 
@@ -391,13 +399,14 @@ public final class FileSummaryLN extends LN {
      * written to the log.
      */
     private void getOffsets() {
+        assert !isDeleted();
         if (needOffsets) {
             long[] offsets = trackedSummary.getObsoleteOffsets();
             if (offsets != null) {
                 int oldSize = obsoleteOffsets.getExtraMemorySize();
                 obsoleteOffsets.pack(offsets);
                 int newSize = obsoleteOffsets.getExtraMemorySize();
-                memBudget.updateTreeMemoryUsage(newSize - oldSize);
+                extraMarshaledMemorySize = newSize - oldSize;
             }
             needOffsets = false;
         }
@@ -422,4 +431,30 @@ public final class FileSummaryLN extends LN {
         super.makeDeleted();
         obsoleteOffsets = new PackedOffsets();
     }
+
+    /**
+     * Adds the extra memory used by obsoleteOffsets to the parent BIN memory
+     * size. Must be called after LN is inserted into the BIN and logged,
+     * while the cursor is still positioned on the inserted LN.  The BIN must
+     * be latched.  [#17462]
+     *
+     * <p>The obsoleteOffsets memory size is not intially budgeted in the usual
+     * way because PackedOffsets.pack (which changes the memory size) is called
+     * during marshalling (see getOffset).  This amount is not counted in the
+     * parent IN size in the usual way, because LN logging / marshalling occurs
+     * after the LN is inserted in the BIN and its memory size has been counted
+     * (see Tree.insert).</p>
+     * 
+     * <p>Note that the tree memory usage cannot be updated directly in
+     * getOffsets because the tree memory usage must always be the sum of all
+     * IN sizes, and it is reset to this sum each checkpoint.</p>
+     */
+    public void addExtraMarshaledMemorySize(BIN parentBIN) {
+        if (extraMarshaledMemorySize != 0) {
+            assert trackedSummary != null; /* Must be set during the insert. */
+            assert parentBIN.isLatchOwnerForWrite();
+            parentBIN.updateMemorySize(0, extraMarshaledMemorySize);
+            extraMarshaledMemorySize = 0;
+        }
+    }
 }
