@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: LN.java,v 1.151 2008/03/25 02:26:36 linda Exp $
+ * $Id: LN.java,v 1.152.2.5 2010/01/04 15:30:36 cwl Exp $
  */
 
 package com.sleepycat.je.tree;
@@ -78,17 +78,18 @@ public class LN extends Node implements Loggable {
      */
     public LN(DatabaseEntry dbt, EnvironmentImpl envImpl, boolean replicated) {
         super(envImpl, replicated);
-        byte[] data = dbt.getData();
-        if (data == null) {
-            this.data = null;
+
+        byte[] dat = dbt.getData();
+        if (dat == null) {
+            data = null;
         } else if (dbt.getPartial()) {
-            init(data,
+            init(dat,
                  dbt.getOffset(),
                  dbt.getPartialOffset() + dbt.getSize(),
                  dbt.getPartialOffset(),
                  dbt.getSize());
         } else {
-            init(data, dbt.getOffset(), dbt.getSize());
+            init(dat, dbt.getOffset(), dbt.getSize());
         }
         setDirty();
     }
@@ -463,8 +464,9 @@ public class LN extends Node implements Loggable {
 	boolean logAbortKnownDeleted;
         Txn logTxn;
         if (locker != null && locker.isTransactional()) {
-            entryType = isDelDup ? LogEntryType.LOG_DEL_DUPLN_TRANSACTIONAL
-                                 : getTransactionalLogType();
+            entryType = isDelDup ?
+                LogEntryType.LOG_DEL_DUPLN_TRANSACTIONAL :
+                getTransactionalLogType();
 	    WriteLockInfo info = locker.getWriteLockInfo(getNodeId());
 	    logAbortLsn = info.getAbortLsn();
 	    logAbortKnownDeleted = info.getAbortKnownDeleted();
@@ -474,8 +476,7 @@ public class LN extends Node implements Loggable {
                 info.setAbortInfo(dbImpl, getLastLoggedSize());
             }
         } else {
-            entryType = isDelDup ? LogEntryType.LOG_DEL_DUPLN
-                                 : getLogType();
+            entryType = isDelDup ? LogEntryType.LOG_DEL_DUPLN : getLogType();
             logAbortLsn = DbLsn.NULL_LSN;
 	    logAbortKnownDeleted = false;
             logTxn = null;
@@ -507,16 +508,32 @@ public class LN extends Node implements Loggable {
 
         long lsn = DbLsn.NULL_LSN;
 	try {
-	    lsn = env.getLogManager().log(logEntry, isProvisional,
-					  backgroundIO, oldLsn, dbImpl,
-					  repContext);
+            if (logTxn != null) {
+
+                /*
+                 * Writing an LN_TX entry requires looking at the Txn's
+                 * lastLoggedLsn.  The Txn may be used by multiple threads so
+                 * ensure that the view we get is consistent. [#17204]
+                 */
+                synchronized (logTxn) {
+                    lsn = env.getLogManager().log(logEntry, isProvisional,
+                                                  backgroundIO, oldLsn, dbImpl,
+                                                  repContext);
+                }
+            } else {
+                lsn = env.getLogManager().log(logEntry, isProvisional,
+                                              backgroundIO, oldLsn, dbImpl,
+                                              repContext);
+            }
 	} catch (DatabaseException DE) {
 
 	    /*
 	     * If any exception happens during logging, then force this txn
 	     * to onlyAbortable. [#15768]
 	     */
-	    locker.setOnlyAbortable();
+            if (locker != null) {
+                locker.setOnlyAbortable();
+            }
 	    throw DE;
 	}
         clearDirty();
