@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: TxnAbort.java,v 1.25 2008/05/13 01:57:01 linda Exp $
+ * $Id: TxnAbort.java,v 1.26.2.2 2010/01/04 15:30:37 cwl Exp $
  */
 
 package com.sleepycat.je.txn;
@@ -41,6 +41,9 @@ public class TxnAbort extends TxnEnd {
         if (!(other instanceof TxnAbort))
             return false;
 
-        return (id == ((TxnAbort) other).id);
+        TxnAbort otherAbort = (TxnAbort) other;
+
+        return ((id == otherAbort.id) && 
+                (repMasterNodeId == otherAbort.repMasterNodeId));
     }
 }
