@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: TxnEnd.java,v 1.40 2008/05/21 18:50:02 sam Exp $
+ * $Id: TxnEnd.java,v 1.41.2.2 2010/01/04 15:30:37 cwl Exp $
  */
 
 package com.sleepycat.je.txn;
@@ -25,7 +25,7 @@ public abstract class TxnEnd implements Loggable {
     private long lastLsn;
 
     /* For replication - master node which wrote this record. */
-    private int repMasterNodeId;
+    int repMasterNodeId;
 
     TxnEnd(long id, long lastLsn, int repMasterNodeId) {
         this.id = id;
@@ -56,7 +56,7 @@ public abstract class TxnEnd implements Loggable {
         return lastLsn;
     }
 
-    int getMasterNodeId() {
+    public int getMasterNodeId() {
         return repMasterNodeId;
     }
 
