@@ -1,19 +1,21 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: DaemonRunner.java,v 1.5.2.1 2007/02/01 14:49:54 cwl Exp $
+ * $Id: DaemonRunner.java,v 1.8.2.2 2010/01/04 15:30:38 cwl Exp $
  */
 
 package com.sleepycat.je.utilint;
 
+import com.sleepycat.je.ExceptionListener;
 
 /**
  * An object capable of running (run/pause/shutdown/etc) a daemon thread.
  * See DaemonThread for details.
  */
 public interface DaemonRunner {
+    void setExceptionListener(ExceptionListener exceptionListener);
     void runOrPause(boolean run);
     void requestShutdown();
     void shutdown();
