@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: TestHook.java,v 1.7.2.1 2007/02/01 14:49:54 cwl Exp $
+ * $Id: TestHook.java,v 1.11.2.2 2010/01/04 15:30:38 cwl Exp $
  */
 
 package com.sleepycat.je.utilint;
@@ -18,9 +18,16 @@ import java.io.IOException;
  * To use this, a unit test should extend TestHook with a class that overrides
  * the desired method. The desired code will have a method that allows the unit
  * test to specify a hook, and will execute the hook if it is non-null.
+ * This should be done within an assert like so:
+ *
+ *    assert TestHookExecute(myTestHook);
+ *
+ * See Tree.java for examples.
  */
 public interface TestHook {
 
+    public void hookSetup();
+
     public void doIOHook()
 	throws IOException;
 
