@@ -1,15 +1,18 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: BasicCursor.java,v 1.12 2008/02/05 23:28:21 mark Exp $
+ * $Id: BasicCursor.java,v 1.12.2.3 2010/01/04 15:30:38 cwl Exp $
  */
 
 package com.sleepycat.persist;
 
 import java.util.Iterator;
 
+/* <!-- begin JE only --> */
+import com.sleepycat.je.CacheMode;
+/* <!-- end JE only --> */
 import com.sleepycat.je.DatabaseEntry;
 import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.LockMode;
@@ -199,6 +202,18 @@ class BasicCursor<V> implements EntityCursor<V> {
         cursor.close();
     }
 
+    /* <!-- begin JE only --> */
+    public void setCacheMode(CacheMode cacheMode) {
+        cursor.getCursor().setCacheMode(cacheMode);
+    }
+    /* <!-- end JE only --> */
+
+    /* <!-- begin JE only --> */
+    public CacheMode getCacheMode() {
+        return cursor.getCursor().getCacheMode();
+    }
+    /* <!-- end JE only --> */
+
     void checkInitialized()
         throws IllegalStateException {
 
