@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: EntityStore.java,v 1.35 2008/05/19 20:33:31 mark Exp $
+ * $Id: EntityStore.java,v 1.35.2.3 2010/01/04 15:30:38 cwl Exp $
  */
 
 package com.sleepycat.persist;
@@ -165,13 +165,6 @@ public class EntityStore {
         store = new Store(env, storeName, config, false /*rawAccess*/);
     }
 
-    /**
-     * Only public for debugging.
-     */
-    void dumpCatalog() {
-        store.dumpCatalog();
-    }
-
     /**
      * Returns the environment associated with this store.
      *
