@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: CompositeKeyFormat.java,v 1.26 2008/01/07 14:28:59 cwl Exp $
+ * $Id: CompositeKeyFormat.java,v 1.27.2.3 2010/01/04 15:30:38 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -16,6 +16,7 @@ import java.util.List;
 import java.util.Map;
 
 import com.sleepycat.persist.model.ClassMetadata;
+import com.sleepycat.persist.model.EntityModel;
 import com.sleepycat.persist.model.FieldMetadata;
 import com.sleepycat.persist.raw.RawField;
 import com.sleepycat.persist.raw.RawObject;
@@ -97,7 +98,8 @@ public class CompositeKeyFormat extends Format {
         }
 
         /* Populate fields list in fieldNames order. */
-        List<FieldInfo> instanceFields = FieldInfo.getInstanceFields(cls);
+        List<FieldInfo> instanceFields =
+            FieldInfo.getInstanceFields(cls, metadata);
         fields = new ArrayList<FieldInfo>(instanceFields.size());
         for (String fieldName : fieldNames) {
             FieldInfo field = null;
@@ -141,7 +143,7 @@ public class CompositeKeyFormat extends Format {
     }
 
     @Override
-    ClassMetadata getClassMetadata() {
+    public ClassMetadata getClassMetadata() {
         if (metadata == null) {
             throw new IllegalStateException(getClassName());
         }
@@ -178,10 +180,10 @@ public class CompositeKeyFormat extends Format {
     }
 
     @Override
-    void initialize(Catalog catalog, int initVersion) {
+    void initialize(Catalog catalog, EntityModel model, int initVersion) {
         /* Initialize all fields. */
         for (FieldInfo field : fields) {
-            field.initialize(catalog, initVersion);
+            field.initialize(catalog, model, initVersion);
         }
         /* Create the accessor. */
         Class type = getType();
