@@ -1,13 +1,14 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: ConverterReader.java,v 1.10 2008/06/03 04:52:23 mark Exp $
+ * $Id: ConverterReader.java,v 1.10.2.3 2010/01/04 15:30:38 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
 
+import com.sleepycat.persist.model.EntityModel;
 import com.sleepycat.persist.evolve.Converter;
 import com.sleepycat.persist.raw.RawObject;
 
@@ -28,6 +29,7 @@ public class ConverterReader implements Reader {
     }
 
     public void initializeReader(Catalog catalog,
+                                 EntityModel model,
                                  int initVersion,
                                  Format oldFormat) {
         this.oldFormat = oldFormat;
