@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: Evolver.java,v 1.14 2008/02/06 19:48:02 linda Exp $
+ * $Id: Evolver.java,v 1.15.2.3 2010/01/04 15:30:38 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -213,7 +213,7 @@ class Evolver {
      * Called by PersistCatalog after calling evolveFormat or
      * addNonEntityFormat for all old formats.
      *
-     * We do not require deletion of an unreferenced class for three
+     * We do not require deletion of an unreferenced class for two
      * reasons: 1) built-in proxy classes may not be referenced, 2) the
      * user may wish to declare persistent classes that are not yet used.
      */
@@ -221,12 +221,7 @@ class Evolver {
         /* Process unreferenced classes. */
         for (Format oldFormat : unprocessedFormats) {
             oldFormat.setUnused(true);
-            Integer oldFormatId = oldFormat.getId();
-            if (!evolvedFormats.containsKey(oldFormatId)) {
-                evolvedFormats.put(oldFormatId, true);
-                boolean result = evolveFormatInternal(oldFormat);
-                evolvedFormats.put(oldFormatId, result);
-            }
+            evolveFormat(oldFormat);
         }
     }
 
@@ -235,9 +230,12 @@ class Evolver {
      * methods for all potentially referenced non-entity formats.
      */
     boolean evolveFormat(Format oldFormat) {
+        if (oldFormat.isNew()) {
+            return true;
+        }
         boolean result;
-        boolean trackEntityChanges =
-            oldFormat.getLatestVersion().getEntityFormat() != null;
+        Format oldEntityFormat = oldFormat.getEntityFormat();
+        boolean trackEntityChanges = oldEntityFormat != null;
         boolean saveNestedFormatsChanged = nestedFormatsChanged;
         if (trackEntityChanges) {
             nestedFormatsChanged = false;
@@ -255,7 +253,7 @@ class Evolver {
         }
         if (trackEntityChanges) {
             if (nestedFormatsChanged) {
-                Format latest = oldFormat.getLatestVersion().getEntityFormat();
+                Format latest = oldEntityFormat.getLatestVersion();
                 if (latest != null) {
                     latest.setEvolveNeeded(true);
                 }
@@ -354,9 +352,11 @@ class Evolver {
                 newFormat.getPreviousVersion() == null) {
                 assert newFormats.containsValue(newFormat);
                 Format oldLatestFormat = oldFormat.getLatestVersion();
-                evolveFormat(oldLatestFormat);
+                if (!evolveFormat(oldLatestFormat)) {
+                    return false;
+                }
                 if (oldLatestFormat == oldLatestFormat.getLatestVersion()) {
-                    assert !newFormats.containsValue(newFormat);
+                    assert !newFormats.containsValue(newFormat) : newFormat;
                     /* newFormat equals oldLatestFormat and was discarded. */
                     newFormat = oldLatestFormat;
                 }
@@ -380,7 +380,7 @@ class Evolver {
 
         /* Apply class Renamer and continue if successful. */
         if (renamer != null) {
-            if (!applyRenamer(renamer, oldFormat, newFormat)) {
+            if (!applyClassRenamer(renamer, oldFormat, newFormat)) {
                 return false;
             }
         }
@@ -415,7 +415,7 @@ class Evolver {
                      "@Entity or @Persistent annotation is still present");
                 return false;
             }
-            return applyDeleter(deleter, oldFormat, newFormat);
+            return applyClassDeleter(deleter, oldFormat, newFormat);
         } else {
             if (needDeleter) {
                 if (newFormat == null) {
@@ -456,8 +456,23 @@ class Evolver {
             assert renamedFormat == newFormat;
             useEvolvedFormat(oldFormat, renamedFormat, renamedFormat);
         } else if (newFormat != null &&
-                   oldFormat.getVersion() != newFormat.getVersion()) {
-            /* The user wants a new version number, but no other changes. */
+                   (oldFormat.getVersion() != newFormat.getVersion() ||
+                    !oldFormat.isCurrentVersion())) {
+
+            /*
+             * If the user wants a new version number, but ther are no other
+             * changes, we will oblige.  Or, if an attempt is being made to
+             * use an old version, then the following events happened and we
+             * must evolve the old format:
+             * 1) The (previously) latest version of the format was evolved
+             * because it is not equal to the live class version.  Note that
+             * evolveFormatInternal always evolves the latest version first.
+             * 2) We are now attempting to evolve an older version of the same
+             * format, and it happens to be equal to the live class version.
+             * However, we're already committed to the new format, and we must
+             * evolve all versions.
+             * [#16467]
+             */
             useEvolvedFormat(oldFormat, newFormat, newFormat);
         } else {
             /* The new format is discarded. */
@@ -482,9 +497,9 @@ class Evolver {
         setFormatsChanged(oldFormat);
     }
 
-    private boolean applyRenamer(Renamer renamer,
-                                 Format oldFormat,
-                                 Format newFormat) {
+    private boolean applyClassRenamer(Renamer renamer,
+                                      Format oldFormat,
+                                      Format newFormat) {
         if (!checkUpdatedVersion(renamer, oldFormat, newFormat)) {
             return false;
         }
@@ -517,20 +532,18 @@ class Evolver {
     /**
      * Called by ComplexFormat when a secondary key name is changed.
      */
-    void renameSecondaryDatabase(Format oldFormat,
-                                 Format newFormat,
+    void renameSecondaryDatabase(String oldEntityClass,
+                                 String newEntityClass,
                                  String oldKeyName,
                                  String newKeyName) {
         renameDbs.put
-            (Store.makeSecDbName
-                (storePrefix, oldFormat.getClassName(), oldKeyName),
-             Store.makeSecDbName
-                (storePrefix, newFormat.getClassName(), newKeyName));
+            (Store.makeSecDbName(storePrefix, oldEntityClass, oldKeyName),
+             Store.makeSecDbName(storePrefix, newEntityClass, newKeyName));
     }
 
-    private boolean applyDeleter(Deleter deleter,
-                                 Format oldFormat,
-                                 Format newFormat) {
+    private boolean applyClassDeleter(Deleter deleter,
+                                      Format oldFormat,
+                                      Format newFormat) {
         if (!checkUpdatedVersion(deleter, oldFormat, newFormat)) {
             return false;
         }
@@ -546,7 +559,7 @@ class Evolver {
         }
 
         /*
-         * Set the format to deleted last,  so that the above test using
+         * Set the format to deleted last, so that the above test using
          * isCurrentVersion works properly.
          */
         oldFormat.setDeleted(true);
@@ -561,9 +574,9 @@ class Evolver {
     /**
      * Called by ComplexFormat when a secondary key is dropped.
      */
-    void deleteSecondaryDatabase(Format oldFormat, String keyName) {
+    void deleteSecondaryDatabase(String oldEntityClass, String keyName) {
         deleteDbs.add(Store.makeSecDbName
-            (storePrefix, oldFormat.getClassName(), keyName));
+            (storePrefix, oldEntityClass, keyName));
     }
 
     private boolean applyConverter(Converter converter,
