@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: NonPersistentFormat.java,v 1.14 2008/01/07 14:28:59 cwl Exp $
+ * $Id: NonPersistentFormat.java,v 1.14.2.3 2010/01/04 15:30:38 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -11,6 +11,8 @@ package com.sleepycat.persist.impl;
 import java.lang.reflect.Array;
 import java.util.Map;
 
+import com.sleepycat.persist.model.EntityModel;
+
 /**
  * Format for a non-persistent class that is only used for declared field
  * types and arrays.  Currently used only for Object and interface types.
@@ -26,7 +28,7 @@ class NonPersistentFormat extends Format {
     }
 
     @Override
-    void initialize(Catalog catalog, int initVersion) {
+    void initialize(Catalog catalog, EntityModel model, int initVersion) {
     }
 
     @Override
