@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: PersistCatalog.java,v 1.46 2008/03/18 18:38:08 mark Exp $
+ * $Id: PersistCatalog.java,v 1.47.2.3 2010/01/04 15:30:38 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -17,6 +17,7 @@ import java.io.ObjectOutputStream;
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.IdentityHashMap;
@@ -44,6 +45,7 @@ import com.sleepycat.persist.model.ClassMetadata;
 import com.sleepycat.persist.model.EntityMetadata;
 import com.sleepycat.persist.model.EntityModel;
 import com.sleepycat.persist.raw.RawObject;
+import com.sleepycat.persist.raw.RawType;
 import com.sleepycat.util.RuntimeExceptionWrapper;
 
 /**
@@ -264,12 +266,12 @@ public class PersistCatalog implements Catalog {
                         }
                     }
                 }
+                model = new StoredModel(this);
                 for (Format format : formatList) {
                     if (format != null) {
-                        format.initializeIfNeeded(this);
+                        format.initializeIfNeeded(this, model);
                     }
                 }
-                model = new StoredModel(this);
                 success = true;
                 return;
             }
@@ -374,7 +376,7 @@ public class PersistCatalog implements Catalog {
             /* Initialize all formats. */
             for (Format format : formatList) {
                 if (format != null) {
-                    format.initializeIfNeeded(this);
+                    format.initializeIfNeeded(this, model);
                     if (format == format.getLatestVersion()) {
                         latestFormatMap.put(format.getClassName(), format);
                     }
@@ -547,9 +549,10 @@ public class PersistCatalog implements Catalog {
     }
 
     /**
-     * If the given class format is not already present in the given map,
-     * creates an uninitialized format, adds it to the map, and also collects
-     * related formats in the map.
+     * If the given class format is not already present in the given map and
+     * a format for this class name does not already exist, creates an
+     * uninitialized format, adds it to the map, and also collects related
+     * formats in the map.
      */
     public Format createFormat(Class type, Map<String,Format> newFormats) {
         /* Return a new or existing format for this class. */
@@ -653,7 +656,20 @@ public class PersistCatalog implements Catalog {
                 classes.add(format.getClassName());
             }
         }
-        return classes;
+        return Collections.unmodifiableSet(classes);
+    }
+
+    /**
+     * Returns all formats as RawTypes.
+     */
+    public List<RawType> getAllRawTypes() {
+        List<RawType> list = new ArrayList<RawType>();
+        for (RawType type : formatList) {
+            if (type != null) {
+                list.add(type);
+            }
+        }
+        return Collections.unmodifiableList(list);
     }
 
     /**
@@ -798,7 +814,7 @@ public class PersistCatalog implements Catalog {
         Catalog newFormatCatalog =
             new ReadOnlyCatalog(newFormatList, newFormatMap);
         for (Format newFormat : newFormats.values()) {
-            newFormat.initializeIfNeeded(newFormatCatalog);
+            newFormat.initializeIfNeeded(newFormatCatalog, model);
             newLatestFormatMap.put(newFormat.getClassName(), newFormat);
         }
 
@@ -918,11 +934,24 @@ public class PersistCatalog implements Catalog {
     public Object convertRawObject(RawObject o, IdentityHashMap converted) {
         Format format = (Format) o.getType();
         if (this != format.getCatalog()) {
-	    String className = format.getClassName();
-            format = getFormat(className);
+
+            /*
+             * Use the corresponding format in this catalog when the external
+             * raw object was created using a different catalog.  Create the
+             * format if it does not already exist, for example, when this
+             * store is empty. [#16253].
+             */
+	    String clsName = format.getClassName();
+            Class cls;
+            try {
+                cls = SimpleCatalog.classForName(clsName);
+                format = getFormat(cls, true /*openEntitySubclassIndexes*/);
+            } catch (ClassNotFoundException e) {
+                format = null;
+            }
             if (format == null) {
                 throw new IllegalArgumentException
-                    ("External raw type not found: " + className);
+                    ("External raw type not found: " + clsName);
             }
         }
         Format proxiedFormat = format.getProxiedFormat();
@@ -934,19 +963,4 @@ public class PersistCatalog implements Catalog {
         }
         return format.convertRawObject(this, false, o, converted);
     }
-
-    public void dump() {
-        System.out.println("--- Begin formats ---");
-        for (Format format : formatList) {
-            if (format != null) {
-                System.out.println
-                    ("ID: " + format.getId() +
-                     " class: " + format.getClassName() +
-                     " version: " + format.getVersion() +
-                     " current: " +
-                     (format == formatMap.get(format.getClassName())));
-            }
-        }
-        System.out.println("--- End formats ---");
-    }
 }
