@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: PersistEntityBinding.java,v 1.20 2008/03/18 18:38:08 mark Exp $
+ * $Id: PersistEntityBinding.java,v 1.20.2.3 2010/01/04 15:30:39 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -33,21 +33,10 @@ public class PersistEntityBinding implements EntityBinding {
                                 String entityClassName,
                                 boolean rawAccess) {
         this.catalog = catalog;
-        if (rawAccess) {
-            entityFormat = catalog.getFormat(entityClassName);
-            if (entityFormat == null || !entityFormat.isEntity()) {
-                throw new IllegalArgumentException
-                    ("Not an entity class: " + entityClassName);
-            }
-        } else {
-            Class entityCls;
-            try {
-                entityCls = EntityModel.classForName(entityClassName);
-            } catch (ClassNotFoundException e) {
-                throw new IllegalArgumentException(e);
-            }
-            entityFormat = catalog.getFormat
-                (entityCls, true /*openEntitySubclassIndexes*/);
+        entityFormat = getOrCreateFormat(catalog, entityClassName, rawAccess);
+        if (!entityFormat.isEntity()) {
+            throw new IllegalArgumentException
+                ("Not an entity class: " + entityClassName);
         }
         this.rawAccess = rawAccess;
     }
@@ -167,4 +156,24 @@ public class PersistEntityBinding implements EntityBinding {
 
         return format;
     }
+    
+    /**
+     * Utility method for getting or creating a format as appropriate for
+     * bindings and key creators.
+     */
+    static Format getOrCreateFormat(Catalog catalog,
+                                    String clsName,
+                                    boolean rawAccess) {
+        if (rawAccess) {
+            Format format = catalog.getFormat(clsName);
+            if (format == null) {
+                throw new IllegalArgumentException
+                    ("Not a persistent class: " + clsName);
+            }
+            return format;
+        } else {
+            Class cls = SimpleCatalog.keyClassForName(clsName);
+            return catalog.getFormat(cls, true /*openEntitySubclassIndexes*/);
+        }
+    }
 }
