@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: PersistKeyBinding.java,v 1.22 2008/03/18 18:38:08 mark Exp $
+ * $Id: PersistKeyBinding.java,v 1.22.2.4 2010/01/04 15:30:39 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -30,11 +30,8 @@ public class PersistKeyBinding implements EntryBinding {
                              String clsName,
                              boolean rawAccess) {
         this.catalog = catalog;
-        keyFormat = catalog.getFormat(clsName);
-        if (keyFormat == null) {
-            throw new IllegalArgumentException
-                ("Class is not persistent: " + clsName);
-        }
+        keyFormat = PersistEntityBinding.getOrCreateFormat
+            (catalog, clsName, rawAccess);
         if (!keyFormat.isSimple() &&
             (keyFormat.getClassMetadata() == null ||
              keyFormat.getClassMetadata().getCompositeKeyFields() == null)) {
@@ -60,7 +57,7 @@ public class PersistKeyBinding implements EntryBinding {
             keyFormat =
                 catalog.getFormat(cls, false /*openEntitySubclassIndexes*/);
         }
-        keyFormat.initializeIfNeeded(catalog);
+        keyFormat.initializeIfNeeded(catalog, null /*model*/);
     }
 
     /**
