@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: PersistKeyCreator.java,v 1.18 2008/01/07 14:28:59 cwl Exp $
+ * $Id: PersistKeyCreator.java,v 1.18.2.3 2010/01/04 15:30:39 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -49,13 +49,15 @@ public class PersistKeyCreator implements SecondaryKeyCreator,
     public PersistKeyCreator(Catalog catalog,
                              EntityMetadata entityMeta,
                              String keyClassName,
-                             SecondaryKeyMetadata secKeyMeta) {
+                             SecondaryKeyMetadata secKeyMeta,
+                             boolean rawAccess) {
         this.catalog = catalog;
-        Format priKeyFormat =
-            catalog.getFormat(entityMeta.getPrimaryKey().getClassName());
+        Format priKeyFormat = PersistEntityBinding.getOrCreateFormat
+            (catalog, entityMeta.getPrimaryKey().getClassName(), rawAccess);
         priKeyFormatId = priKeyFormat.getId();
         keyName = secKeyMeta.getKeyName();
-        keyFormat = catalog.getFormat(keyClassName);
+        keyFormat = PersistEntityBinding.getOrCreateFormat
+            (catalog, keyClassName, rawAccess);
         if (keyFormat == null) {
             throw new IllegalArgumentException
                 ("Not a key class: " + keyClassName);
