@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: PrimitiveArrayFormat.java,v 1.24 2008/03/18 18:38:08 mark Exp $
+ * $Id: PrimitiveArrayFormat.java,v 1.24.2.3 2010/01/04 15:30:39 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -13,6 +13,7 @@ import java.util.IdentityHashMap;
 import java.util.Map;
 import java.util.Set;
 
+import com.sleepycat.persist.model.EntityModel;
 import com.sleepycat.persist.raw.RawObject;
 
 /**
@@ -53,9 +54,14 @@ public class PrimitiveArrayFormat extends Format {
     }
 
     @Override
-    void initialize(Catalog catalog, int initVersion) {
+    void initialize(Catalog catalog, EntityModel model, int initVersion) {
+
+        /*
+         * getExistingType is allowed (to support raw mode) because primitive
+         * arrays are always available in Java.
+         */
         componentFormat = (SimpleFormat)
-            catalog.getFormat(getType().getComponentType(),
+            catalog.getFormat(getExistingType().getComponentType(),
                               false /*openEntitySubclassIndexes*/);
     }
 
