@@ -1,15 +1,17 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: Reader.java,v 1.3.2.2 2007/11/03 02:44:53 mark Exp $
+ * $Id: Reader.java,v 1.6.2.3 2010/01/04 15:30:39 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
 
 import java.io.Serializable;
 
+import com.sleepycat.persist.model.EntityModel;
+
 /**
  * Interface to the "read object" methods of the Format class.  For the
  * latest version format, the Format object provides the implementation of
@@ -21,7 +23,10 @@ import java.io.Serializable;
  */
 interface Reader extends Serializable {
 
-    void initializeReader(Catalog catalog, int initVersion, Format oldFormat);
+    void initializeReader(Catalog catalog,
+                          EntityModel model,
+                          int initVersion,
+                          Format oldFormat);
 
     Object newInstance(EntityInput input, boolean rawAccess);
 
