@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: SimpleCatalog.java,v 1.26 2008/03/18 18:38:08 mark Exp $
+ * $Id: SimpleCatalog.java,v 1.26.2.3 2010/01/04 15:30:39 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -191,7 +191,7 @@ public class SimpleCatalog implements Catalog {
      */
     private void setFormat(int id, SimpleFormat format) {
         format.setId(id);
-        format.initializeIfNeeded(this);
+        format.initializeIfNeeded(this, null /*model*/);
         formatList.set(id, format);
         formatMap.put(format.getClassName(), format);
     }
