@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: SimpleFormat.java,v 1.24 2008/01/07 14:28:59 cwl Exp $
+ * $Id: SimpleFormat.java,v 1.24.2.3 2010/01/04 15:30:39 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -15,6 +15,7 @@ import java.util.Map;
 import java.util.Set;
 
 import com.sleepycat.je.DatabaseEntry;
+import com.sleepycat.persist.model.EntityModel;
 
 /**
  * Format for simple types, including primitives.  Additional methods are
@@ -60,7 +61,7 @@ public abstract class SimpleFormat extends Format {
     }
 
     @Override
-    void initialize(Catalog catalog, int initVersion) {
+    void initialize(Catalog catalog, EntityModel model, int initVersion) {
     }
 
     @Override
