@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: Store.java,v 1.36 2008/05/19 20:33:32 mark Exp $
+ * $Id: Store.java,v 1.36.2.5 2010/01/04 15:30:39 cwl Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -259,10 +259,6 @@ public class Store {
         return storeName;
     }
 
-    public void dumpCatalog() {
-        catalog.dump();
-    }
-
     /* <!-- begin JE only --> */
     public static Set<String> getStoreNames(Environment env)
         throws DatabaseException {
@@ -1045,7 +1041,7 @@ public class Store {
                                        rel == Relationship.MANY_TO_MANY);
             setBtreeComparator(config, secKeyMeta.getClassName());
             PersistKeyCreator keyCreator = new PersistKeyCreator
-                (catalog, entityMeta, keyClassName, secKeyMeta);
+                (catalog, entityMeta, keyClassName, secKeyMeta, rawAccess);
             if (rel == Relationship.ONE_TO_MANY ||
                 rel == Relationship.MANY_TO_MANY) {
                 config.setMultiKeyCreator(keyCreator);
@@ -1322,9 +1318,10 @@ public class Store {
                     oneWritten = true;
                     nWritten += 1;
                 }
+                /* Update event stats, even if no listener. [#17024] */
+                EvolveInternal.updateEvent
+                    (event, entityClassName, 1, oneWritten ? 1 : 0);
                 if (listener != null) {
-                    EvolveInternal.updateEvent
-                        (event, entityClassName, 1, oneWritten ? 1 : 0);
                     if (!listener.evolveProgress(event)) {
                         break;
                     }
