@@ -1,14 +1,15 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: ClassMetadata.java,v 1.13 2008/01/07 14:28:59 cwl Exp $
+ * $Id: ClassMetadata.java,v 1.14.2.2 2010/01/04 15:30:39 cwl Exp $
  */
 
 package com.sleepycat.persist.model;
 
 import java.io.Serializable;
+import java.util.Collection;
 import java.util.List;
 import java.util.Map;
 
@@ -37,6 +38,23 @@ public class ClassMetadata implements Serializable {
     private PrimaryKeyMetadata primaryKey;
     private Map<String,SecondaryKeyMetadata> secondaryKeys;
     private List<FieldMetadata> compositeKeyFields;
+    private Collection<FieldMetadata> persistentFields;
+
+    /**
+     * Used by an {@code EntityModel} to construct persistent class metadata.
+     * The optional {@link #getPersistentFields} property will be set to null.
+     */
+    public ClassMetadata(String className,
+                         int version,
+                         String proxiedClassName,
+                         boolean entityClass,
+                         PrimaryKeyMetadata primaryKey,
+                         Map<String,SecondaryKeyMetadata> secondaryKeys,
+                         List<FieldMetadata> compositeKeyFields) {
+
+        this(className, version, proxiedClassName, entityClass, primaryKey,
+             secondaryKeys, compositeKeyFields, null /*persistentFields*/);
+    }
 
     /**
      * Used by an {@code EntityModel} to construct persistent class metadata.
@@ -47,7 +65,8 @@ public class ClassMetadata implements Serializable {
                          boolean entityClass,
                          PrimaryKeyMetadata primaryKey,
                          Map<String,SecondaryKeyMetadata> secondaryKeys,
-                         List<FieldMetadata> compositeKeyFields) {
+                         List<FieldMetadata> compositeKeyFields,
+                         Collection<FieldMetadata> persistentFields) {
         this.className = className;
         this.version = version;
         this.proxiedClassName = proxiedClassName;
@@ -55,6 +74,7 @@ public class ClassMetadata implements Serializable {
         this.primaryKey = primaryKey;
         this.secondaryKeys = secondaryKeys;
         this.compositeKeyFields = compositeKeyFields;
+        this.persistentFields = persistentFields;
     }
 
     /**
@@ -119,6 +139,23 @@ public class ClassMetadata implements Serializable {
         return compositeKeyFields;
     }
 
+    /**
+     * Returns an unmodifiable list of metadata for the persistent fields in
+     * this class, or null if the default rules for persistent fields should be
+     * used.  All fields returned must be declared in this class and must be
+     * non-static.
+     *
+     * <p>By default (if null is returned) the persistent fields of a class
+     * will be all declared instance fields that are non-transient (are not
+     * declared with the <code>transient</code> keyword).  The default rules
+     * may be overridden by an {@link EntityModel}.  For example, the {@link
+     * AnnotationModel} overrides the default rules when the {@link
+     * NotPersistent} or {@link NotTransient} annotation is specified.</p>
+     */
+    public Collection<FieldMetadata> getPersistentFields() {
+        return persistentFields;
+    }
+
     @Override
     public boolean equals(Object other) {
         if (other instanceof ClassMetadata) {
