@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: FieldMetadata.java,v 1.12 2008/01/07 14:28:59 cwl Exp $
+ * $Id: FieldMetadata.java,v 1.13.2.2 2010/01/04 15:30:39 cwl Exp $
  */
 
 package com.sleepycat.persist.model;
@@ -78,4 +78,10 @@ public class FieldMetadata implements Serializable {
                ClassMetadata.hashCode(className) +
                ClassMetadata.hashCode(declaringClassName);
     }
+
+    @Override
+    public String toString() {
+        return "[FieldMetadata name: " + name + " className: " + className +
+               " declaringClassName: " + declaringClassName + ']';
+    }
 }
