@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: SecondaryKeyMetadata.java,v 1.15 2008/01/07 14:28:59 cwl Exp $
+ * $Id: SecondaryKeyMetadata.java,v 1.15.2.3 2010/01/04 15:30:39 cwl Exp $
  */
 
 package com.sleepycat.persist.model;
@@ -100,7 +100,7 @@ public class SecondaryKeyMetadata extends FieldMetadata {
             SecondaryKeyMetadata o = (SecondaryKeyMetadata) other;
             return super.equals(o) &&
                    relationship == o.relationship &&
-                   deleteAction == o.deleteAction &&
+                   ClassMetadata.nullOrEqual(deleteAction, o.deleteAction) &&
                    ClassMetadata.nullOrEqual(keyName, o.keyName) &&
                    ClassMetadata.nullOrEqual(elementClassName,
                                              o.elementClassName) &&
@@ -114,7 +114,7 @@ public class SecondaryKeyMetadata extends FieldMetadata {
     public int hashCode() {
         return super.hashCode() +
                relationship.hashCode() +
-               deleteAction.hashCode() +
+               ClassMetadata.hashCode(deleteAction) +
                ClassMetadata.hashCode(keyName) +
                ClassMetadata.hashCode(elementClassName) +
                ClassMetadata.hashCode(relatedEntity);
