@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: RawType.java,v 1.12 2008/01/07 14:29:00 cwl Exp $
+ * $Id: RawType.java,v 1.12.2.3 2010/01/04 15:30:41 cwl Exp $
  */
 
 package com.sleepycat.persist.raw;
@@ -11,7 +11,9 @@ package com.sleepycat.persist.raw;
 import java.util.List;
 import java.util.Map;
 
+import com.sleepycat.persist.model.ClassMetadata;
 import com.sleepycat.persist.model.Entity;
+import com.sleepycat.persist.model.EntityMetadata;
 import com.sleepycat.persist.model.Persistent;
 
 /**
@@ -46,6 +48,11 @@ public interface RawType {
      */
     int getVersion();
 
+    /**
+     * Returns the internal unique ID for this type.
+     */
+    int getId();
+
     /**
      * Returns whether this is a {@link <a
      * href="../model/Entity.html#simpleTypes">simple type</a>}: primitive,
@@ -132,4 +139,21 @@ public interface RawType {
      * an array type).
      */
     RawType getSuperType();
+
+    /**
+     * Returns the original model class metadata used to create this class, or
+     * null if this is not a model class.
+     */
+    ClassMetadata getClassMetadata();
+
+    /**
+     * Returns the original model entity metadata used to create this class, or
+     * null if this is not an entity class.
+     */
+    EntityMetadata getEntityMetadata();
+
+    /**
+     * Returns an XML representation of the raw type.
+     */
+    String toString();
 }
