@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: CursorEdgeTest.java,v 1.38 2008/01/07 14:29:04 cwl Exp $
+ * $Id: CursorEdgeTest.java,v 1.39.2.2 2010/01/04 15:30:42 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -541,7 +541,7 @@ public class CursorEdgeTest extends TestCase {
          * Check that it's a transaction problem. Crude, but since we
          * don't want to add exception types, necessary.
          */
-        String eMsg = e.getMessage();
+        e.getMessage();
         assertTrue(TestUtils.skipVersion(e).startsWith("A transaction was not supplied"));
     }
 
