@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: EnvironmentConfigTest.java,v 1.13 2008/01/07 14:29:05 cwl Exp $
+ * $Id: EnvironmentConfigTest.java,v 1.14.2.2 2010/01/04 15:30:42 cwl Exp $
  */
 
 package com.sleepycat.je;
@@ -87,7 +87,7 @@ public class EnvironmentConfigTest extends TestCase {
 	    config.setLocking(false);
 	    config.setTransactional(true);
 	    File envHome = new File(System.getProperty(TestUtils.DEST_DIR));
-	    Environment env = new Environment(envHome, config);
+	    new Environment(envHome, config);
             fail("Should fail because of inconsistent param values");
         } catch (IllegalArgumentException e) {
             // expected.
