@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: CleanerTest.java,v 1.102 2008/05/23 18:50:50 mark Exp $
+ * $Id: CleanerTest.java,v 1.103.2.8 2010/01/30 01:10:55 mark Exp $
  */
 
 package com.sleepycat.je.cleaner;
@@ -37,6 +37,7 @@ import com.sleepycat.je.OperationStatus;
 import com.sleepycat.je.StatsConfig;
 import com.sleepycat.je.Transaction;
 import com.sleepycat.je.cleaner.Cleaner;
+import com.sleepycat.je.cleaner.FileSelector;
 import com.sleepycat.je.cleaner.FileSummary;
 import com.sleepycat.je.cleaner.TrackedFileSummary;
 import com.sleepycat.je.cleaner.UtilizationProfile;
@@ -45,11 +46,13 @@ import com.sleepycat.je.dbi.CursorImpl;
 import com.sleepycat.je.dbi.DatabaseImpl;
 import com.sleepycat.je.dbi.EnvironmentImpl;
 import com.sleepycat.je.dbi.MemoryBudget;
+import com.sleepycat.je.junit.JUnitThread;
 import com.sleepycat.je.log.FileManager;
 import com.sleepycat.je.recovery.Checkpointer;
 import com.sleepycat.je.tree.BIN;
 import com.sleepycat.je.tree.FileSummaryLN;
 import com.sleepycat.je.tree.IN;
+import com.sleepycat.je.tree.Node;
 import com.sleepycat.je.txn.BasicLocker;
 import com.sleepycat.je.txn.LockType;
 import com.sleepycat.je.util.StringDbt;
@@ -71,6 +74,8 @@ public class CleanerTest extends TestCase {
     private Environment exampleEnv;
     private Database exampleDb;
     private CheckpointConfig forceConfig;
+    private JUnitThread junitThread;
+    private volatile int synchronizer;
 
     public CleanerTest() {
         envHome = new File(System.getProperty(TestUtils.DEST_DIR));
@@ -95,8 +100,6 @@ public class CleanerTest extends TestCase {
         envConfig.setTxnNoSync(Boolean.getBoolean(TestUtils.NO_SYNC));
         envConfig.setConfigParam(EnvironmentParams.LOG_FILE_MAX.getName(),
                                  Integer.toString(FILE_SIZE));
-        envConfig.setConfigParam(EnvironmentParams.ENV_CHECK_LEAKS.getName(),
-                                 "false");
         envConfig.setConfigParam(EnvironmentParams.ENV_RUN_CLEANER.getName(),
                                  "false");
         envConfig.setConfigParam(EnvironmentParams.CLEANER_REMOVE.getName(),
@@ -109,10 +112,6 @@ public class CleanerTest extends TestCase {
         envConfig.setConfigParam(EnvironmentParams.BIN_DELTA_PERCENT.getName(),
                                  "75");
 
-        /* Don't use detail tracking in this test. */
-        envConfig.setConfigParam
-            (EnvironmentParams.CLEANER_TRACK_DETAIL.getName(), "false");
-
         exampleEnv = new Environment(envHome, envConfig);
 
         String databaseName = "cleanerDb";
@@ -126,6 +125,14 @@ public class CleanerTest extends TestCase {
     public void tearDown()
         throws IOException, DatabaseException {
 
+        if (junitThread != null) {
+            while (junitThread.isAlive()) {
+                junitThread.interrupt();
+                Thread.yield();
+            }
+            junitThread = null;
+        }
+
         if (exampleEnv != null) {
             try {
                 exampleEnv.close();
@@ -136,7 +143,7 @@ public class CleanerTest extends TestCase {
         exampleDb = null;
         exampleEnv = null;
 
-        /*
+        //*
         try {
             TestUtils.removeLogFiles("TearDown", envHome, true);
             TestUtils.removeFiles("TearDown", envHome, FileManager.DEL_SUFFIX);
@@ -190,7 +197,8 @@ public class CleanerTest extends TestCase {
         EnvironmentImpl environment =
             DbInternal.envGetEnvironmentImpl(exampleEnv);
         FileManager fileManager = environment.getFileManager();
-        Map expectedMap = new HashMap();
+        Map<String, Set<String>> expectedMap =
+            new HashMap<String, Set<String>>();
         doLargePut(expectedMap, nKeys, nDupsPerKey, true);
         Long lastNum = fileManager.getLastFileNum();
 
@@ -246,7 +254,8 @@ public class CleanerTest extends TestCase {
             DbInternal.envGetEnvironmentImpl(exampleEnv);
         FileManager fileManager = environment.getFileManager();
         /* Insert a lot of keys. ExpectedMap holds the expected data */
-        Map expectedMap = new HashMap();
+        Map<String, Set<String>> expectedMap =
+            new HashMap<String, Set<String>>();
         doLargePut(expectedMap, nKeys, 1, true);
 
         /* Modify every other piece of data. */
@@ -302,7 +311,8 @@ public class CleanerTest extends TestCase {
         FileManager fileManager = environment.getFileManager();
 
         /* Insert some non dup data, modify, insert dup data. */
-        Map expectedMap = new HashMap();
+        Map<String, Set<String>> expectedMap =
+            new HashMap<String, Set<String>>();
         doLargePut(expectedMap, nKeys, 1, true);
         modifyData(expectedMap, 10, true);
         doLargePut(expectedMap, nKeys, nDupsPerKey, true);
@@ -503,6 +513,84 @@ public class CleanerTest extends TestCase {
         env.close();
     }
 
+    /**
+     * Tests that the FileSelector memory budget is subtracted when the
+     * environment is closed.  Before the fix in SR [#16368], it was not.
+     */
+    public void testFileSelectorMemBudget()
+        throws Throwable {
+
+        final int fileSize = 1000000;
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+        envConfig.setAllowCreate(true);
+        envConfig.setConfigParam
+            (EnvironmentParams.ENV_RUN_CLEANER.getName(), "false");
+        envConfig.setConfigParam
+            (EnvironmentParams.ENV_RUN_CHECKPOINTER.getName(), "false");
+        envConfig.setConfigParam
+            (EnvironmentParams.LOG_FILE_MAX.getName(),
+             Integer.toString(fileSize));
+        envConfig.setConfigParam
+            (EnvironmentParams.CLEANER_MIN_UTILIZATION.getName(), "80");
+        Environment env = new Environment(envHome, envConfig);
+
+        DatabaseConfig dbConfig = new DatabaseConfig();
+        dbConfig.setAllowCreate(true);
+        Database db = env.openDatabase(null, "foo", dbConfig);
+
+        DatabaseEntry key = new DatabaseEntry(new byte[1]);
+        DatabaseEntry data = new DatabaseEntry(new byte[fileSize]);
+        for (int i = 0; i <= 10; i += 1) {
+            db.put(null, key, data);
+        }
+        env.checkpoint(forceConfig);
+
+        int nFiles = env.cleanLog();
+        assertTrue(nFiles > 0);
+
+        db.close();
+
+        /*
+         * To force the memory leak to be detected we have to close without a
+         * checkpoint.  The checkpoint will finish processing all cleaned files
+         * and subtract them from the budget.  But this should happen during
+         * close, even without a checkpoint.
+         */
+        EnvironmentImpl envImpl = DbInternal.envGetEnvironmentImpl(env);
+        envImpl.close(false /*doCheckpoint*/);
+    }
+
+    /**
+     * Tests that the cleanLog cannot be called in a read-only environment.
+     * [#16368]
+     */
+    public void testCleanLogReadOnly()
+        throws Throwable {
+
+        /* Open read-write. */
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+        envConfig.setAllowCreate(true);
+        exampleEnv = new Environment(envHome, envConfig);
+        exampleEnv.close();
+        exampleEnv = null;
+
+        /* Open read-only. */
+        envConfig.setAllowCreate(false);
+        envConfig.setReadOnly(true);
+        exampleEnv = new Environment(envHome, envConfig);
+
+        /* Try cleanLog in a read-only env. */
+        try {
+            exampleEnv.cleanLog();
+            fail();
+        } catch (IllegalStateException e) {
+            assertEquals
+                ("Log cleaning not allowed in a read-only or memory-only " +
+                 "environment", e.getMessage());
+
+        }
+    }
+
     /**
      * Tests that when a file being cleaned is deleted, we ignore the error and
      * don't repeatedly try to clean it.  This is happening when we mistakedly
@@ -517,7 +605,7 @@ public class CleanerTest extends TestCase {
         initEnv(true, false);
         EnvironmentMutableConfig config = exampleEnv.getMutableConfig();
         config.setConfigParam
-            (EnvironmentParams.ENV_RUN_CLEANER.getName(), "true");
+            (EnvironmentParams.ENV_RUN_CLEANER.getName(), "false");
         config.setConfigParam
             (EnvironmentParams.CLEANER_MIN_UTILIZATION.getName(), "80");
         exampleEnv.setMutableConfig(config);
@@ -525,17 +613,31 @@ public class CleanerTest extends TestCase {
         final EnvironmentImpl envImpl =
             DbInternal.envGetEnvironmentImpl(exampleEnv);
         final Cleaner cleaner = envImpl.getCleaner();
+        final FileSelector fileSelector = cleaner.getFileSelector();
 
-        Map expectedMap = new HashMap();
+        Map<String, Set<String>> expectedMap =
+            new HashMap<String, Set<String>>();
         doLargePut(expectedMap, 1000, 1, true);
         checkData(expectedMap);
 
+        final long file1 = 0;
+        final long file2 = 1;
+
         for (int i = 0; i < 100; i += 1) {
             modifyData(expectedMap, 1, true);
             checkData(expectedMap);
-            cleaner.injectFileForCleaning(new Long(0));
-            exampleEnv.cleanLog();
+            fileSelector.injectFileForCleaning(new Long(file1));
+            fileSelector.injectFileForCleaning(new Long(file2));
+            assertTrue(fileSelector.getToBeCleanedFiles().contains(file1));
+            assertTrue(fileSelector.getToBeCleanedFiles().contains(file2));
+            while (exampleEnv.cleanLog() > 0) {}
+            assertTrue(!fileSelector.getToBeCleanedFiles().contains(file1));
+            assertTrue(!fileSelector.getToBeCleanedFiles().contains(file2));
             exampleEnv.checkpoint(forceConfig);
+            Map<Long,FileSummary> allFiles = envImpl.getUtilizationProfile().
+                getFileSummaryMap(true /*includeTrackedFiles*/);
+            assertTrue(!allFiles.containsKey(file1));
+            assertTrue(!allFiles.containsKey(file2));
         }
         checkData(expectedMap);
 
@@ -546,7 +648,7 @@ public class CleanerTest extends TestCase {
      * Helper routine. Generates keys with random alpha values while data
      * is numbered numerically.
      */
-    private void doLargePut(Map expectedMap,
+    private void doLargePut(Map<String, Set<String>> expectedMap,
                             int nKeys,
                             int nDupsPerKey,
                             boolean commit)
@@ -562,7 +664,7 @@ public class CleanerTest extends TestCase {
              * The data map is keyed by key value, and holds a hash
              * map of all data values.
              */
-            Set dataVals = new HashSet();
+            Set<String> dataVals = new HashSet<String>();
             if (commit) {
                 expectedMap.put(keyString, dataVals);
             }
@@ -584,7 +686,7 @@ public class CleanerTest extends TestCase {
     /**
      * Increment each data value.
      */
-    private void modifyData(Map expectedMap,
+    private void modifyData(Map<String, Set<String>> expectedMap,
                             int increment,
                             boolean commit)
         throws DatabaseException {
@@ -610,7 +712,7 @@ public class CleanerTest extends TestCase {
                 /* If committing, adjust the expected map. */
                 if (commit) {
 
-                    Set dataVals = (Set) expectedMap.get(foundKeyString);
+                    Set<String> dataVals = expectedMap.get(foundKeyString);
                     if (dataVals == null) {
                         fail("Couldn't find " +
                              foundKeyString + "/" + foundDataString);
@@ -647,7 +749,7 @@ public class CleanerTest extends TestCase {
     /**
      * Delete data.
      */
-    private void deleteData(Map expectedMap,
+    private void deleteData(Map<String, Set<String>> expectedMap,
                             boolean everyOther,
                             boolean commit)
         throws DatabaseException {
@@ -671,7 +773,7 @@ public class CleanerTest extends TestCase {
                 /* If committing, adjust the expected map */
                 if (commit) {
 
-                    Set dataVals = (Set) expectedMap.get(foundKeyString);
+                    Set dataVals = expectedMap.get(foundKeyString);
                     if (dataVals == null) {
                         fail("Couldn't find " +
                              foundKeyString + "/" + foundDataString);
@@ -707,7 +809,7 @@ public class CleanerTest extends TestCase {
     /**
      * Check what's in the database against what's in the expected map.
      */
-    private void checkData(Map expectedMap)
+    private void checkData(Map<String, Set<String>> expectedMap)
         throws DatabaseException {
 
         StringDbt foundKey = new StringDbt();
@@ -722,13 +824,14 @@ public class CleanerTest extends TestCase {
          * Also make a set of counts for each key value, to test count.
          */
 
-        Map checkMap = new HashMap();
-        Map countMap = new HashMap();
-        Iterator iter = expectedMap.entrySet().iterator();
+        Map<String, Set<String>> checkMap = new HashMap<String, Set<String>>();
+        Map<String, Integer>countMap = new HashMap<String, Integer>();
+        Iterator<Map.Entry<String, Set<String>>> iter =
+        		expectedMap.entrySet().iterator();
         while (iter.hasNext()) {
-            Map.Entry entry = (Map.Entry) iter.next();
-            Set copySet = new HashSet();
-            copySet.addAll((Set) entry.getValue());
+            Map.Entry<String, Set<String>> entry = iter.next();
+            Set<String> copySet = new HashSet<String>();
+            copySet.addAll(entry.getValue());
             checkMap.put(entry.getKey(), copySet);
             countMap.put(entry.getKey(), new Integer(copySet.size()));
         }
@@ -738,7 +841,7 @@ public class CleanerTest extends TestCase {
             String foundDataString = foundData.getString();
 
             /* Check that the current value is in the check values map */
-            Set dataVals = (Set) checkMap.get(foundKeyString);
+            Set dataVals = checkMap.get(foundKeyString);
             if (dataVals == null) {
                 fail("Couldn't find " +
                      foundKeyString + "/" + foundDataString);
@@ -756,7 +859,7 @@ public class CleanerTest extends TestCase {
 
             /* Check that the count is right. */
             int count = cursor.count();
-            assertEquals(((Integer)countMap.get(foundKeyString)).intValue(),
+            assertEquals(countMap.get(foundKeyString).intValue(),
                          count);
 
             status = cursor.getNext(foundKey, foundData, LockMode.DEFAULT);
@@ -937,8 +1040,6 @@ public class CleanerTest extends TestCase {
         EnvironmentConfig envConfig = TestUtils.initEnvConfig();
         envConfig.setAllowCreate(true);
         envConfig.setTransactional(true);
-        envConfig.setConfigParam
-            (EnvironmentParams.ENV_CHECK_LEAKS.getName(), "false");
         envConfig.setConfigParam
             (EnvironmentParams.ENV_RUN_CLEANER.getName(), "false");
         envConfig.setConfigParam
@@ -1044,8 +1145,8 @@ public class CleanerTest extends TestCase {
         exampleEnv = new Environment(envHome, envConfig);
 
         EnvironmentImpl envImpl = DbInternal.envGetEnvironmentImpl(exampleEnv);
-        DatabaseImpl fileSummaryDb =
-            envImpl.getUtilizationProfile().getFileSummaryDb();
+        UtilizationProfile up = envImpl.getUtilizationProfile();
+        DatabaseImpl fileSummaryDb = up.getFileSummaryDb();
         MemoryBudget memBudget = envImpl.getMemoryBudget();
 
         BasicLocker locker = null;
@@ -1079,12 +1180,18 @@ public class CleanerTest extends TestCase {
                 (keyBytes, ln, false /*allowDuplicates*/,
                  fileSummaryDb.getRepContext());
             assertSame(status, OperationStatus.SUCCESS);
+
+            assertSame(parent, cursor.latchBIN());
+            ln.addExtraMarshaledMemorySize(parent);
+            cursor.releaseBIN();
+
             long newSize = ln.getMemorySizeIncludedByParent();
             long newParentSize = getMemSize(parent, memBudget);
 
             /* The size of the LN increases during logging. */
             assertEquals(newSize,
-                oldSize + ln.getObsoleteOffsets().getExtraMemorySize());
+                         oldSize +
+                         ln.getObsoleteOffsets().getExtraMemorySize());
 
             /* The correct size is accounted for by the parent BIN. */
             assertEquals(newSize + keySize, newParentSize - oldParentSize);
@@ -1094,6 +1201,7 @@ public class CleanerTest extends TestCase {
             cursor.evict();
             newParentSize = getMemSize(parent, memBudget);
             assertEquals(oldParentSize - newSize, newParentSize);
+            long evictedParentSize = newParentSize;
 
             /* Fetch a fresh FileSummaryLN before deleting it. */
             oldParentSize = newParentSize;
@@ -1116,6 +1224,13 @@ public class CleanerTest extends TestCase {
             assertTrue(oldSize - newSize >
                        ln.getObsoleteOffsets().getExtraMemorySize());
             assertEquals(newSize - oldSize, newParentSize - oldParentSize);
+
+            /* Correct size is subtracted during eviction. */
+            oldParentSize = newParentSize;
+            cursor.evict();
+            newParentSize = getMemSize(parent, memBudget);
+            assertEquals(oldParentSize - newSize, newParentSize);
+            assertEquals(evictedParentSize, newParentSize);
         } finally {
             if (cursor != null) {
                 cursor.releaseBINs();
@@ -1126,6 +1241,18 @@ public class CleanerTest extends TestCase {
             }
         }
 
+        TestUtils.validateNodeMemUsage(envImpl, true /*assertOnError*/);
+
+        /* Insert again, this time using the UtilizationProfile method. */
+        FileSummaryLN ln = new FileSummaryLN(envImpl, new FileSummary());
+        TrackedFileSummary tfs = new TrackedFileSummary
+            (envImpl.getUtilizationTracker(), 0 /*fileNum*/,
+             true /*trackDetail*/);
+        tfs.trackObsolete(0);
+        ln.setTrackedSummary(tfs);
+        assertTrue(up.insertFileSummary(ln, 0 /*fileNum*/, 123 /*sequence*/));
+        TestUtils.validateNodeMemUsage(envImpl, true /*assertOnError*/);
+
         closeEnv();
     }
 
@@ -1262,4 +1389,326 @@ public class CleanerTest extends TestCase {
                in.getInMemorySize() -
                in.getBudgetedMemorySize();
     }
+
+    /**
+     * Tests that dirtiness is logged upwards during a checkpoint, even if a
+     * node is evicted and refetched after being added to the checkpointer's
+     * dirty map, and before that entry in the dirty map is processed by the
+     * checkpointer.  [#16523]
+     *
+     *  Root INa
+     *      /  \
+     *     INb  ...
+     *    /
+     *   INc
+     *  /
+     * BINd
+     *
+     * The scenario that causes the bug is:
+     *
+     * 1) Prior to the final checkpoint, the cleaner processes a log file
+     * containing BINd.  The cleaner marks BINd dirty so that it will be
+     * flushed prior to the end of the next checkpoint, at which point the file
+     * containing BINd will be deleted.  The cleaner also calls
+     * setProhibitNextDelta on BINd to ensure that a full version will be
+     * logged.
+     *
+     * 2) At checkpoint start, BINd is added to the checkpoiner's dirty map.
+     * It so happens that INa is also dirty, perhaps as the result of a split,
+     * and added to the dirty map.  The checkpointer's max flush level is 4.
+     *
+     * 3) The evictor flushes BINd and then its parent INc.  Both are logged
+     * provisionally, since their level is less than 4, the checkpointer's max
+     * flush level.  INb, the parent of INc, is dirty.
+     *
+     * 4) INc, along with BINd, is loaded back into the Btree as the result of
+     * reading an LN in BINd.  INc and BINd are both non-dirty.  INb, the
+     * parent of INc, is still dirty.
+     *
+     * 5) The checkpointer processes its reference to BINd in the dirty map.
+     * It finds that BINd is not dirty, so does not need to be logged.  It
+     * attempts to add the parent, INc, to the dirty map in order to propogate
+     * changes upward.  However, becaue INc is not dirty, it is not added to
+     * the dirty map -- this was the bug, it should be added even if not dirty.
+     * So as the result of this step, the checkpointer does no logging and does
+     * not add anything to the dirty map.
+     *
+     * 6) The checkpointer logs INa (it was dirty at the start of the
+     * checkpoint) and the checkpoint finishes.  It deletes the cleaned log
+     * file that contains the original version of BINd.
+     *
+     * The key thing is that INb is now dirty and was not logged.  It should
+     * have been logged as the result of being an ancestor of BINd, which was
+     * in the dirty map.  Its parent INa was logged, but does not refer to the
+     * latest version of INb/INc/BINd.
+     *
+     * 7) Now we recover.  INc and BINd, which were evicted during step (3),
+     * are not replayed because they are provisional -- they are lost.  When a
+     * search for an LN in BINd is performed, we traverse down to the old
+     * version of BINd, which causes LogFileNotFound.
+     *
+     * The fix is to add INc to the dirty map at step (5), even though it is
+     * not dirty.  When the reference to INc in the dirty map is processed we
+     * will not log INc, but we will add its parent INb to the dirty map.  Then
+     * when the reference to INb is processed, it will be logged because it is
+     * dirty.  Then INa is logged and refers to the latest version of
+     * INb/INc/BINd.
+     *
+     * This problem could only occur with a Btree of depth 4 or greater.
+     */
+    public void testEvictionDuringCheckpoint()
+        throws DatabaseException {
+
+        /* Use small fanout to create a deep tree. */
+        final int FANOUT = 6;
+        final int N_KEYS = FANOUT * FANOUT * FANOUT;
+
+        /* Open environment without interference of daemon threads. */
+        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+        envConfig.setAllowCreate(true);
+        envConfig.setConfigParam
+            (EnvironmentParams.ENV_RUN_CHECKPOINTER.getName(), "false");
+        envConfig.setConfigParam
+            (EnvironmentParams.ENV_RUN_CLEANER.getName(), "false");
+        envConfig.setConfigParam
+            (EnvironmentParams.ENV_RUN_INCOMPRESSOR.getName(), "false");
+        envConfig.setConfigParam
+            (EnvironmentParams.ENV_RUN_EVICTOR.getName(), "false");
+        exampleEnv = new Environment(envHome, envConfig);
+        final EnvironmentImpl envImpl =
+            DbInternal.envGetEnvironmentImpl(exampleEnv);
+
+        /* Open database. */
+        DatabaseConfig dbConfig = new DatabaseConfig();
+        dbConfig.setAllowCreate(true);
+        dbConfig.setNodeMaxEntries(FANOUT);
+        exampleDb = exampleEnv.openDatabase(null, "foo", dbConfig);
+        DatabaseImpl dbImpl = DbInternal.dbGetDatabaseImpl(exampleDb);
+
+        /* Write to database to create a 4 level Btree. */
+        final DatabaseEntry keyEntry = new DatabaseEntry();
+        final DatabaseEntry dataEntry = new DatabaseEntry(new byte[0]);
+        int nRecords;
+        for (nRecords = 1;; nRecords += 1) {
+            LongBinding.longToEntry(nRecords, keyEntry);
+            assertSame(OperationStatus.SUCCESS,
+                       exampleDb.put(null, keyEntry, dataEntry));
+            if (nRecords % 10 == 0) {
+                int level = envImpl.getDbTree().getHighestLevel(dbImpl);
+                if ((level & IN.LEVEL_MASK) >= 4) {
+                    break;
+                }
+            }
+        }
+
+        /* Flush all dirty nodes. */
+        exampleEnv.sync();
+        
+        /* Get BINd and its ancestors.  Mark BINd and INa dirty. */
+        final IN nodeINa = dbImpl.getTree().getRootIN(CacheMode.DEFAULT);
+        nodeINa.releaseLatch();
+        final IN nodeINb = (IN) nodeINa.getTarget(0);
+        final IN nodeINc = (IN) nodeINb.getTarget(0);
+        final BIN nodeBINd = (BIN) nodeINc.getTarget(0);
+        assertNotNull(nodeBINd);
+        nodeINa.setDirty(true);
+        nodeBINd.setDirty(true);
+
+        /*
+         * The test hook is called after creating the checkpoint dirty map and
+         * just before flushing dirty nodes.
+         */
+        final StringBuilder hookCalledFlag = new StringBuilder();
+
+        Checkpointer.setBeforeFlushHook(new TestHook() {
+            public void doHook() {
+                hookCalledFlag.append(1);
+                /* Don't call the test hook repeatedly. */
+                Checkpointer.setBeforeFlushHook(null);
+                try {
+                    /* Evict BINd and INc. */
+                    simulateEviction(exampleEnv, envImpl, nodeBINd, nodeINc);
+                    simulateEviction(exampleEnv, envImpl, nodeINc, nodeINb);
+
+                    /*
+                     * Force BINd and INc to be loaded into cache by fetching
+                     * the left-most record.
+                     *
+                     * Note that nodeINc and nodeBINd are different instances
+                     * and are no longer in the Btree but we don't change these
+                     * variables because they are final.  They should not be
+                     * used past this point.
+                     */
+                    LongBinding.longToEntry(1, keyEntry);
+                    assertSame(OperationStatus.SUCCESS,
+                               exampleDb.get(null, keyEntry, dataEntry, null));
+                } catch (DatabaseException e) {
+                    throw new RuntimeException(e);
+                }
+            }
+            public Object getHookValue() {
+                throw new UnsupportedOperationException();
+            }
+            public void doIOHook() throws IOException {
+                throw new UnsupportedOperationException();
+            }
+            public void hookSetup() {
+                throw new UnsupportedOperationException();
+            }
+        });
+        exampleEnv.checkpoint(forceConfig);
+        assertTrue(hookCalledFlag.length() > 0);
+        assertTrue(!nodeINa.getDirty());
+        assertTrue(!nodeINb.getDirty()); /* This failed before the bug fix. */
+
+        closeEnv();
+    }
+
+    /**
+     * Simulate eviction by logging this node, updating the LSN in its
+     * parent slot, setting the Node to null in the parent slot, and
+     * removing the IN from the INList.  Logging is provisional.  The
+     * parent is dirtied.  May not be called unless this node is dirty and
+     * none of its children are dirty.  Children may be resident.
+     */
+    private void simulateEviction(Environment env,
+                                  EnvironmentImpl envImpl,
+                                  IN nodeToEvict,
+                                  IN parentNode)
+        throws DatabaseException {
+
+        assertTrue("not dirty " + nodeToEvict.getNodeId(),
+                   nodeToEvict.getDirty());
+        assertTrue(!hasDirtyChildren(nodeToEvict));
+        parentNode.latch();
+        long lsn = TestUtils.logIN
+            (env, nodeToEvict, true /*provisional*/, parentNode);
+        int index;
+        for (index = 0;; index += 1) {
+            if (index >= parentNode.getNEntries()) {
+                fail();
+            }
+            if (parentNode.getTarget(index) == nodeToEvict) {
+                break;
+            }
+        }
+        parentNode.updateNode(index, null /*node*/, lsn, null /*lnSlotKey*/);
+        parentNode.releaseLatch();
+        envImpl.getInMemoryINs().remove(nodeToEvict);
+    }
+
+    private boolean hasDirtyChildren(IN parent) {
+        for (int i = 0; i < parent.getNEntries(); i += 1) {
+            Node child = parent.getTarget(i);
+            if (child instanceof IN) {
+                IN in = (IN) child;
+                if (in.getDirty()) {
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+
+    public void testMultiCleaningBug()
+        throws DatabaseException {
+
+        initEnv(true, false);
+
+        final EnvironmentImpl envImpl =
+            DbInternal.envGetEnvironmentImpl(exampleEnv);
+        final FileSelector fileSelector =
+            envImpl.getCleaner().getFileSelector();
+
+        Map<String, Set<String>> expectedMap =
+            new HashMap<String, Set<String>>();
+        doLargePut(expectedMap, 1000, 1, true);
+        modifyData(expectedMap, 1, true);
+        checkData(expectedMap);
+
+        final TestHook hook = new TestHook() {
+            public void doHook() {
+                /* Signal that hook was called. */
+                if (synchronizer != 99) {
+                    synchronizer = 1;
+                }
+                /* Wait for signal to proceed with cleaning. */
+                while (synchronizer != 2 &&
+                       synchronizer != 99 &&
+                       !Thread.interrupted()) {
+                    Thread.yield();
+                }
+            }
+            public Object getHookValue() {
+                throw new UnsupportedOperationException();
+            }
+            public void doIOHook() throws IOException {
+                throw new UnsupportedOperationException();
+            }
+            public void hookSetup() {
+                throw new UnsupportedOperationException();
+            }
+        };
+
+        junitThread = new JUnitThread("TestMultiCleaningBug") {
+            public void testBody()
+                throws DatabaseException {
+
+                try {
+                    while (synchronizer != 99) {
+                        /* Wait for initial state. */
+                        while (synchronizer != 0 &&
+                               synchronizer != 99 &&
+                               !Thread.interrupted()) {
+                            Thread.yield();
+                        }
+                        /* Clean with hook set, hook is called next. */
+                        fileSelector.setFileChosenHook(hook);
+                        exampleEnv.cleanLog();
+                        /* Signal that cleaning is done. */
+                        if (synchronizer != 99) {
+                            synchronizer = 3;
+                        }
+                    }
+                } catch (Throwable e) {
+                    e.printStackTrace();
+                }
+            }
+        };
+
+        /* Kick off thread above. */
+        synchronizer = 0;
+        junitThread.start();
+
+        for (int i = 0; i < 100 && junitThread.isAlive(); i += 1) {
+            /* Wait for hook to be called when a file is chosen. */
+            while (synchronizer != 1 && junitThread.isAlive()) {
+                Thread.yield();
+            }
+            /* Allow the thread to clean the chosen file. */
+            synchronizer = 2;
+            /* But immediately clean here, which could select the same file. */
+            fileSelector.setFileChosenHook(null);
+            exampleEnv.cleanLog();
+            /* Wait for both cleaner runs to finish. */
+            while (synchronizer != 3 && junitThread.isAlive()) {
+                Thread.yield();
+            }
+            /* Make more waste to be cleaned. */
+            modifyData(expectedMap, 1, true);
+            synchronizer = 0;
+        }
+
+        synchronizer = 99;
+
+        try {
+            junitThread.finishTest();
+            junitThread = null;
+        } catch (Throwable e) {
+            e.printStackTrace();
+            fail(e.toString());
+        }
+
+        closeEnv();
+    }
 }
