@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: FileSelectionTest.java,v 1.41 2008/03/27 14:53:28 mark Exp $
+ * $Id: FileSelectionTest.java,v 1.42.2.4 2010/01/04 15:30:42 cwl Exp $
  */
 
 package com.sleepycat.je.cleaner;
@@ -36,8 +36,10 @@ import com.sleepycat.je.EnvironmentStats;
 import com.sleepycat.je.LockMode;
 import com.sleepycat.je.OperationStatus;
 import com.sleepycat.je.Transaction;
+import com.sleepycat.je.cleaner.DbFileSummaryMap;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.dbi.CursorImpl;
+import com.sleepycat.je.dbi.DatabaseImpl;
 import com.sleepycat.je.dbi.EnvironmentImpl;
 import com.sleepycat.je.dbi.MemoryBudget;
 import com.sleepycat.je.junit.JUnitThread;
@@ -88,8 +90,6 @@ public class FileSelectionTest extends TestCase {
                               "false");
         config.setConfigParam
 	    (EnvironmentParams.ENV_RUN_CHECKPOINTER.getName(), "false");
-        config.setConfigParam
-	    (EnvironmentParams.CLEANER_MIN_FILES_TO_DELETE.getName(), "1");
         config.setConfigParam
 	    (EnvironmentParams.CLEANER_LOCK_TIMEOUT.getName(), "1");
         config.setConfigParam
@@ -602,7 +602,8 @@ public class FileSelectionTest extends TestCase {
             int fileCount =
                 envImpl.getFileManager().getAllFileNumbers().length;
             assertTrue("fileCount=" + fileCount +
-                       " maxFileCount=" + maxFileCount,
+                       " maxFileCount=" + maxFileCount +
+                       " iteration=" + i,
                        fileCount <= maxFileCount);
             if (false) {
                 System.out.println("fileCount=" + fileCount +
@@ -818,6 +819,11 @@ public class FileSelectionTest extends TestCase {
      * Tests that when cleaned files are deleted during a compression, the
      * flushing of the local tracker does not transfer tracker information
      * for the deleted files. [#15528]
+     *
+     * This test also checks that tracker information is not transfered to the
+     * MapLN's per-DB utilization information in DbFileSummaryMap.  This was
+     * occuring in JE 3.3.74 and earlier, under the same circumstances as
+     * tested here (IN compression).  [#16610] 
      */
     public void testCompressionBug()
         throws DatabaseException {
@@ -832,10 +838,13 @@ public class FileSelectionTest extends TestCase {
             return;
         }
 
-        /* Disable compressor so we can compress explicitly. */
         EnvironmentConfig envConfig = initConfig();
+        /* Disable compressor so we can compress explicitly. */
         envConfig.setConfigParam
 	    (EnvironmentParams.ENV_RUN_INCOMPRESSOR.getName(), "false");
+        /* Ensure that we check for resurrected file leaks. */
+        envConfig.setConfigParam
+            (EnvironmentParams.ENV_CHECK_LEAKS.getName(), "true");
         openEnv(envConfig);
 
         /* Write and then delete all data. */
@@ -871,7 +880,9 @@ public class FileSelectionTest extends TestCase {
             public void doHook() {
                 synchronizer = 1;
                 /* Wait for log cleaning to complete. */
-                while (synchronizer < 2) Thread.yield();
+                while (synchronizer < 2 && !Thread.interrupted()) {
+                    Thread.yield();
+                }
             }
             public Object getHookValue() {
                 throw new UnsupportedOperationException();
@@ -911,6 +922,89 @@ public class FileSelectionTest extends TestCase {
         closeEnv();
     }
 
+    /**
+     * Checks that DB utilization is repaired when damaged by JE 3.3.74 or
+     * earlier. Go to some trouble to create a DatabaseImpl with the repair
+     * done flag not set, and with a DB file summary for a deleted file.
+     * [#16610] 
+     */
+    public void testDbUtilizationRepair()
+        throws DatabaseException, IOException {
+
+        openEnv();
+        writeData();
+        forceCleanOne();
+        verifyDeletedFiles(new int[] {0});
+
+        DatabaseImpl dbImpl = DbInternal.dbGetDatabaseImpl(db);
+        DbFileSummaryMap summaries = dbImpl.getDbFileSummaries();
+
+        /* New version DB does not need repair. */
+        assertTrue(dbImpl.getUtilizationRepairDone());
+
+        /* Deleted file is absent from summary map. */
+        assertNull(summaries.get(0L /*fileNum*/, true /*adjustMemBudget*/,
+                                 true /*checkResurrected*/,
+                                 envImpl.getFileManager()));
+
+        /*
+         * Force addition of deleted file to summary map by creating a dummy
+         * file to prevent assertions from firing.
+         */
+        File dummyFile = new File(env.getHome(), "00000000.jdb");
+        assertTrue(dummyFile.createNewFile());
+        assertNotNull(summaries.get(0L /*fileNum*/, true /*adjustMemBudget*/,
+                                    false /*checkResurrected*/,
+                                    envImpl.getFileManager()));
+        assertTrue(dummyFile.delete());
+
+        /* Now an entry in the summary map is there for a deleted file.. */
+        assertNotNull(summaries.get(0L /*fileNum*/, true /*adjustMemBudget*/,
+                                    true /*checkResurrected*/,
+                                    envImpl.getFileManager()));
+
+        /* Force the MapLN with the bad entry to be flushed. */
+        dbImpl.setDirtyUtilization();
+        env.checkpoint(forceConfig);
+        closeEnv();
+
+        /* If the DB is temporary, we can't test it further. */
+        if (temporary) {
+            return;
+        }
+
+        /*
+         * When the DB is opened, the repair should not take place, because we
+         * did not clear the repair done flag above.
+         */
+        openEnv();
+        dbImpl = DbInternal.dbGetDatabaseImpl(db);
+        summaries = dbImpl.getDbFileSummaries();
+        assertTrue(dbImpl.getUtilizationRepairDone());
+        assertNotNull(summaries.get(0L /*fileNum*/, true /*adjustMemBudget*/,
+                                    true /*checkResurrected*/,
+                                    envImpl.getFileManager()));
+
+        /* Clear the repair done flag and force the MapLN to be flushed. */
+        dbImpl.clearUtilizationRepairDone();
+        dbImpl.setDirtyUtilization();
+        env.checkpoint(forceConfig);
+        closeEnv();
+
+        /*
+         * Since the repair done flag was cleared above, when the DB is opened,
+         * the repair should take place.
+         */
+        openEnv();
+        dbImpl = DbInternal.dbGetDatabaseImpl(db);
+        summaries = dbImpl.getDbFileSummaries();
+        assertTrue(dbImpl.getUtilizationRepairDone());
+        assertNull(summaries.get(0L /*fileNum*/, true /*adjustMemBudget*/,
+                                 true /*checkResurrected*/,
+                                 envImpl.getFileManager()));
+        closeEnv();
+    }
+
     /**
      * Force cleaning of one file.
      */
