@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: LoggableTest.java,v 1.93 2008/05/13 01:57:02 linda Exp $
+ * $Id: LoggableTest.java,v 1.94.2.2 2010/01/04 15:30:44 cwl Exp $
  */
 
 package com.sleepycat.je.log;
@@ -12,8 +12,6 @@ import java.io.File;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 
-import javax.xml.parsers.DocumentBuilderFactory;
-
 import junit.framework.TestCase;
 
 import com.sleepycat.je.DatabaseConfig;
@@ -59,12 +57,6 @@ public class LoggableTest extends TestCase {
         throws Exception {
 
         envHome = new File(System.getProperty(TestUtils.DEST_DIR));
-
-        /*
-         * Attempt to validate that the db print log output is valid
-         * xml -- isn't quite working
-         */
-        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
     }
 
     public void setUp()
