@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: MakeLogEntryVersionData.java,v 1.19 2008/05/28 15:35:18 linda Exp $
+ * $Id: MakeLogEntryVersionData.java,v 1.19.2.4 2010/01/04 15:30:44 cwl Exp $
  */
 
 package com.sleepycat.je.logversion;
@@ -218,12 +218,14 @@ public class MakeLogEntryVersionData {
         env.close();
 
         /*
-         * Get the set of all log entry types we expect to output.  We punt on
-         * one type -- MapLN_TX -- because MapLN (non-transactional) is now
-         * used instead.
+         * Get the set of all log entry types we expect to output.  We exclude
+         * two types:
+         * - MapLN_TX, because MapLN (non-transactional) is now used instead.
+         * - INDelete, because root compression is no longer used.
          */
         Set<LogEntryType> expectedTypes = LogEntryType.getAllTypes();
         expectedTypes.remove(LogEntryType.LOG_MAPLN_TRANSACTIONAL);
+        expectedTypes.remove(LogEntryType.LOG_IN_DELETE_INFO);
 
         /* Open read-only and write all LogEntryType names to a text file. */
         envConfig.setReadOnly(true);
@@ -234,7 +236,8 @@ public class MakeLogEntryVersionData {
             (DbInternal.envGetEnvironmentImpl(env2), true /* readFullItem */);
         while (reader.readNextEntry()) {
             LogEntryType type = reader.getEntryType();
-            writer.println(type.toString());
+            writer.println(type.toStringNoVersion() + '/' +
+                           reader.getEntryVersion());
             expectedTypes.remove(type);
         }
         writer.close();
