@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2004,2008 Oracle.  All rights reserved.
  *
- * $Id: CheckSplitsTest.java,v 1.16 2008/01/07 14:29:10 cwl Exp $
+ * $Id: CheckSplitsTest.java,v 1.18 2008/06/30 20:54:48 linda Exp $
  */
 package com.sleepycat.je.recovery;
 
@@ -21,6 +21,7 @@ import com.sleepycat.je.DbInternal;
 import com.sleepycat.je.EnvironmentConfig;
 import com.sleepycat.je.OperationStatus;
 import com.sleepycat.je.config.EnvironmentParams;
+import com.sleepycat.je.recovery.stepwise.TestData;
 import com.sleepycat.je.util.TestUtils;
 import com.sleepycat.je.utilint.Tracer;
 
@@ -34,7 +35,6 @@ public class CheckSplitsTest extends CheckBase {
      */
     public void testBasicInsert()
         throws Throwable {
-        boolean foo = TestUtils.runLongTests();
 
         EnvironmentConfig envConfig = TestUtils.initEnvConfig();
         turnOffEnvDaemons(envConfig);
@@ -66,7 +66,7 @@ public class CheckSplitsTest extends CheckBase {
          * log entry. We start the steps before the inserts, so the base
          * expected set is empty.
          */
-        HashSet currentExpected = new HashSet();
+        HashSet<TestData> currentExpected = new HashSet<TestData>();
         stepwiseLoop(DB_NAME, envConfig, dbConfig, currentExpected,  0);
     }
 
@@ -138,7 +138,7 @@ public class CheckSplitsTest extends CheckBase {
          * log entry. We start the steps before the inserts, so the base
          * expected set is empty.
          */
-        HashSet currentExpected = new HashSet();
+        HashSet<TestData> currentExpected = new HashSet<TestData>();
         if (TestUtils.runLongTests()) {
             stepwiseLoop(DB_NAME, envConfig, dbConfig, currentExpected,  0);
         }
@@ -236,7 +236,7 @@ public class CheckSplitsTest extends CheckBase {
          * log entry. We start the steps before the inserts, so the base
          * expected set is empty.
          */
-        HashSet currentExpected = new HashSet();
+        HashSet<TestData> currentExpected = new HashSet<TestData>();
         if (TestUtils.runLongTests()) {
             stepwiseLoop(DB_NAME, envConfig, dbConfig, currentExpected,  0);
         }
@@ -329,7 +329,7 @@ public class CheckSplitsTest extends CheckBase {
          * log entry. We start the steps before the inserts, so the base
          * expected set is empty.
          */
-        HashSet currentExpected = new HashSet();
+        HashSet<TestData> currentExpected = new HashSet<TestData>();
         if (TestUtils.runLongTests()) {
             stepwiseLoop(DB_NAME, envConfig, dbConfig, currentExpected,  0);
         }
