@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2004,2008 Oracle.  All rights reserved.
  *
- * $Id: MultiEnvTest.java,v 1.15 2008/05/20 03:27:37 linda Exp $
+ * $Id: MultiEnvTest.java,v 1.16 2008/06/30 20:54:48 linda Exp $
  */
 
 package com.sleepycat.je.recovery;
@@ -13,17 +13,12 @@ import java.io.IOException;
 
 import junit.framework.TestCase;
 
-import com.sleepycat.je.DatabaseException;
-import com.sleepycat.je.Environment;
-import com.sleepycat.je.EnvironmentConfig;
 import com.sleepycat.je.util.TestUtils;
 
 public class MultiEnvTest extends TestCase {
 
     private File envHome1;
     private File envHome2;
-    private Environment env1;
-    private Environment env2;
 
     public MultiEnvTest() {
         envHome1 = new File(System.getProperty(TestUtils.DEST_DIR));
@@ -55,14 +50,4 @@ public class MultiEnvTest extends TestCase {
              * a static field.
              */
     }
-
-    private Environment openEnv(File envHome)
-        throws DatabaseException {
-
-        /* Create an environment. */
-        EnvironmentConfig envConfig = TestUtils.initEnvConfig();
-        envConfig.setAllowCreate(true);
-        Environment e = new Environment(envHome, envConfig);
-        return e;
-    }
 }
