@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: RecoveryTestBase.java,v 1.117 2008/05/19 17:52:22 linda Exp $
+ * $Id: RecoveryTestBase.java,v 1.118.2.2 2010/01/04 15:30:44 cwl Exp $
  */
 
 package com.sleepycat.je.recovery;
@@ -15,7 +15,6 @@ import java.util.Arrays;
 import java.util.Comparator;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.Hashtable;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -171,8 +170,9 @@ public class RecoveryTestBase extends TestCase {
 
         DatabaseConfig dbConfig = new DatabaseConfig();
 	if (btreeComparisonFunction != null) {
-	    dbConfig.setBtreeComparator((Class<Comparator<byte[]>>)
-                                        btreeComparisonFunction.getClass());
+	    dbConfig.setBtreeComparator
+                ((Class<? extends Comparator<byte[]>>)
+                 btreeComparisonFunction.getClass());
 	}
         dbConfig.setTransactional(true);
         dbConfig.setAllowCreate(true);
@@ -249,7 +249,7 @@ public class RecoveryTestBase extends TestCase {
      * Recover the databases and check the data. Return a list of the
      * RecoveryInfos generated by each recovery.
      */
-    protected List recoverAndVerify(Hashtable expectedData, int numDbs)
+    protected List<RecoveryInfo> recoverAndVerify(Map<TestData, Set<TestData>> expectedData, int numDbs)
         throws DatabaseException {
 
 	return recoverAndVerifyInternal(expectedData, numDbs,
@@ -257,7 +257,7 @@ public class RecoveryTestBase extends TestCase {
                                         false); // readOnly
     }
 
-    protected List recoverROAndVerify(Hashtable expectedData, int numDbs)
+    protected List<RecoveryInfo> recoverROAndVerify(Map<TestData, Set<TestData>> expectedData, int numDbs)
         throws DatabaseException {
 
 	return recoverAndVerifyInternal(expectedData, numDbs,
@@ -269,7 +269,7 @@ public class RecoveryTestBase extends TestCase {
      * Recover the databases and check the data. Return a list of the
      * RecoveryInfos generated by each recovery.
      */
-    protected List xaRecoverAndVerify(Hashtable expectedData, int numDbs)
+    protected List<RecoveryInfo> xaRecoverAndVerify(Map<TestData, Set<TestData>> expectedData, int numDbs)
         throws DatabaseException {
 
 	return recoverAndVerifyInternal(expectedData, numDbs,
@@ -277,13 +277,13 @@ public class RecoveryTestBase extends TestCase {
 					false); // readOnly
     }
 
-    private List recoverAndVerifyInternal(Hashtable expectedData,
+    private List<RecoveryInfo> recoverAndVerifyInternal(Map<TestData, Set<TestData>> expectedData,
 					  int numDbs,
 					  boolean createXAEnv,
                                           boolean readOnlyMode)
         throws DatabaseException {
 
-	List infoList = recoverOnlyInternal(numDbs, createXAEnv, readOnlyMode);
+	List<RecoveryInfo> infoList = recoverOnlyInternal(numDbs, createXAEnv, readOnlyMode);
         verifyData(expectedData, numDbs);
 	TestUtils.validateNodeMemUsage(DbInternal.envGetEnvironmentImpl(env),
 				       false);	
@@ -330,7 +330,7 @@ public class RecoveryTestBase extends TestCase {
 	return recoveryConfig;
     }
 
-    protected List recoverOnly(int numDbs)
+    protected List<RecoveryInfo> recoverOnly(int numDbs)
 	throws DatabaseException {
 
 	return recoverOnlyInternal(numDbs,
@@ -338,7 +338,7 @@ public class RecoveryTestBase extends TestCase {
                                    false);  // read only
     }
 
-    protected List xaRecoverOnly(int numDbs)
+    protected List<RecoveryInfo> xaRecoverOnly(int numDbs)
 	throws DatabaseException {
 
 	return recoverOnlyInternal(numDbs,
@@ -346,12 +346,12 @@ public class RecoveryTestBase extends TestCase {
                                    false); // read only
     }
 
-    private List recoverOnlyInternal(int numDbs,
+    private List<RecoveryInfo> recoverOnlyInternal(int numDbs,
                                      boolean createXAEnv,
                                      boolean readOnlyMode)
 	throws DatabaseException {
 
-        List infoList = new ArrayList();
+        List<RecoveryInfo> infoList = new ArrayList<RecoveryInfo>();
 
         /* Open it again, which will run recovery. */
 	if (createXAEnv) {
@@ -372,7 +372,7 @@ public class RecoveryTestBase extends TestCase {
      * Compare the data in the databases agains the data in the expected data
      * set.
      */
-    protected void verifyData(Hashtable expectedData, int numDbs)
+    protected void verifyData(Map<TestData, Set<TestData>> expectedData, int numDbs)
         throws DatabaseException  {
 
         verifyData(expectedData, true, numDbs);
@@ -382,7 +382,7 @@ public class RecoveryTestBase extends TestCase {
      * Compare the data in the databases against the data in the expected data
      * set.
      */
-    protected void verifyData(Hashtable expectedData,
+    protected void verifyData(Map<TestData, Set<TestData>> expectedData,
                               boolean checkInList,
                               int numDbs)
         throws DatabaseException  {
@@ -390,7 +390,8 @@ public class RecoveryTestBase extends TestCase {
 	verifyData(expectedData, checkInList, 0, numDbs);
     }
 
-    protected void verifyData(Hashtable expectedData,
+    @SuppressWarnings("unchecked")
+	protected void verifyData(Map<TestData, Set<TestData>> expectedData,
                               boolean checkInList,
 			      int startDb,
                               int endDb)
@@ -409,21 +410,27 @@ public class RecoveryTestBase extends TestCase {
          * need to keep the expectedData set intact because we call verify
          * repeatedly.
          */
-        Map useData = new Hashtable();
-        Iterator iter = expectedData.entrySet().iterator();
+        Map<TestData, Set<TestData>> useData = 
+            new HashMap<TestData, Set<TestData>>();
+
+        Iterator<Map.Entry<TestData, Set<TestData>>> iter = 
+            expectedData.entrySet().iterator();
+
         while (iter.hasNext()) {
-            Map.Entry entry = (Map.Entry) iter.next();
-            useData.put(entry.getKey(), ((HashSet) entry.getValue()).clone());
+            Map.Entry<TestData, Set<TestData>> entry = iter.next();
+            Set<TestData> clone = (Set<TestData>)((HashSet<TestData>)entry.getValue()).clone();
+            useData.put(entry.getKey(), clone);
         }
 
         /* Generate an expected count map. */
-        Map countMap = generateCountMap(expectedData);
+        Map<TestData, Integer> countMap = generateCountMap(expectedData);
 
         /* Check each db in turn. */
         DatabaseConfig dbConfig = new DatabaseConfig();
 	if (btreeComparisonFunction != null) {
-	    dbConfig.setBtreeComparator((Class<Comparator<byte[]>>)
-                                        btreeComparisonFunction.getClass());
+	    dbConfig.setBtreeComparator
+                ((Class<? extends Comparator<byte[]>>)
+                 btreeComparisonFunction.getClass());
 	}
         dbConfig.setTransactional(env.getConfig().getTransactional());
         dbConfig.setSortedDuplicates(true);
@@ -471,18 +478,19 @@ public class RecoveryTestBase extends TestCase {
      * Process the expected data map to generate expected counts. For each
      * database, make a map of key value to count.
      */
-    private Map generateCountMap(Map expectedData) {
+    private Map<TestData, Integer> 
+        generateCountMap(Map<TestData, Set<TestData>> expectedData) {
 
-        Map countMap = new HashMap();
+        Map<TestData, Integer> countMap = new HashMap<TestData, Integer>();
 
-        Iterator iter = expectedData.values().iterator();
+        Iterator<Set<TestData>> iter = expectedData.values().iterator();
         while (iter.hasNext()) {
-            Set dataSet = (Set) iter.next();
-            Iterator dataIter = dataSet.iterator();
+            Set<TestData> dataSet = iter.next();
+            Iterator<TestData> dataIter = dataSet.iterator();
             while (dataIter.hasNext()) {
-                TestData t = (TestData) dataIter.next();
+                TestData t = dataIter.next();
                 TestData countKey = new TestData(t.dbNum, t.key);
-                Integer count = (Integer) countMap.get(countKey);
+                Integer count = countMap.get(countKey);
                 if (count == null) {
                     countMap.put(countKey, new Integer(1));
                 } else {
@@ -496,11 +504,10 @@ public class RecoveryTestBase extends TestCase {
     /**
      * @return the expected count value for a given key in a given db.
      */
-    private int getExpectedCount(Map countMap,
+    private int getExpectedCount(Map<TestData, Integer> countMap,
 				 int whichDb,
 				 DatabaseEntry key) {
-        return ((Integer)
-                countMap.get(new TestData(whichDb, key.getData()))).intValue();
+        return countMap.get(new TestData(whichDb, key.getData())).intValue();
     }
 
     /**
@@ -509,7 +516,7 @@ public class RecoveryTestBase extends TestCase {
     protected void insertData(Transaction txn,
                               int startVal,
                               int endVal,
-                              Map expectedData,
+                              Map<TestData, Set<TestData>> expectedData,
                               int nDuplicatesPerKey,
                               boolean addToExpectedData,
                               int numDbs)
@@ -523,7 +530,7 @@ public class RecoveryTestBase extends TestCase {
     protected void insertData(Transaction txn,
                               int startVal,
                               int endVal,
-                              Map expectedData,
+                              Map<TestData, Set<TestData>> expectedData,
                               int nDuplicatesPerKey,
                               boolean addToExpectedData,
                               int startDb,
@@ -543,7 +550,7 @@ public class RecoveryTestBase extends TestCase {
     protected void insertData(Transaction txn,
                               int startVal,
                               int endVal,
-                              Map expectedData,
+                              Map<TestData, Set<TestData>> expectedData,
                               int nDuplicatesPerKey,
                               boolean toggle,
                               boolean addToExpectedData,
@@ -562,7 +569,7 @@ public class RecoveryTestBase extends TestCase {
     protected void insertData(Transaction txn,
                               int startVal,
                               int endVal,
-                              Map expectedData,
+                              Map<TestData, Set<TestData>> expectedData,
                               int nDuplicatesPerKey,
                               boolean toggle,
                               boolean addToExpectedData,
@@ -599,16 +606,16 @@ public class RecoveryTestBase extends TestCase {
      * object that wraps db number and key, and points to sets of TestData
      * objects that wrap db number, key, and data.
      */
-    protected void addExpectedData(Map expectedData,
+    protected void addExpectedData(Map<TestData, Set<TestData>> expectedData,
 				   int dbNum,
 				   DatabaseEntry key,
 				   DatabaseEntry data,
 				   boolean expectCommit) {
         if (expectCommit) {
             TestData keyTestData = new TestData(dbNum, key, null);
-            Set dataSet = (Set) expectedData.get(keyTestData);
+            Set<TestData> dataSet = expectedData.get(keyTestData);
             if (dataSet == null) {
-                dataSet = new HashSet();
+                dataSet = new HashSet<TestData>();
                 expectedData.put(keyTestData, dataSet);
             }
 
@@ -619,14 +626,14 @@ public class RecoveryTestBase extends TestCase {
     /**
      * Remove from the set of expected results.
      */
-    private void removeExpectedData(Map expectedData,
+    private void removeExpectedData(Map<TestData, Set<TestData>> expectedData,
 				    int dbNum,
 				    DatabaseEntry key,
                                     DatabaseEntry data,
 				    boolean expectCommit) {
         if (expectCommit) {
             TestData keyTestData = new TestData(dbNum, key, null);
-            Set dataSet = (Set) expectedData.get(keyTestData);
+            Set<TestData> dataSet = expectedData.get(keyTestData);
             assertTrue("Should be a data set for " + keyTestData,
                        (dataSet != null));
             assertTrue("Should be able to remove key " + key +
@@ -656,7 +663,7 @@ public class RecoveryTestBase extends TestCase {
      */
     private void insertOneRecord(Cursor[] cursors,
                                  int val,
-                                 Map expectedData,
+                                 Map<TestData, Set<TestData>> expectedData,
                                  int nDuplicatesPerKey,
                                  boolean expectCommit)
         throws DatabaseException {
@@ -688,7 +695,7 @@ public class RecoveryTestBase extends TestCase {
     /**
      * Delete either every other or all data.
      */
-    protected void deleteData(Transaction txn, Map expectedData,
+    protected void deleteData(Transaction txn, Map<TestData, Set<TestData>> expectedData,
                               boolean all, boolean expectCommit, int numDbs)
         throws DatabaseException {
 
@@ -732,7 +739,7 @@ public class RecoveryTestBase extends TestCase {
      *         we don't want to do this because we plan to make the txn abort.
      */
     protected void modifyData(Transaction txn, int endVal,
-                              Map expectedData, int increment,
+                              Map<TestData, Set<TestData>> expectedData, int increment,
                               boolean expectCommit, int numDbs)
         throws DatabaseException {
 
@@ -783,8 +790,9 @@ public class RecoveryTestBase extends TestCase {
 
         DatabaseConfig dbConfig = new DatabaseConfig();
 	if (btreeComparisonFunction != null) {
-	    dbConfig.setBtreeComparator((Class<Comparator<byte[]>>)
-                                        btreeComparisonFunction.getClass());
+	    dbConfig.setBtreeComparator
+                ((Class<? extends Comparator<byte[]>>)
+                 btreeComparisonFunction.getClass());
 	}
         dbConfig.setSortedDuplicates(true);
 	dbConfig.setTransactional(true);
@@ -816,15 +824,15 @@ public class RecoveryTestBase extends TestCase {
     /**
      * Print the contents of the expected map for debugging.
      */
-    protected void dumpExpected(Map expectedData)
+    protected void dumpExpected(Map<TestData, Set<TestData>> expectedData)
         throws DatabaseException {
         System.out.println("Expected = " );
-        Iterator iter = expectedData.values().iterator();
+        Iterator<Set<TestData>> iter = expectedData.values().iterator();
         while (iter.hasNext()) {
-            Set dataSet = (Set) iter.next();
-            Iterator dataIter = dataSet.iterator();
+            Set<TestData> dataSet = iter.next();
+            Iterator<TestData> dataIter = dataSet.iterator();
             while (dataIter.hasNext()) {
-                TestData t = (TestData) dataIter.next();
+                TestData t = dataIter.next();
                 System.out.println(t);
             }
         }
