@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: CommitEntry.java,v 1.7 2008/01/07 14:29:11 cwl Exp $
+ * $Id: CommitEntry.java,v 1.8.2.2 2010/01/04 15:30:44 cwl Exp $
  */
 
 package com.sleepycat.je.recovery.stepwise;
@@ -30,27 +30,29 @@ public class CommitEntry extends LogEntryInfo {
         this.txnId = txnId;
     }
 
-    public void updateExpectedSet(Set useExpected,
-                                  Map newUncommittedRecords,
-                                  Map deletedUncommittedRecords) {
+    @Override
+    public void updateExpectedSet
+        (Set<TestData>  useExpected,
+         Map<Long, Set<TestData>> newUncommittedRecords,
+         Map<Long, Set<TestData>> deletedUncommittedRecords) {
 
         Long mapKey = new Long(txnId);
 
         /* Add any new records to the expected set. */
-        Set records = (Set) newUncommittedRecords.get(mapKey);
+        Set<TestData> records = newUncommittedRecords.get(mapKey);
         if (records != null) {
-            Iterator iter = records.iterator();
+            Iterator<TestData> iter = records.iterator();
             while (iter.hasNext()) {
-                useExpected.add((TestData) iter.next());
+                useExpected.add(iter.next());
             }
         }
 
         /* Remove any deleted records from expected set. */
-        records = (Set) deletedUncommittedRecords.get(mapKey);
+        records = deletedUncommittedRecords.get(mapKey);
         if (records != null) {
-            Iterator iter = records.iterator();
+            Iterator<TestData> iter = records.iterator();
             while (iter.hasNext()) {
-                useExpected.remove((TestData) iter.next());
+                useExpected.remove(iter.next());
             }
         }
     }
