@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: TxnalEntry.java,v 1.7 2008/01/07 14:29:11 cwl Exp $
+ * $Id: TxnalEntry.java,v 1.8.2.2 2010/01/04 15:30:44 cwl Exp $
  */
 
 package com.sleepycat.je.recovery.stepwise;
@@ -36,9 +36,12 @@ public class TxnalEntry extends LogEntryInfo {
      * expected set. A txnal delete LN record would delete the record
      * from the expecte set at commit time.
      */
-    public void updateExpectedSet(Set useExpected,
-                                  Map newUncommittedRecords,
-                                  Map deletedUncommittedRecords) {
+    @Override
+    public void updateExpectedSet
+        (Set<TestData> useExpected,
+         Map<Long, Set<TestData>> newUncommittedRecords,
+         Map<Long, Set<TestData>> deletedUncommittedRecords) {
+
         DatabaseEntry keyEntry = new DatabaseEntry();
         DatabaseEntry dataEntry = new DatabaseEntry();
 
@@ -46,9 +49,9 @@ public class TxnalEntry extends LogEntryInfo {
         IntegerBinding.intToEntry(data, dataEntry);
 
         Long mapKey = new Long(txnId);
-        Set records = (Set) newUncommittedRecords.get(mapKey);
+        Set<TestData> records = newUncommittedRecords.get(mapKey);
         if (records == null) {
-            records = new HashSet();
+            records = new HashSet<TestData>();
             newUncommittedRecords.put(mapKey, records);
         }
         records.add(new TestData(keyEntry, dataEntry));
