@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: DeferredWriteTest.java,v 1.17 2008/05/30 14:04:21 mark Exp $
+ * $Id: DeferredWriteTest.java,v 1.18.2.5 2010/02/05 14:22:24 cwl Exp $
  */
 
 package com.sleepycat.je.test;
@@ -547,6 +547,67 @@ public class DeferredWriteTest extends TestCase {
         closeEnv(true /*normalClose*/);
     }
 
+    public void testTempEvictionAndObsoleteCounting()
+        throws DatabaseException {
+
+        EnvironmentConfig envConfig = getEnvConfig(false);
+        envConfig.setConfigParam
+            (EnvironmentParams.NODE_MAX.getName(), "128");
+        envConfig.setConfigParam
+            (EnvironmentParams.NODE_MAX_DUPTREE.getName(), "128");
+        /* Use a small cache to cause eviction. */
+        envConfig.setCacheSize(MemoryBudget.MIN_MAX_MEMORY_SIZE);
+        envConfig.setConfigParam("je.env.runCleaner", "false");
+        envConfig.setConfigParam("je.env.runCheckpointer", "false");
+        envConfig.setConfigParam("je.env.runINCompressor", "false");
+        env = new Environment(envHome, envConfig);
+
+        /* Create DB and insert until 1000 INs are evicted. */
+        Database db = createTempDb(DBNAME);
+        int start;
+        for (start = 1;; start += 1000) {
+            insert(db,
+                   null,          // txn
+                   start,         // start
+                   start + 1000,  // end
+                   new HashSet(), // expected
+                   true);         // useRandom
+            EnvironmentStats stats = env.getStats(null);
+            if (stats.getNNodesExplicitlyEvicted() > 1000) {
+                break;
+            }
+        }
+
+        /*
+         * Update all records twice, to cause eviction and log multiple
+         * versions of the INs.
+         */
+        int lastStart = start;
+        for (start = 1; start <= lastStart; start += 1000) {
+            update(db,
+                   null,          // txn
+                   start,         // start
+                   start + 1000); // end
+        }
+        for (start = 1; start < lastStart; start += 1000) {
+            update(db,
+                   null,          // txn
+                   start,         // start
+                   start + 1000); // end
+        }
+
+        assertTrue(DbInternal.envGetEnvironmentImpl(env).
+                              getUtilizationProfile().
+                              getFileSummaryMap(true).
+                              get(0L).
+                              obsoleteINCount > 1000);
+
+        db.close();
+        closeEnv(true  /*normalClose*/,
+                 true /*expectAccurateObsoleteLNCount*/,
+                 false /*expectAccurateDbUtilization*/);
+    }
+
     private HashSet insertAndCheck(Database db)
         throws DatabaseException {
 
@@ -603,7 +664,7 @@ public class DeferredWriteTest extends TestCase {
                            boolean syncBeforeRecovery,
                            boolean expectEviction)
         throws DatabaseException {
-	
+
         env = new Environment(envHome, envConfig);
         Database db = createDb(true);
         HashSet expected = new HashSet();
@@ -659,7 +720,7 @@ public class DeferredWriteTest extends TestCase {
      */
     public void testDups()
         throws DatabaseException {
-	
+
         EnvironmentConfig envConfig = getEnvConfig(false);
         env = new Environment(envHome, envConfig);
 
@@ -743,7 +804,7 @@ public class DeferredWriteTest extends TestCase {
      */
     public void testCompressAfterSlotReuse()
         throws DatabaseException {
-	
+
         EnvironmentConfig envConfig = getEnvConfig(false);
         /* Disable daemons to prevent async compression. */
         envConfig.setConfigParam("je.env.runCleaner", "false");
@@ -1036,7 +1097,6 @@ public class DeferredWriteTest extends TestCase {
         /* We'll do inserts in two batches. */
         HashSet expectedBatch1 = new HashSet();
         HashSet expectedBatch2 = new HashSet();
-        HashSet finalExpectedSet = null;
 
         int batch1Size = 100;
         int batch2Size = 110;
@@ -1115,6 +1175,7 @@ public class DeferredWriteTest extends TestCase {
                         boolean useRandom)
         throws DatabaseException{
 
+        OperationStatus status;
         DatabaseEntry entry = new DatabaseEntry();
         Random rand = new Random();
         for (int i = start; i < end; i++) {
@@ -1122,11 +1183,12 @@ public class DeferredWriteTest extends TestCase {
 
             IntegerBinding.intToEntry(value, entry);
             if (dups) {
-                assertEquals(OperationStatus.SUCCESS,
-                             db.putNoDupData(txn, MAIN_KEY_FOR_DUPS, entry));
+                status = db.putNoDupData(txn, MAIN_KEY_FOR_DUPS, entry);
             } else {
-                assertEquals(OperationStatus.SUCCESS,
-                             db.putNoOverwrite(txn, entry, entry));
+                status = db.putNoOverwrite(txn, entry, entry);
+            }
+            if (!useRandom) {
+                assertEquals(OperationStatus.SUCCESS, status);
             }
             expected.add(new Integer(value));
         }
@@ -1173,6 +1235,63 @@ public class DeferredWriteTest extends TestCase {
         }
     }
 
+    /**
+     * Update a set of records.
+     */
+    private void update(Database db,
+                        Transaction txn,
+                        int start,
+                        int end)
+        throws DatabaseException{
+
+        OperationStatus status;
+        DatabaseEntry entry = new DatabaseEntry();
+        for (int i = start; i < end; i++) {
+            IntegerBinding.intToEntry(i, entry);
+            if (dups) {
+                status = db.put(txn, MAIN_KEY_FOR_DUPS, entry);
+            } else {
+                status = db.put(txn, entry, entry);
+            }
+            assertEquals(OperationStatus.SUCCESS, status);
+        }
+    }
+
+    /**
+     * Delete a set of records, update the values in the expected set.
+     * @param useRandom If True, use random values.
+     */
+    private void delete(Database db,
+                        Transaction txn,
+                        int start,
+                        int end,
+                        Set expected,
+                        boolean useRandom)
+        throws DatabaseException{
+
+        DatabaseEntry entry = new DatabaseEntry();
+        Random rand = new Random();
+        for (int i = start; i < end; i++) {
+            int value = useRandom ? (start + rand.nextInt(end - start)) : i;
+
+            IntegerBinding.intToEntry(value, entry);
+            if (dups) {
+                final Cursor c = db.openCursor(txn, null);
+                try {
+                    if (c.getSearchBoth(MAIN_KEY_FOR_DUPS, entry, null) ==
+                        OperationStatus.SUCCESS) {
+                        c.delete();
+                    }
+                } finally {
+                    c.close();
+                }
+            } else {
+                db.delete(txn, entry);
+            }
+            expected.remove(new Integer(value));
+        }
+    }
+
     /**
      * The database should hold exactly the values in the expected set.
      */
@@ -1251,4 +1370,164 @@ public class DeferredWriteTest extends TestCase {
             env.checkpoint(force);
         }
     }
+
+    /**
+     * Tests that record deletion is durable after Database.sync, when a crash
+     * ocurs after the sync and the previous version of the LN is in the
+     * recovery interval.  Before logging deferred-write LNs provisionally, the
+     * previous version of the LN was reinserted into the BIN by recovery.
+     *
+     * [#16864]
+     */
+    public void testDelete()
+        throws DatabaseException {
+
+        final EnvironmentConfig envConfig = getEnvConfig(false);
+        env = new Environment(envHome, envConfig);
+        Database db = createDb(true);
+
+        final int NUM_RECORDS = 100;
+        final HashSet expected = new HashSet();
+        insert(db, null, 1, NUM_RECORDS, expected, false);
+        db.sync();
+        delete(db, null, 1, NUM_RECORDS, expected, false);
+        db.sync();
+        assertTrue(expected.isEmpty());
+        checkExactContentMatch(db, expected);
+
+        /* Close without a checkpoint to redo the LNs during recovery. */
+        DbInternal.envGetEnvironmentImpl(env).abnormalClose();
+        env = null;
+
+        /* Recover and check again. */
+        env = new Environment(envHome, envConfig);
+        db = createDb(true);
+        checkExactContentMatch(db, expected);
+        db.close();
+        env.close();
+        env = null;
+    }
+
+    /**
+     * Tests a fix for a LogFileNotFound exception in the following sequence
+     * for a deferred-write database.
+     *
+     * 100 LN-A
+     * 200 BIN-B, parent of LN-A
+     * ... LN-A is deleted, marked dirty and not logged
+     * ... BIN-B is compressed, LN-A is counted obsolete in utilization tracker
+     * 300 BIN-B flushed by eviction (this step is optional)
+     * 400 FileSummaryLN with LN-A obsolete offset is flushed as the result of
+     *     utilization tracker eviction
+     * ... Crash and recover, LN-A is mistakedly inserted into BIN-B by redo
+     *
+     * When the log file containing 100 LN-A is cleaned, it will not be
+     * migrated because it was counted obsolete.  Yet it is referenced by its
+     * BIN parent.  This caused a LogFileNotFound exception later when
+     * attempting to access the LN.
+     *
+     * [#16864]
+     */
+    public void testCleanAfterDelete()
+        throws DatabaseException{
+
+        if (dups) {
+            /* There is no variant of this test for dups. */
+            return;
+        }
+        final int CACHE_SIZE = 4 << 20;
+        final EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+        envConfig.setAllowCreate(true);
+        envConfig.setConfigParam
+            (EnvironmentConfig.MAX_MEMORY, String.valueOf(CACHE_SIZE));
+        envConfig.setConfigParam
+            (EnvironmentConfig.CLEANER_DETAIL_MAX_MEMORY_PERCENTAGE, "1");
+        envConfig.setConfigParam
+            (EnvironmentConfig.CLEANER_EXPUNGE, "false");
+        /* Disable daemons to prevent async compression. */
+        envConfig.setConfigParam
+            (EnvironmentConfig.ENV_RUN_CLEANER, "false");
+        envConfig.setConfigParam
+            (EnvironmentConfig.ENV_RUN_CHECKPOINTER, "false");
+        envConfig.setConfigParam
+            (EnvironmentConfig.ENV_RUN_IN_COMPRESSOR, "false");
+        env = new Environment(envHome, envConfig);
+
+        final DatabaseConfig dbConfig = new DatabaseConfig();
+        dbConfig.setAllowCreate(true);
+        dbConfig.setDeferredWrite(true);
+        Database db = env.openDatabase(null, DBNAME, dbConfig);
+
+        final DatabaseEntry key = new DatabaseEntry();
+        final DatabaseEntry data = new DatabaseEntry(new byte[1000]);
+        final int N_RECORDS = 10000;
+
+        IntegerBinding.intToEntry(0, key);
+        assertSame(OperationStatus.SUCCESS,
+                   db.putNoOverwrite(null, key, data));
+        IntegerBinding.intToEntry(1, key);
+        assertSame(OperationStatus.SUCCESS,
+                   db.putNoOverwrite(null, key, data));
+        db.sync();
+        IntegerBinding.intToEntry(0, key);
+        assertSame(OperationStatus.SUCCESS,
+                   db.delete(null, key));
+        env.compress();
+        db.sync();
+
+        /* Cause enough eviction to flush the FileSummaryLNs. */
+        for (int i = 1; i < N_RECORDS; i += 1) {
+            IntegerBinding.intToEntry(i, key);
+            assertSame(OperationStatus.SUCCESS,
+                       db.put(null, key, data));
+        }
+        db.sync();
+        for (int i = 1; i < N_RECORDS; i += 1) {
+            IntegerBinding.intToEntry(i, key);
+            assertSame(OperationStatus.SUCCESS,
+                       db.put(null, key, data));
+        }
+        db.sync();
+        for (int i = 1; i < N_RECORDS; i += 1) {
+            IntegerBinding.intToEntry(i, key);
+            assertSame(OperationStatus.SUCCESS,
+                       db.put(null, key, data));
+        }
+        db.sync();
+
+        /* Crash and recover. */
+        DbInternal.envGetEnvironmentImpl(env).abnormalClose();
+        db = null;
+        env = null;
+        envConfig.setAllowCreate(false);
+        env = new Environment(envHome, envConfig);
+        dbConfig.setAllowCreate(false);
+        db = env.openDatabase(null, DBNAME, dbConfig);
+
+        /* Create enough waste to cause log file zero to be cleaned. */
+        for (int i = 1; i < N_RECORDS; i += 1) {
+            IntegerBinding.intToEntry(i, key);
+            assertSame(OperationStatus.SUCCESS,
+                       db.put(null, key, data));
+        }
+        db.sync();
+        for (int i = 1; i < N_RECORDS; i += 1) {
+            IntegerBinding.intToEntry(i, key);
+            assertSame(OperationStatus.SUCCESS,
+                       db.delete(null, key));
+        }
+        db.sync();
+        env.cleanLog();
+        env.checkpoint(CHECKPOINT_FORCE_CONFIG);
+        assertTrue(!(new File(envHome, TestUtils.LOG_FILE_NAME)).exists());
+
+        /* Before the fix, a LogFileNotFound exception was thrown here. */
+        IntegerBinding.intToEntry(0, key);
+        assertSame(OperationStatus.NOTFOUND,
+                   db.get(null, key, data, null));
+
+        db.close();
+        env.close();
+        env = null;
+    }
 }
