@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: SecondaryTest.java,v 1.43 2008/01/24 14:59:32 linda Exp $
+ * $Id: SecondaryTest.java,v 1.43.2.3 2010/01/04 15:30:47 cwl Exp $
  */
 
 package com.sleepycat.je.test;
@@ -1200,12 +1200,39 @@ public class SecondaryTest extends MultiKeyTxnTestCase {
             fail();
         } catch (DatabaseException expected) {}
 
+        /* Cursor.dup works whether initialized or not. */
+        {
+            Cursor c2 = secCursor.dup(false);
+            c2.close();
+            c2 = secCursor.dup(true);
+            c2.close();
+            c2 = secCursor.dup(false);
+            c2.close();
+            c2 = secCursor.dup(true);
+            c2.close();
+        }
+
         /* Initialize, then close, then check all operations. */
 
         assertEquals(OperationStatus.SUCCESS,
                      cursor.getSearchKey(key, found, null));
         assertEquals(OperationStatus.SUCCESS,
                      secCursor.getSearchKey(secKey, found, null));
+
+        /* Cursor.dup works whether initialized or not. */
+        {
+            Cursor c2 = cursor.dup(false);
+            c2.close();
+            c2 = cursor.dup(true);
+            c2.close();
+            c2 = secCursor.dup(false);
+            c2.close();
+            c2 = secCursor.dup(true);
+            c2.close();
+        }
+
+        /* Close, then check all operations. */
+
         secCursor.close();
         cursor.close();
 
