@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: BindingTest.java,v 1.33 2008/05/19 17:52:23 linda Exp $
+ * $Id: BindingTest.java,v 1.33.2.3 2010/01/04 15:30:49 cwl Exp $
  */
 
 package com.sleepycat.persist.test;
@@ -1879,7 +1879,8 @@ public class BindingTest extends TestCase {
 
         /* Create key creator/nullifier. */
         SecondaryKeyCreator keyCreator = new PersistKeyCreator
-            (catalog, entityMeta, keyCls.getName(), secKeyMeta);
+            (catalog, entityMeta, keyCls.getName(), secKeyMeta,
+             false /*rawAcess*/);
 
         /* Convert entity to bytes. */
         PersistEntityBinding entityBinding =
@@ -1924,7 +1925,8 @@ public class BindingTest extends TestCase {
 
         /* Create key creator/nullifier. */
         SecondaryMultiKeyCreator keyCreator = new PersistKeyCreator
-            (catalog, entityMeta, keyCls.getName(), secKeyMeta);
+            (catalog, entityMeta, keyCls.getName(), secKeyMeta,
+             false /*rawAcess*/);
 
         /* Convert entity to bytes. */
         PersistEntityBinding entityBinding =
@@ -1968,7 +1970,8 @@ public class BindingTest extends TestCase {
 
         /* Create key creator/nullifier. */
         ForeignMultiKeyNullifier keyNullifier = new PersistKeyCreator
-            (catalog, entityMeta, keyCls.getName(), secKeyMeta);
+            (catalog, entityMeta, keyCls.getName(), secKeyMeta,
+             false /*rawAcess*/);
 
         /* Convert entity to bytes. */
         PersistEntityBinding entityBinding =
@@ -2017,7 +2020,8 @@ public class BindingTest extends TestCase {
 
         /* Create key creator/nullifier. */
         ForeignMultiKeyNullifier keyNullifier = new PersistKeyCreator
-            (catalog, entityMeta, keyCls.getName(), secKeyMeta);
+            (catalog, entityMeta, keyCls.getName(), secKeyMeta,
+             false /*rawAcess*/);
 
         /* Convert entity to bytes. */
         PersistEntityBinding entityBinding =
