@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2000,2008 Oracle.  All rights reserved.
  *
- * $Id: EvolveTest.java,v 1.13 2008/02/05 23:28:28 mark Exp $
+ * $Id: EvolveTest.java,v 1.13.2.1 2009/03/27 17:12:55 mark Exp $
  */
 package com.sleepycat.persist.test;
 
@@ -29,6 +29,9 @@ import com.sleepycat.util.test.SharedTestUtils;
  */
 public class EvolveTest extends EvolveTestBase {
 
+    /* Toggle to use listener every other test case. */
+    private static boolean useEvolveListener;
+
     public static Test suite()
         throws Exception {
 
@@ -141,14 +144,22 @@ public class EvolveTest extends EvolveTestBase {
         }
 
         EvolveConfig config = new EvolveConfig();
-        config.setEvolveListener(new EvolveListener() {
-            public boolean evolveProgress(EvolveEvent event) {
-                EvolveStats stats = event.getStats();
-                evolveNRead = stats.getNRead();
-                evolveNConverted = stats.getNConverted();
-                return true;
-            }
-        });
+
+        /*
+         * Use listener every other time to ensure that the stats are returned
+         * correctly when no listener is configured. [#17024]
+         */
+        useEvolveListener = !useEvolveListener;
+        if (useEvolveListener) {
+            config.setEvolveListener(new EvolveListener() {
+                public boolean evolveProgress(EvolveEvent event) {
+                    EvolveStats stats = event.getStats();
+                    evolveNRead = stats.getNRead();
+                    evolveNConverted = stats.getNConverted();
+                    return true;
+                }
+            });
+        }
 
         openEnv();
 
@@ -166,11 +177,13 @@ public class EvolveTest extends EvolveTestBase {
         if (nExpected > 0) {
             assertTrue(PersistCatalog.unevolvedFormatsEncountered);
         }
-        assertTrue(evolveNRead == nExpected);
-        assertTrue(evolveNConverted == nExpected);
-        assertTrue(evolveNConverted >= evolveNRead);
-        assertEquals(evolveNRead, stats.getNRead());
-        assertEquals(evolveNConverted, stats.getNConverted());
+        assertTrue(stats.getNRead() == nExpected);
+        assertTrue(stats.getNConverted() == nExpected);
+        assertTrue(stats.getNConverted() >= stats.getNRead());
+        if (useEvolveListener) {
+            assertEquals(evolveNRead, stats.getNRead());
+            assertEquals(evolveNConverted, stats.getNConverted());
+        }
 
         /* Evolve again and expect that no entities are converted. */
         evolveNRead = 0;
@@ -178,10 +191,12 @@ public class EvolveTest extends EvolveTestBase {
         PersistCatalog.unevolvedFormatsEncountered = false;
         stats = store.evolve(config);
         assertTrue(!PersistCatalog.unevolvedFormatsEncountered);
-        assertTrue(evolveNRead == 0);
-        assertTrue(evolveNConverted == 0);
         assertEquals(0, stats.getNRead());
         assertEquals(0, stats.getNConverted());
+        if (useEvolveListener) {
+            assertTrue(evolveNRead == 0);
+            assertTrue(evolveNConverted == 0);
+        }
 
         /* Ensure that we can read all entities without evolution. */
         PersistCatalog.unevolvedFormatsEncountered = false;
