@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2000,2008 Oracle.  All rights reserved.
  *
- * $Id: EvolveTestBase.java,v 1.14 2008/06/03 04:52:24 mark Exp $
+ * $Id: EvolveTestBase.java,v 1.15.2.1 2009/03/27 17:12:55 mark Exp $
  */
 package com.sleepycat.persist.test;
 
@@ -185,6 +185,8 @@ public abstract class EvolveTestBase extends TestCase {
         null,
         "AllowFieldTypeChanges",
         null,
+        "ConvertFieldContent_Entity",
+        null,
         "ConvertExample1_Entity",
         null,
         "ConvertExample2_Person",
@@ -215,6 +217,8 @@ public abstract class EvolveTestBase extends TestCase {
         null,
         "AllowChangeKeyMetadata",
         null,
+        "AllowChangeKeyMetadataInSubclass",
+        null,
         "AllowAddSecondary",
         null,
         "FieldAddAndConvert",
