@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: OperationTest.java,v 1.22 2008/05/19 20:33:32 mark Exp $
+ * $Id: OperationTest.java,v 1.23.2.4 2010/01/04 15:30:49 cwl Exp $
  */
 
 package com.sleepycat.persist.test;
@@ -20,6 +20,9 @@ import java.util.Set;
 
 import junit.framework.Test;
 
+/* <!-- begin JE only --> */
+import com.sleepycat.je.CacheMode;
+/* <!-- end JE only --> */
 import com.sleepycat.je.Database;
 import com.sleepycat.je.DatabaseConfig;
 import com.sleepycat.je.DatabaseException;
@@ -33,6 +36,8 @@ import com.sleepycat.persist.StoreConfig;
 import com.sleepycat.persist.impl.Store;
 import com.sleepycat.persist.model.Entity;
 import com.sleepycat.persist.model.KeyField;
+import com.sleepycat.persist.model.NotPersistent;
+import com.sleepycat.persist.model.NotTransient;
 import com.sleepycat.persist.model.Persistent;
 import com.sleepycat.persist.model.PrimaryKey;
 import com.sleepycat.persist.model.SecondaryKey;
@@ -138,6 +143,43 @@ public class OperationTest extends TxnTestCase {
     }
     /* <!-- end JE only --> */
 
+    /* <!-- begin JE only --> */
+    public void testCacheMode()
+        throws DatabaseException {
+
+        open();
+
+        PrimaryIndex<Integer,MyEntity> priIndex =
+            store.getPrimaryIndex(Integer.class, MyEntity.class);
+
+        Transaction txn = txnBeginCursor();
+
+        MyEntity e = new MyEntity();
+        e.priKey = 1;
+        e.secKey = 1;
+        priIndex.put(txn, e);
+
+        EntityCursor<MyEntity> entities = priIndex.entities(txn, null);
+
+        assertSame(CacheMode.DEFAULT, entities.getCacheMode());
+        e = entities.first();
+        assertNotNull(e);
+        assertSame(CacheMode.DEFAULT, entities.getCacheMode());
+        entities.setCacheMode(CacheMode.KEEP_HOT);
+        assertSame(CacheMode.KEEP_HOT, entities.getCacheMode());
+        e = entities.first();
+        assertNotNull(e);
+        assertSame(CacheMode.KEEP_HOT, entities.getCacheMode());
+        entities.setCacheMode(CacheMode.UNCHANGED);
+        entities.update(e);
+        entities.setCacheMode(CacheMode.UNCHANGED);
+
+        entities.close();
+        txnCommit(txn);
+        close();
+    }
+    /* <!-- end JE only --> */
+
     public void testUninitializedCursor()
         throws DatabaseException {
 
@@ -1007,4 +1049,71 @@ public class OperationTest extends TxnTestCase {
         assertNotNull(x);
         close();
     }
+
+    public void testPersistentFields()
+        throws DatabaseException {
+
+        open();
+        PrimaryIndex<Integer, PersistentFields> pri =
+            store.getPrimaryIndex(Integer.class, PersistentFields.class);
+        PersistentFields o1 = new PersistentFields(-1, 1, 2, 3, 4, 5, 6);
+        assertNull(pri.put(o1));
+        PersistentFields o2 = pri.get(-1);
+        assertNotNull(o2);
+        assertEquals(0, o2.transient1);
+        assertEquals(0, o2.transient2);
+        assertEquals(0, o2.transient3);
+        assertEquals(4, o2.persistent1);
+        assertEquals(5, o2.persistent2);
+        assertEquals(6, o2.persistent3);
+        close();
+    }
+
+    @Entity
+    static class PersistentFields {
+
+        @PrimaryKey int key;
+
+        transient int transient1;
+        @NotPersistent int transient2;
+        @NotPersistent transient int transient3;
+
+        int persistent1;
+        @NotTransient int persistent2;
+        @NotTransient transient int persistent3;
+
+        PersistentFields(int k,
+                         int t1,
+                         int t2,
+                         int t3,
+                         int p1,
+                         int p2,
+                         int p3) {
+            key = k;
+            transient1 = t1;
+            transient2 = t2;
+            transient3 = t3;
+            persistent1 = p1;
+            persistent2 = p2;
+            persistent3 = p3;
+        }
+
+        private PersistentFields() {}
+    }
+
+    /**
+     * When a primary or secondary has a persistent key class, the key class
+     * was not recognized as being persistent when getPrimaryConfig,
+     * getSecondaryConfig, or getSubclassIndex was called, if that key class
+     * was not previously referenced.  All three cases are tested by calling
+     * getSecondaryConfig.  This was a bug in JE 3.3.69, reported on OTN.
+     * [#16407]
+     */
+    public void testKeyClassInitialization()
+        throws DatabaseException {
+
+        open();
+        store.getSecondaryConfig(ToManyKeyEntity.class, "key2");
+        close();
+    }
 }
