@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2010 Oracle.  All rights reserved.
  *
- * $Id: SubclassIndexTest.java,v 1.9 2008/02/05 23:28:28 mark Exp $
+ * $Id: SubclassIndexTest.java,v 1.9.2.3 2010/01/04 15:30:49 cwl Exp $
  */
 
 package com.sleepycat.persist.test;
@@ -34,6 +34,7 @@ public class SubclassIndexTest extends TestCase {
 
     private File envHome;
     private Environment env;
+    private EntityStore store;
 
     public void setUp()
         throws IOException {
@@ -48,20 +49,20 @@ public class SubclassIndexTest extends TestCase {
         if (env != null) {
             try {
                 env.close();
-            } catch (DatabaseException e) {
+            } catch (Throwable e) {
                 System.out.println("During tearDown: " + e);
             }
         }
         try {
-            SharedTestUtils.emptyDir(envHome);
-        } catch (Error e) {
+            //SharedTestUtils.emptyDir(envHome);
+        } catch (Throwable e) {
             System.out.println("During tearDown: " + e);
         }
         envHome = null;
         env = null;
     }
 
-    public void testSubclassIndex()
+    private void open()
         throws IOException, DatabaseException {
 
         EnvironmentConfig envConfig = TestEnv.BDB.getConfig();
@@ -70,7 +71,22 @@ public class SubclassIndexTest extends TestCase {
 
         StoreConfig storeConfig = new StoreConfig();
         storeConfig.setAllowCreate(true);
-        EntityStore store = new EntityStore(env, "foo", storeConfig);
+        store = new EntityStore(env, "foo", storeConfig);
+    }
+
+    private void close()
+        throws DatabaseException {
+
+        store.close();
+        store = null;
+        env.close();
+        env = null;
+    }
+
+    public void testSubclassIndex()
+        throws IOException, DatabaseException {
+
+        open();
 
         PrimaryIndex<String, Employee> employeesById =
             store.getPrimaryIndex(String.class, Employee.class);
@@ -133,9 +149,50 @@ public class SubclassIndexTest extends TestCase {
         } catch (IllegalArgumentException expected) {
         }
 
-        store.close();
-        env.close();
-        env = null;
+        close();
+    }
+
+    public void testAddSecKey()
+        throws IOException, DatabaseException {
+
+        open();
+        PrimaryIndex<String, Employee> employeesById =
+            store.getPrimaryIndex(String.class, Employee.class);
+        employeesById.put(new Employee("1"));
+        assertTrue(!hasEntityKey("dept"));
+        close();
+
+        open();
+        employeesById = store.getPrimaryIndex(String.class, Employee.class);
+        assertTrue(!hasEntityKey("dept"));
+        employeesById.put(new Manager("2", "a"));
+        assertTrue(hasEntityKey("dept"));
+        close();
+
+        open();
+        assertTrue(hasEntityKey("dept"));
+        close();
+        
+        open();
+        employeesById = store.getPrimaryIndex(String.class, Employee.class);
+        assertTrue(!hasEntityKey("salary"));
+        employeesById.put(new SalariedManager("3", "a", "111"));
+        assertTrue(hasEntityKey("salary"));
+        close();
+
+        open();
+        assertTrue(hasEntityKey("dept"));
+        assertTrue(hasEntityKey("salary"));
+        close();
+    }
+
+    private boolean hasEntityKey(String keyName) {
+        return store.getModel().
+               getRawType(Employee.class.getName()).
+               getEntityMetadata().
+               getSecondaryKeys().
+               keySet().
+               contains(keyName);
     }
 
     @Entity
@@ -167,4 +224,18 @@ public class SubclassIndexTest extends TestCase {
 
         private Manager() {}
     }
+
+    @Persistent
+    private static class SalariedManager extends Manager {
+
+        @SecondaryKey(relate=MANY_TO_ONE)
+        String salary;
+
+        SalariedManager(String id, String dept, String salary) {
+            super(id, dept);
+            this.salary = salary;
+        }
+
+        private SalariedManager() {}
+    }
 }
