@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2005,2008 Oracle.  All rights reserved.
  *
- * $Id: CleanWithSmallCache.java,v 1.3 2008/01/28 21:17:42 linda Exp $
+ * $Id: CleanWithSmallCache.java,v 1.3.2.1 2009/07/28 20:32:24 mark Exp $
  */
 
 import java.io.File;
@@ -35,7 +35,7 @@ import com.sleepycat.je.StatsConfig;
  *
  * And then to clean that set of logs:
  *
- * java -Xmx10m -cp .:before.jar CleanWithSmallCache \
+ * java -Xmx15m -cp .:before.jar CleanWithSmallCache \
  *  -records 40000 -key 48 -data 10 -h tmp -random -cache 250k \
  *  -seconds 22000 -read 10 -clean
  */
@@ -69,6 +69,12 @@ public class CleanWithSmallCache {
 
     public static void main(String[] args) {
         try {
+            System.out.print("Command line: ");
+            for (String s : args) {
+                System.out.print(s);
+                System.out.print(' ');
+            }
+            System.out.println();
             CleanWithSmallCache test = new CleanWithSmallCache(args);
             long start = System.currentTimeMillis();
             System.out.println("Opening environment");
@@ -78,8 +84,10 @@ public class CleanWithSmallCache {
             test.close();
             long end = System.currentTimeMillis();
             System.out.println("Time: " + ((end - start) / 1000) + " sec");
+            System.exit(0);
         } catch (Throwable e) {
-            e.printStackTrace(System.out);
+            e.printStackTrace();
+            System.exit(1);
         }
     }
 
